package koropeta;
import koropeta.core.*;
import koropeta.gui.*;
import java.util.*;
import java.io.*;
import javax.swing.*;

public class Main {
    public static final int VERSION = 2;

    /* ini関連 */
    private static Properties pref;
    public static Properties ini() {
        return pref;
    }
    protected static void load_ini() {
        pref = new Properties();
        try {
            pref.load(new FileInputStream(new File("koropeta.ini")));
        }
        catch (Exception e) {}
    }
    public static void save_ini() {
        try {
            pref.store(
                new FileOutputStream(new File("koropeta.ini")),
                "This is a preference file generated by Koropeta #"
                +VERSION+". Do not edit by hand.");
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /* 開いているシェル一覧 */
    // koropeta.gui.SurfaceGridFrame
    protected static Vector shell_frames = new Vector();
    public static void add_shell_frame(SurfaceGridFrame sgf) {
        shell_frames.add(sgf);
    }
    public static void remove_shell_frame(SurfaceGridFrame sgf) {
        shell_frames.remove(sgf);

        if (shell_frames.isEmpty()) {
            // シェルウインドウが一枚も無くなったら終了
            exit();
        }
    }
    public static List shell_frames() {
        return Collections.unmodifiableList(shell_frames);
    }

    /* その他 */
    public static void error(String msg) {
        // FIXME: Dialog表示
        System.err.println(msg);
    }

    public static void log(String msg) {
        System.err.println(msg);
    }
    
    public static void exit() {
        save_ini();
        System.exit(0);
    }

    // ファイルの複製
    public static void cp(File src, File dst) throws IOException {
        if (dst.exists()) {
            dst.delete();
        }
	
        FileInputStream is = new FileInputStream(src);
        FileOutputStream os = new FileOutputStream(dst);

        byte[] buf = new byte[1024];
        int n_read;
        while ((n_read = is.read(buf)) != -1) {
            os.write(buf, 0, n_read);
        }

        is.close();
        os.close();
    }

    // ファイルまたはディレクトリの再歸的な削除
    public static void rm_rf(File f) throws IOException {
        if (f.isDirectory()) {
            File[] files = f.listFiles();
            for (int i = 0; i < files.length; i++) {
                rm_rf(files[i]);
            }
        }
        f.delete();
    }

    // 文字列化 & 連結
    public static String join(String delim, List list) {
        if (list == null) {
            return "";
        }
	
        StringBuffer buf = new StringBuffer();
        boolean is_first = true;
	
        for (Iterator ite = list.iterator(); ite.hasNext(); ) {
            if (is_first) {
                is_first = false;
            }
            else {
                buf.append(delim);
            }
	    
            buf.append(ite.next().toString());
        }

        return buf.toString();
    }

    public static Locale get_locale() {
        String str = Main.ini().getProperty("koropeta.gui.locale");
        if (str == null || str.length() == 0) {
            return Locale.getDefault();
        }

        // この部分、何故Localeクラスがやってくれないのか。
        String[] parts = str.split("_");
        if (parts.length >= 3) {
            return new Locale(parts[0], parts[1], parts[2]);
        }
        else if (parts.length >= 2) {
            return new Locale(parts[0], parts[1]);
        }
        else {
            return new Locale(parts[0]);
        }
    }
    
    public static void main(String[] args) {
        try {
            // まず koropeta.ini を読む
            load_ini();

            // Look & Feelを設定
            try{
                UIManager.setLookAndFeel(
                    pref.getProperty(
                        "koropeta.gui.lookAndFeel",
                        "javax.swing.plaf.metal.MetalLookAndFeel"));
                JFrame.setDefaultLookAndFeelDecorated(true);
                JDialog.setDefaultLookAndFeelDecorated(true);
            } catch(Exception ex) {
                ex.printStackTrace();
            }

            // GTを初期化
            GT.init(get_locale());

            if (args.length < 1) {
                // 空のシェルフレームを作る
                add_shell_frame(new SurfaceGridFrame());
            }
            else {
                File shelldir = new File(args[0]);
                if (shelldir.isDirectory()) {
                    add_shell_frame(new SurfaceGridFrame(shelldir));
                }
                else {
                    add_shell_frame(new SurfaceGridFrame());
                }
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
}