package koropeta.core;
import koropeta.*;
import java.awt.image.*;
import java.awt.color.*;
import javax.imageio.*;
import java.io.*;

public class AlphaSplittedImage {
    public static BufferedImage read(File dir, String basename) throws IOException {
	assert dir != null;
	assert basename != null;
	
	return AlphaSplittedImageReader.read(dir, basename);
    }

    static void write(File dir, String basename, BufferedImage image) throws IOException {
	assert dir != null;
	assert basename != null;
	assert image != null;
	
	AlphaSplittedImageWriter.write(dir, basename, image);
    }

    public static String stringify_int_array(int[] array) {
	StringBuffer buf = new StringBuffer();
	for (int i = 0; i < array.length; i++) {
	    if (buf.length() > 0) {
		buf.append(", ");
	    }
	    buf.append(Integer.toString(array[i]));
	}
	return buf.toString();
    }

    public static String describe_color_model(ColorModel cmodel) {
	String basic_info = "Class: "+cmodel.getClass().getName()+"\n"+
	    "|-- ColorSpace Type: "+cmodel.getColorSpace().getType()+"\n"+
	    "|-- NumComponents  : "+cmodel.getNumComponents()+"\n"+
	    "|-- TransferType   : "+cmodel.getTransferType()+"\n"+
	    "|-- PixelSize      : "+cmodel.getPixelSize()+"\n"+
	    "|-- hasAlpha       : "+(cmodel.hasAlpha()?"yes":"no")+"\n"+
	    "`-- isAlphaPremultiplied: "+(cmodel.isAlphaPremultiplied()?"yes":"no");
	return basic_info;
    }

    public static String describe_sample_model(SampleModel smodel) {
	String basic_info = "Class: "+smodel.getClass().getName()+"\n"+
	    "|-- DataType  : "+smodel.getDataType()+"\n"+
	    "|-- Width     : "+smodel.getWidth()+"  Height: "+smodel.getHeight()+"\n"+
	    "|-- NumBands  : "+smodel.getNumBands()+"\n"+
	    "`-- SampleSize: "+stringify_int_array(smodel.getSampleSize());

	if (smodel instanceof PixelInterleavedSampleModel) {
	    // このサンプルモデルではバンクは常に1つ。
	    PixelInterleavedSampleModel pi = (PixelInterleavedSampleModel)smodel;
	    return basic_info+"\n"+
		" `+-- PixelStride   : "+pi.getPixelStride()+"\n"+
		"  |-- ScanlineStride: "+pi.getScanlineStride()+"\n"+
		"  `-- BandOffsets   : "+stringify_int_array(pi.getBandOffsets());
	}
	else {
	    return basic_info;
	}
    }

    public static boolean need_pna(BufferedImage image) {
	// TYPE_INT_ARGBで作られてゐなければならない
	final int[] data = ((DataBufferInt)(image.getRaster().getDataBuffer())).getData();

	for (int i = 0; i < data.length; i++) {
	    final int alpha = (data[i] >> 24) & 0xff;

	    if (alpha != 0xff && alpha != 0x00) {
		return true;
	    }
	}

	return false;
    }
}