package koropeta.core;
import java.lang.ref.*;
import javax.imageio.*;
import java.io.*;
import java.util.*;
import java.awt.image.*;

public class AlphaSplittedImageCache {
    Map cache; // ImagePath => CachedImage

    public AlphaSplittedImageCache() {
	cache = new Hashtable();
    }

    public BufferedImage cache(File dir, String basename) {
	ImagePath path = new ImagePath(dir, basename);
	CachedImage cached = (CachedImage)cache.get(path);

	if (cached != null && !cached.is_expired()) {
	    return cached.image();
	}
	
	return null;
    }

    public BufferedImage cache(File dir, String basename, BufferedImage img) {
	ImagePath path = new ImagePath(dir, basename);
	CachedImage cached = new CachedImage(path, img);

	cache.put(path, cached);
	return img;
    }

    public BufferedImage raw_read(File file) {
	BufferedImage img;
	try {
	    img = ImageIO.read(file);
	}
	catch (IOException e) {
	    throw new RuntimeException(
		"An exception occured while reading an image: "+
		"the image file "+file+" can be corrupted, or "+
		"your implementation of ImageIO has a bug.");
	}
	return img;
    }

    private class ImagePath {
	File dir;
	String basename;

	File pngfile;
	File pnafile;
	
	long png_lastmod;
	long pna_lastmod;

	public ImagePath(File dir, String basename) {
	    this.dir = dir;
	    this.basename = basename;
	    
	    this.pngfile = new File(dir, basename+".png");
	    this.pnafile = new File(dir, basename+".pna");
	    this.png_lastmod = pngfile.lastModified();
	    this.pna_lastmod = pnafile.isFile() ? pnafile.lastModified() : 0;
	}

	public boolean is_obsoleted() {
	    return
		pngfile.lastModified() > png_lastmod ||
		(pnafile.isFile() && pnafile.lastModified() > pna_lastmod);
	}

	public int hashCode() {
	    return dir.hashCode() + basename.hashCode();
	}

	public boolean equals(Object obj) {
	    if (obj instanceof ImagePath) {
		ImagePath path = (ImagePath)obj;
		return dir.equals(path.dir) && basename.equals(path.basename);
	    }
	    else {
		return false;
	    }
	}

	public String toString() {
	    return this.getClass().getName()+": dir=["+dir+"]; basename=["+basename+"]";
	}
    }

    private class CachedImage {
	ImagePath path;
	SoftReference image; // BufferedImage

	public CachedImage(ImagePath path, BufferedImage image) {
	    this.path = path;
	    this.image = new SoftReference(image);
	}

	public boolean is_collected() {
	    return image.get() == null;
	}

	public boolean is_expired() {
	    return is_collected() || path.is_obsoleted();
	}

	public BufferedImage image() {
	    return (BufferedImage)image.get();
	}

	public String toString() {
	    return this.getClass().getName()+": path=["+path+"]; image=["+image+"]";
	}
    }
}