package koropeta.core;
import koropeta.*;
import java.awt.image.*;
import java.awt.color.*;
import javax.imageio.*;
import java.io.*;
import java.text.*;

public class AlphaSplittedImageWriter extends AlphaSplittedImage {
    public static void write(File dir, String basename, BufferedImage image) throws IOException {
	// 0xff, 0x00以外のアルファ値が存在するならpnaが必要。
	File pngfile = new File(dir, basename+".png");
	File pnafile = new File(dir, basename+".pna");

	Main.log("Attempt to write image: "+pngfile.getAbsolutePath());
	long t = System.currentTimeMillis();

	write(pngfile, extract_rgb(image));
	if (need_pna(image)) {
	    write(pnafile, extract_alpha(image));
	}
	else {
	    if (pnafile.exists()) {
		pnafile.delete();
	    }
	}

	Main.log("... took "+(System.currentTimeMillis() - t)+" milliseconds.");
    }

    protected static BufferedImage extract_rgb(BufferedImage src) {
	// 畫像内に存在しない色を探し、それを透明色とする。
	// imageはTYPE_INT_ARGBで作られたBufferedImageでなければならない。
	BufferedImage dst = new BufferedImage(
	    src.getWidth(), src.getHeight(), BufferedImage.TYPE_INT_RGB);

	int[] src_data = ((DataBufferInt)(src.getRaster().getDataBuffer())).getData();
	int[] dst_data = ((DataBufferInt)(dst.getRaster().getDataBuffer())).getData();

	int transparent_rgb = 0x000000;
	for (; transparent_rgb <= 0xffffff; transparent_rgb++) {
	    Main.log(
		MessageFormat.format(
		    "scan: R={0} G={1} B={2}",
		    new Object[] {
			Integer.toString((transparent_rgb >> 16) & 0xff),
			Integer.toString((transparent_rgb >>  8) & 0xff),
			Integer.toString( transparent_rgb        & 0xff),
		    }));
	    
	    boolean found = false;
	    
	    for (int i = 0; i < src_data.length; i++) {
		final int alpha = (src_data[i] >> 24) & 0xff;
		final int rgb = src_data[i] & 0xffffff;

		if (alpha == 0x00) {
		    // このピクセルは透明なのでスキップ
		}
		else if (rgb == transparent_rgb) {
		    // この色は存在する…
		    found = true;
		    break;
		}
	    }

	    if (!found) {
		Main.log("  is not used.");
		break;
	    }
	}

	for (int i = 0; i < src_data.length; i++) {
	    final int alpha = (src_data[i] >> 24) & 0xff;

	    if (alpha == 0x00) {
		dst_data[i] = transparent_rgb;
	    }
	    else {
		final int rgb = src_data[i] & 0xffffff;

		dst_data[i] = rgb;
	    }
	}

	return dst;
    }

    static BufferedImage extract_alpha(BufferedImage src) {
	BufferedImage dst = new BufferedImage(
	    src.getWidth(), src.getHeight(), BufferedImage.TYPE_BYTE_GRAY);

	int[] src_data = ((DataBufferInt)src.getRaster().getDataBuffer()).getData();
	byte[] dst_data = ((DataBufferByte)dst.getRaster().getDataBuffer()).getData();

	for (int i = 0; i < src_data.length; i++) {
	    final int alpha = (src_data[i] >> 24) & 0xff;

	    dst_data[i] = (byte)alpha;
	}

	return dst;
    }

    protected static void write(File outfile, BufferedImage image) throws IOException {
	// FIXME: 最適化
	// 24bit RGB => 8bit index等
	
	ImageIO.write(image, "png", outfile);
    }
}