package koropeta.core;
import koropeta.*;
import koropeta.core.*;
import java.util.*;
import java.util.regex.*;
import java.awt.*;
import java.awt.geom.*;
import javax.swing.event.*;

public class Collisions implements Cloneable {
    Vector list; // <Collision>
    boolean is_dirty;

    Vector list_data_listeners; // <ListDataListener>

    public Collisions() {
        list = new Vector();
        is_dirty = false;
        list_data_listeners = new Vector();
    }

    public Object clone() {
        Collisions clo = new Collisions();

        for (Iterator ite = list.iterator(); ite.hasNext(); ) {
            Collision c = (Collision)ite.next();
            clo.list.add(c.clone());
        }
	
        clo.is_dirty = is_dirty;
        return clo;
    }

    public void add(Collision col) {
        insert(list.size(), col);
    }

    public void add(Collision[] colls) {
        insert(list.size(), colls);
    }

    static final Pattern re_collision_key =
        Pattern.compile("^collision(\\d+)$");
    static final Pattern re_collision_value =
        Pattern.compile("^(-?\\d+),(-?\\d+),(-?\\d+),(-?\\d+),(.+)$");
    
    public void load(Map desc) {
        // desc: surfaces.txt内、該當ブロックの内容。
        Matcher m;
        for (Iterator ite_key = desc.keySet().iterator(); ite_key.hasNext(); ) {
            String key = (String)ite_key.next();
	    
            if ((m = re_collision_key.matcher(key)).find()) {
                String value = (String)desc.get(key);

                if ((m = re_collision_value.matcher(value)).find()) {
                    try {
                        int left = Integer.parseInt(m.group(1));
                        int top = Integer.parseInt(m.group(2));
                        int width = Integer.parseInt(m.group(3)) - left;
                        int height = Integer.parseInt(m.group(4)) - top;
			
                        add(new Collision(
                                new Rectangle(left, top, width, height),
                                m.group(5)));
                    }
                    catch (Exception e) {
                        throw new RuntimeException(e);
                    }
                }
                else {
                    Main.error(
                        "Invalid collision: "+key+","+value);
                }
            }
        }
    }

    public void save(Map desc) {
        Matcher m;

        if (!is_dirty) {
            return;
        }
	
        // 古い情報があれば消す。
        for (Iterator ite_key = desc.keySet().iterator(); ite_key.hasNext(); ) {
            String key = (String)ite_key.next();

            if ((m = re_collision_key.matcher(key)).find()) {
                ite_key.remove();
            }
        }

        // 保存
        for (int i = 0; i < size(); i++) {
            Collision c = get(i);

            StringBuffer sb = new StringBuffer();
            sb.append(c.x()); sb.append(',');
            sb.append(c.y()); sb.append(',');
            sb.append(c.x() + c.width()); sb.append(',');
            sb.append(c.y() + c.height()); sb.append(',');
            sb.append(c.id());
	    
            desc.put("collision"+i, sb.toString());
        }
	
        is_dirty = false;
    }

    public final boolean dirty() {
        return is_dirty;
    }
    public final boolean dirty(boolean bool) {
        is_dirty = bool;
        return bool;
    }

    public void insert(int idx, Collision col) {
        list.add(idx, col);

        changed(ListDataEvent.INTERVAL_ADDED, idx);
    }

    public void insert(int idx, Collision[] colls) {
        synchronized (colls) {
            if (colls.length > 0) {
                
                for (int i = colls.length - 1; i >= 0; i--) {
                    list.add(idx, colls[i]);
                }

                changed(ListDataEvent.INTERVAL_ADDED, idx, idx + colls.length - 1);
            }
        }
    }

    public void remove(int idx) {
        list.remove(idx);

        changed(ListDataEvent.INTERVAL_REMOVED, idx);
    }

    public void remove(Collision[] colls) {
        synchronized (colls) {
            for (int i = 0; i < colls.length; i++) {
                int idx = list.indexOf(colls[i]);

                if (idx != -1) {
                    remove(idx);
                }
            }
        }
    }

    public void clear() {
        int old_size = list.size();
        if (old_size > 0) {
            list.clear();

            changed(ListDataEvent.INTERVAL_REMOVED, 0, old_size-1);
        }
    }

    public Iterator iterator() {
        return list.iterator();
    }

    public final int size() {
        return list.size();
    }

    public final Collision get(int index) {
        return (Collision)list.get(index);
    }

    public final int indexOf(Collision coll) {
        return list.indexOf(coll);
    }

    public void swap(int id0, int id1) {
        Collision coll = (Collision)list.get(id0);
        list.set(id0, list.get(id1));
        list.set(id1, coll);

        if (id0 <= id1) {
            changed(ListDataEvent.CONTENTS_CHANGED, id0, id1);
        }
        else {
            changed(ListDataEvent.CONTENTS_CHANGED, id1, id0);
        }
    }

    public void addListDataListener(ListDataListener listener) {
        if (!list_data_listeners.contains(listener)) {
            list_data_listeners.add(listener);
        }
    }

    public void removeListDataListener(ListDataListener listener) {
        list_data_listeners.remove(listener);
    }

    public void changed(Collision coll) {
        final int idx = list.indexOf(coll);
        if (idx != -1) {
            changed(ListDataEvent.CONTENTS_CHANGED, idx);
        }
    }
        
    protected void changed(int type, int index) {
        changed(type, index, index);
    }
    protected void changed(int type, int index0, int index1) {
        ListDataEvent ev = new ListDataEvent(
            this, type, index0, index1);
	
        for (Iterator ite = list_data_listeners.iterator(); ite.hasNext();) {
            ListDataListener listener = (ListDataListener)ite.next();
	    
            if (type == ListDataEvent.INTERVAL_ADDED) {
                listener.intervalAdded(ev);
            }
            else if (type == ListDataEvent.INTERVAL_REMOVED) {
                listener.intervalRemoved(ev);
            }
            else {
                listener.contentsChanged(ev);
            }
        }
    }   
}