package koropeta.core;
import java.util.*;
import java.util.regex.*;
import java.io.*;
import java.nio.charset.*;

public class Description extends TreeMap {
    static final Pattern re_comment = Pattern.compile("^//");
    static final Pattern re_pair = Pattern.compile("^(.+?)\\s*,\\s*(.+)$");
    
    String charset = "MS932";
    boolean is_dirty = false;
    
    public Description() {
	this(null);
    }

    public Description(File file) {
	if (file == null) {
	    return;
	}
	
	// 最初の行を讀んでみて、それがcharset指定だったら讀み直す。
	try {
	    InputStream is = new FileInputStream(file);
	    is.mark(1024);
	    
	    BufferedReader reader = new BufferedReader(
		new InputStreamReader(is, charset));
	    Matcher m;

	    if ((m = re_pair.matcher(reader.readLine().trim())).find()) {
		if (m.group(1).equals("charset")) {
		    String new_charset = m.group(2);

		    // CP932 / Shift_JIS / SJISを指定されたらMS932を使ふ。
		    if (new_charset.equals("Shift_JIS") ||
			new_charset.equals("SJIS") ||
			new_charset.equals("CP932")) {
			
			new_charset = "MS932";
		    }

		    if (!charset.equals(new_charset)) {
			charset = new_charset;
			is.reset();

			reader = new BufferedReader(
			    new InputStreamReader(is, charset));
		    }
		}
		else {
		    put(m.group(1), m.group(2));
		}
	    }

	    while (true) {
		String line = reader.readLine();
		if (line == null) {
		    break;
		}
		else {
		    line = line.trim();
		}
		
		if ((m = re_comment.matcher(line)).find()) {
		    continue;
		}
		else if ((m = re_pair.matcher(line)).find()) {
		    put(m.group(1), m.group(2));
		}
	    }
	}
	catch (FileNotFoundException e) {
	    // 無視
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

    protected boolean cp932_encodable_p() throws Exception {
	/*
	  現在の内容を完全にCP932にエンコード出來る場合は眞を返す。
	  このメソッドが僞を返す場合、ファイルへの保存はUTF-8で
	  行はれなければならない。
	*/
	Charset cset = Charset.forName("MS932");
	CharsetEncoder enc = cset.newEncoder();
	
	for (Iterator ite_key = keySet().iterator(); ite_key.hasNext(); ) {
	    String key = (String)ite_key.next();

	    if (!enc.canEncode(key)) {
		return false;
	    }

	    String value = (String)get(key);
	    if (!enc.canEncode(value)) {
		return false;
	    }
	}

	return true;
    }

    public void save(File f) {
	try {
	    save(new FileOutputStream(f));
	}
	catch (Exception e) {
	    throw new RuntimeException(e);
	}
    }
    public void save(OutputStream os) throws Exception {
	charset = cp932_encodable_p() ? "MS932" : "UTF-8";
	String crlf = new String(new char[] {0x0d, 0x0a});

	BufferedWriter writer = new BufferedWriter(
	    new OutputStreamWriter(os, charset));
	writer.write("charset,");
	writer.write(charset.equals("MS932") ? "Shift_JIS" : charset);
	writer.write(crlf);
	writer.write(crlf);

	for (Iterator ite_key = keySet().iterator(); ite_key.hasNext(); ) {
	    String key = (String)ite_key.next();
	    String value = (String)get(key);

	    if (key.equals("charset")) {
		continue;
	    }

	    writer.write(key);
	    writer.write(',');
	    writer.write((String)value);
	    writer.write(crlf);
	}

	writer.close();
    }

    public final boolean dirty() {
	return is_dirty;
    }
    public final boolean dirty(boolean b) {
	is_dirty = b;
	return b;
    }
}