package koropeta.core;
import java.awt.image.*;
import java.awt.color.*;

public class ImageScaler {
    public static void scale(
	int offset_x, int offset_y,
	double scale,
	BufferedImage src_img, BufferedImage dst_img) {

	if (scale > 1.0) {
	    throw new RuntimeException(
		"Scaling up is not supported: "+scale);
	}
	else if (offset_x < 0 || offset_y < 0) {
	    throw new RuntimeException(
		"Moving up or left are not supported: "+offset_x+", "+offset_y);
	}

	DataBuffer src_dbuf = src_img.getRaster().getDataBuffer();
	DataBuffer dst_dbuf = dst_img.getRaster().getDataBuffer();
	
	int[] src = ((DataBufferInt)src_dbuf).getData();
	int[] dst = ((DataBufferInt)dst_dbuf).getData();

	int src_width = src_img.getWidth();
	int src_height = src_img.getHeight();
	
	int dst_width = dst_img.getWidth();
	int dst_height = dst_img.getHeight();

	double inv_scale = 1.0 / scale;

	int dst_y_max = dst_height - offset_y;
	if (dst_y_max * inv_scale > src_height) {
	    dst_y_max = (int)(src_height * scale);
	}
	for (int dst_y = 0; dst_y < dst_y_max; dst_y++) {
	    int src_y_min = (int)Math.floor(dst_y * inv_scale);
	    int src_y_max = (int)Math.ceil((dst_y + 1) * inv_scale) - 1;
	    if (src_y_max >= src_height) {
		src_y_max = src_height - 1;
	    }
	    if (src_y_min > src_y_max) {
		src_y_min = src_y_max;
	    }

	    int dst_x_max = dst_width - offset_x;
	    if (dst_x_max * inv_scale > src_height) {
		dst_x_max = (int)(src_width * scale);
	    }
	    for (int dst_x = 0; dst_x < dst_x_max; dst_x++) {
		int src_x_min = (int)Math.floor(dst_x * inv_scale);
		int src_x_max = (int)Math.ceil((dst_x + 1) * inv_scale) - 1;
		if (src_x_max >= src_width) {
		    src_x_max = src_width - 1;
		}
		if (src_x_min > src_x_max) {
		    src_x_min = src_x_max;
		}

		int a = 0, r = 0, g = 0, b = 0;
		for (int y = src_y_min; y <= src_y_max; y++) {
		    for (int x = src_x_min; x <= src_x_max; x++) {
			int argb = src[y * src_width + x];
			a += (argb >> 24) & 0xff;
			r += (argb >> 16) & 0xff;
			g += (argb >>  8) & 0xff;
			b +=  argb        & 0xff;
		    }
		}
		int n_pixels = (src_y_max - src_y_min + 1) * (src_x_max - src_x_min + 1);
		a /= n_pixels;
		r /= n_pixels;
		g /= n_pixels;
		b /= n_pixels;

		dst[(dst_y + offset_y) * dst_width + (dst_x + offset_x)] =
		    a << 24 |
		    r << 16 |
		    g <<  8 |
		    b;
	    }
	}
    }
}