package koropeta.core;
import java.io.*;
import java.nio.*;
import java.nio.charset.*;
import java.util.regex.*;

public class PlainText {
    String text;
    boolean is_dirty = false;

    public PlainText() {
	this(null);
    }

    public PlainText(File f) {
	text = "";
	
	if (f == null) {
	    return;
	}
	
	try {
	    read(new FileInputStream(f));
	}
	catch (FileNotFoundException e) {
	    // 無視
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

    static final String CRLF = new String(new char[] {0x0d, 0x0a});
    static final String LF = new String(new char[] {0x0a});
    static final Pattern re_newline =
	Pattern.compile("\\"+"u000d\\"+"u000a?|\\"+"u000a"); // xgettext対策
    public String text() {
	Matcher m = re_newline.matcher(text);
	return m.replaceAll(LF);
    }
    public String text(String str) {
	// 内部表現は常にCRLF
	Matcher m = re_newline.matcher(str);
	text = m.replaceAll(CRLF);
	return text;
    }

    public String toString() {
	return text;
    }

    protected void read(InputStream is) throws Exception {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	byte[] readbuf = new byte[1024];
	while (true) {
	    int n_read = is.read(readbuf);
	    if (n_read == -1) {
		break;
	    }
	    else {
		baos.write(readbuf, 0, n_read);
	    }
	}
	
	byte[] bytes = baos.toByteArray();
	String str = decode_as_cp932(bytes);
	if (str == null) {
	    str = decode_as_utf8(bytes);
	}
	text(str);
    }

    public void save(File f) {
	try {
	    save(new FileOutputStream(f));
	}
	catch (Exception e) {
	    throw new RuntimeException(e);
	}
    }
    public void save(OutputStream os) throws Exception {
	String charset = cp932_encodable_p() ? "MS932" : "UTF-8";

	BufferedWriter writer = new BufferedWriter(
	    new OutputStreamWriter(os, charset));
	writer.write(text);
	writer.close();
    }

    protected boolean cp932_encodable_p() throws Exception {
	/*
	  現在の内容を完全にCP932にエンコード出來る場合は眞を返す。
	  このメソッドが僞を返す場合、ファイルへの保存はUTF-8で
	  行はれなければならない。
	*/
	Charset cset = Charset.forName("MS932");
	CharsetEncoder enc = cset.newEncoder();

	return enc.canEncode(text);
    }

    protected String decode_as_cp932(byte[] buffer) throws Exception {
	/*
	  指定されたバイト列の内容を完全にCP932でデコード出來る場合は
	  デコード結果の文字列を返す。さうでない場合はnullを返す。
	*/
	Charset cset = Charset.forName("MS932");
	CharsetDecoder dec = cset.newDecoder();
	dec.onMalformedInput(CodingErrorAction.REPORT);
	dec.onUnmappableCharacter(CodingErrorAction.REPORT);

	try {
	    CharBuffer chars = dec.decode(ByteBuffer.wrap(buffer));
	    chars.position(0);
	    return chars.toString();
	}
	catch (Exception e) {
	    return null;
	}
    }

    protected String decode_as_utf8(byte[] buffer) throws Exception {
	return new String(buffer, "UTF-8");
    }

    public final boolean dirty() {
	return is_dirty;
    }
    public final boolean dirty(boolean b) {
	is_dirty = b;
	return b;
    }
}