package koropeta.core;
import koropeta.*;
import koropeta.core.animation.*;
import java.io.*;
import java.util.*;
import java.util.regex.*;
import org.w3c.dom.*;
import java.awt.image.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;

public class Surface implements Cloneable {
    static final Pattern re_element_key = Pattern.compile(
	"^element(\\d+)$");
    static final Pattern re_element_value = Pattern.compile(
	"^(.+?),(.+?)(?:,(-?\\d+),(-?\\d+))?$");
    static final Pattern re_png_file = Pattern.compile(
	"^(.+?)\\.png$", Pattern.CASE_INSENSITIVE);
    
    String id; // "1"であったり"0001"であったり
    int int_id;

    File dir;
    String basename;

    LayeredImage base_surface;
    LayeredImage elements;
    LayeredImage organized; // 基本畫像もしくはElement畫像を持つ。

    Collisions collisions;
    Animation animation;
    
    public Surface(String id) throws NumberFormatException {
	this.id = id;
	int_id = Integer.parseInt(id);
	base_surface = new LayeredImage();
	elements = new LayeredImage();
	organized = new LayeredImage();
	collisions = new Collisions();
	animation = new Animation();
    }

    public String toString() {
	return "koropeta.core.Surface {"+
	    "dir="+dir+", basename="+basename+", id="+id+
	    ", dirty="+dirty()+"}";
    }

    public Object clone() {
	Surface clo = new Surface(id);
	clo.dir = dir;
	clo.basename = basename;
	clo.base_surface = (LayeredImage)base_surface.clone();
	clo.elements = (LayeredImage)elements.clone();
	clo.collisions = (Collisions)collisions.clone();
	clo.animation = (Animation)animation.clone();
	return clo;
    }

    public void set_id(String id) throws NumberFormatException {
	this.id = id;
	this.int_id = Integer.parseInt(id);
    }

    public String get_id() {
	return id;
    }

    public int get_int_id() {
	return int_id;
    }

    public final Collisions collisions() {
	return collisions;
    }

    public final Animation animation() {
	return animation;
    }

    public boolean dirty() {
	if (base_surface.dirty()) {
	    return true;
	}

	if (elements.dirty()) {
	    return true;
	}

	if (collisions.dirty()) {
	    return true;
	}

	if (animation.dirty()) {
	    return true;
	}
	
	return false;
    }

    public int least_version() {
	return animation.least_version();
    }

    public void remove_file() {
	try {
	    Main.rm_rf(new File(dir, basename+".krps"));
	    Main.rm_rf(new File(dir, basename+".png"));
	    Main.rm_rf(new File(dir, basename+".pna"));
	}
	catch (IOException e) {
	    e.printStackTrace();
	}
    }

    public void endirty() {
	if (has_base_surface()) {
	    base_surface.dirty(true);
	}
    }

    public void save() {
	if (base_surface.dirty()) {
	    assert dir != null;
	    assert basename != null;
	    assert base_surface.num_layers() > 0;
	    
	    try {
		AlphaSplittedImage.write(
		    dir, basename, base_surface.compose());
	    }
	    catch (Exception e) {
		e.printStackTrace();
	    }

	    // krpsがあったら消す
	    try {
		Main.rm_rf(new File(dir, basename+".krps"));
	    }
	    catch (Exception e) {
		e.printStackTrace();
	    }
	    
	    if (!base_surface.simple_p()) {
		try {
		    save_krps(new File(dir, basename+".krps"));
		}
		catch (Exception e) {
		    e.printStackTrace();
		}
	    }

	    base_surface.dirty(false);
	}
    }

    public LayeredImage get_base_surface() {
	return base_surface(); // alias
    }
    public LayeredImage base_surface() {
	if (base_surface.num_layers() == 0 &&
	    !base_surface.dirty() &&
	    dir != null) {

	    load_base();
	}
	
	return base_surface;
    }

    public boolean has_base_surface() {
	return get_base_surface().num_layers() > 0;
    }

    public LayeredImage get_elements() {
	return elements(); // alias
    }
    public LayeredImage elements() {
	return elements;
    }

    public boolean has_elements() {
	return elements.num_layers() > 0;
    }

    public LayeredImage image() {
	if (organized.num_layers() == 0) {
	    organized.add("Base Picture", get_base_surface());
	    organized.add("Elements", elements);
	}
	
	return organized;
    }

    public final String basename() {
	return basename;
    }
    public final String basename(String name) {
	basename = name;
	return name;
    }

    public final File dir() {
	return dir;
    }
    public final File dir(File d) {
	dir = d;
	return d;
    }
    
    public void load_elements(Map desc) {
	// desc: surfaces.txt内、該當ブロックの内容。
	Matcher m;
	for (Iterator ite_key = desc.keySet().iterator(); ite_key.hasNext(); ) {
	    String key = (String)ite_key.next();

	    if ((m = re_element_key.matcher(key)).find()) {
		String value = (String)desc.get(key);
		
		if ((m = re_element_value.matcher(value)).find()) {
		    try {
			String method = m.group(1);
			String fname = m.group(2);
			int x = 0, y = 0;
			
			if (m.groupCount() >= 3) {
			    x = Integer.parseInt(m.group(3));
			    y = Integer.parseInt(m.group(4));
			}

			if (!method.equals("overlay")) {
			    throw new RuntimeException(
				"Unsupported method: "+method);
			}

			if ((m = re_png_file.matcher(fname)).find()) {
			    BufferedImage element_img;
			    try {
				element_img = AlphaSplittedImage.read(dir, m.group(1));
			    }
			    catch (Exception e) {
				Main.error(e.toString());
				element_img = new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB);
			    }
			    
			    LayeredImage.Layer l = elements.add(fname, element_img);
			    l.x(x);
			    l.y(y);
			}
			else {
			    throw new RuntimeException(
				"Image file has to be a png: "+fname);
			}
		    }
		    catch (Exception e) {
			throw new RuntimeException(
			    "Failed to load element: "+value, e);
		    }
		}
		else {
		    Main.error(
			"Invalid element: "+key+","+value);
		}
	    }
	}
    }

    public void save_elements(Map desc) {
	Matcher m;

	if (!elements.dirty()) {
	    return;
	}
	
	// 古い情報があれば消す。
	for (Iterator ite_key = desc.keySet().iterator(); ite_key.hasNext(); ) {
	    String key = (String)ite_key.next();

	    if ((m = re_element_key.matcher(key)).find()) {
		ite_key.remove();
	    }
	}

	// 保存
	for (int i = 0; i < elements.num_layers(); i++) {
	    LayeredImage.Layer l = elements.layer(i);

	    StringBuffer sb = new StringBuffer();
	    sb.append("overlay"); sb.append(',');
	    sb.append(l.name()); sb.append(',');
	    sb.append(l.x()); sb.append(',');
	    sb.append(l.y());

	    desc.put("element"+i, sb.toString());
	}

	elements.dirty(false);
    }

    protected void load_base() {
	// basename: "surface001"
	// surface001.png, surface001.pna, surface001.krpsを讀む
	File krps = new File(dir, basename+".krps");
	if (krps.isDirectory()) {
	    try {
		load_krps(krps);
	    }
	    catch (Exception e) {
		throw new RuntimeException(
		    "Failed to load base surface "+basename+".krps", e);
	    }
	    return;
	}

	File png = new File(dir, basename+".png");
	if (png.exists()) {
	    try {
		base_surface.add(
		    basename+".png",
		    AlphaSplittedImage.read(dir, basename));
	    }
	    catch (Exception e) {
		throw new RuntimeException(
		    "Failed to load base surface "+basename+".png", e);
	    }
	    return;
	}
    }

    protected void load_krps(File krps) throws Exception {
	/*
	  surface001.krps/
	  surface001.krps/surface.xml
	  surface001.krps/layer-0.pna
	  surface001.krps/layer-0.png
	  surface001.krps/layer-1.pna
	  surface001.krps/layer-1.png
	  surface001.pna   \__ この二つは
	  surface001.png   /   krpsから生成
	
	  <?xml version="1.0"?>
	  <surface version="1.0">
	    <layers>
	      <layer basename="layer-0" x="10" y="10" hidden="no" />
	      <layer basename="layer-1" x="0" y="50" />
	    </layers>
	  </surface>
	*/
	File xmlfile = new File(krps, "surface.xml");
	
	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	DocumentBuilder builder = factory.newDocumentBuilder();
	Document doc = builder.parse(xmlfile);
	Element root = doc.getDocumentElement();

	if (!root.getTagName().equals("surface") ||
	    !root.getAttribute("version").equals("1.0")) {
	    throw new Exception("Invalid xml: root element is not <surface version=\"1.0\">");
	}

	NodeList layers = 
	    ((Element)root.getElementsByTagName("layers").item(0))
	    .getElementsByTagName("layer");
	for (int i = 0; i < layers.getLength(); i++) {
	    Element layer = (Element)layers.item(i);
	    String l_basename = layer.getAttribute("basename");

	    LayeredImage.Layer l = base_surface.add(
		l_basename+".png",
		AlphaSplittedImage.read(krps, l_basename));
	    l.x(Integer.parseInt(layer.getAttribute("x")));
	    l.y(Integer.parseInt(layer.getAttribute("y")));
	    if (layer.hasAttribute("hidden")) {
		l.visible(layer.getAttribute("hidden").equals("yes") ? false : true);
	    }
	}
    }

    protected void save_krps(File krps) throws Exception {
	krps.mkdir();

	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	DocumentBuilder builder = factory.newDocumentBuilder();
	Document doc = builder.newDocument();

	Element root = doc.createElement("surface");
	root.setAttribute("version", "1.0");
	doc.appendChild(root);

	Element layers = doc.createElement("layers");
	root.appendChild(layers);

	for (Iterator ite = base_surface.iterator(); ite.hasNext(); ) {
	    LayeredImage.Layer l = (LayeredImage.Layer)ite.next();

	    int pos = l.name().lastIndexOf('.');
	    String l_basename = l.name().substring(0, pos);

	    Element layer = doc.createElement("layer");
	    layer.setAttribute("basename", l_basename);
	    layer.setAttribute("x", Integer.toString(l.x()));
	    layer.setAttribute("y", Integer.toString(l.y()));
	    if (!l.visible()) {
		layer.setAttribute("hidden", "yes");
	    }
	    layers.appendChild(layer);

	    AlphaSplittedImage.write(
		krps, l_basename, l.get());
	}

	Transformer tr = TransformerFactory.newInstance().newTransformer();
	tr.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
	tr.setOutputProperty(OutputKeys.INDENT, "yes");

	OutputStreamWriter osw = new OutputStreamWriter(
	    new FileOutputStream(new File(krps, "surface.xml")),
	    "UTF-8");
	tr.transform(new DOMSource(doc), new StreamResult(osw));
	osw.flush();
	osw.close();
    }
}