package koropeta.core;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import javax.swing.*;
import java.util.*;
import java.lang.ref.*;

public class ThumbnailMaker {
    SoftReference thumb_cache; // BufferedImage

    public void expire_cache() {
	thumb_cache = null;
    }

    public synchronized BufferedImage make(BufferedImage original, Dimension dim) {
	return make(original, dim.width, dim.height);
    }
    public synchronized BufferedImage make(BufferedImage original, int width, int height) {
	if (thumb_cache != null && thumb_cache.get() != null) {
	    BufferedImage img = (BufferedImage)thumb_cache.get();
	    
	    if (img.getWidth() == width &&
		img.getHeight() == height) {
		
		return img;
	    }
	}

	if (original == null) {
	    return null;
	}

	BufferedImage bi = new BufferedImage(
	    width, height, BufferedImage.TYPE_INT_ARGB);
	thumb_cache = new SoftReference(bi);
	    
	double scale = Math.min(
	    (double)width / original.getWidth(),
	    (double)height / original.getHeight());
	if (scale > 1.0) {
	    scale = 1.0;
	}

	ImageScaler.scale(
	    (int)((width - (original.getWidth() * scale)) / 2),
	    (int)((height - (original.getHeight() * scale)) / 2),
	    scale,
	    original, bi);

	/*AffineTransform affine = new AffineTransform();
	
	affine.concatenate(
	    AffineTransform.getTranslateInstance(
		(width - (original.getWidth() * scale)) / 2,
		(height - (original.getHeight() * scale)) / 2));
	
	affine.concatenate(
	    AffineTransform.getScaleInstance(scale, scale));
	
	RenderingHints hints = new RenderingHints(null);
	hints.put(
	    RenderingHints.KEY_INTERPOLATION,
	    RenderingHints.VALUE_INTERPOLATION_BILINEAR);

	AffineTransformOp op = new AffineTransformOp(affine, hints);
	op.filter(original, bi);*/
	
	return bi;
    }

    public ImageIcon make_icon(BufferedImage original, Dimension dim) {
	return make_icon(original, dim.width, dim.height);
    }
    public synchronized ImageIcon make_icon(BufferedImage original, int width, int height) {
	BufferedImage thumb = make(original, width, height);
	
	return new ImageIcon(thumb);
    }
}