package koropeta.core.animation;
import koropeta.*;
import java.util.*;
import javax.swing.event.*;

public class AnimGroup implements Cloneable {
    int id;
    
    String interval;
    int interval_arg = 1;
    String option;
    List patterns; // AnimPattern

    boolean is_dirty;

    List list_data_listeners;

    public AnimGroup(int id) {
        this.id = id;
        this.patterns = new Vector();
        this.list_data_listeners = new Vector();
    }

    public Object clone() {
        AnimGroup grp = new AnimGroup(id);
        grp.interval = interval;
        grp.interval_arg = interval_arg;
        grp.option = option;
        for (Iterator ite = patterns.iterator(); ite.hasNext(); ) {
            AnimPattern pat = (AnimPattern)ite.next();
            grp.patterns.add(pat.clone());
        }
        grp.is_dirty = is_dirty;
        return grp;
    }

    public final int id() {
        return id;
    }
    public final int id(int i) {
        id = i;
        return i;
    }

    public final String interval() {
        return interval;
    }
    public final String interval(String itv) {
        interval = itv;
        return itv;
    }

    public final int interval_arg() {
        return interval_arg;
    }
    public final int interval_arg(int arg) {
        interval_arg = arg;
        return arg;
    }

    public final String option() {
        return option;
    }
    public final String option(String opt) {
        option = opt;
        return opt;
    }

    public final AnimPattern get(int idx) {
        return (AnimPattern)patterns.get(idx);
    }
    public final AnimPattern set(int idx, AnimPattern pattern) {
        patterns.set(idx, pattern);
        changed(ListDataEvent.CONTENTS_CHANGED, idx);
        return pattern;
    }

    public final AnimPattern add(AnimPattern pat) {
        patterns.add(pat);
        changed(ListDataEvent.INTERVAL_ADDED, patterns.size() - 1);
        return pat;
    }

    public final AnimPattern insert(int idx, AnimPattern pat) {
        patterns.add(idx, pat);
        changed(ListDataEvent.INTERVAL_ADDED, idx);
        return pat;
    }

    public final void remove(int idx) {
        patterns.remove(idx);
        changed(ListDataEvent.INTERVAL_REMOVED, idx);
    }

    public void swap(int idx0, int idx1) {
        AnimPattern pat = (AnimPattern)patterns.get(idx0);
        patterns.set(idx0, patterns.get(idx1));
        patterns.set(idx1, pat);

        if (idx0 <= idx1) {
            changed(ListDataEvent.CONTENTS_CHANGED, idx0, idx1);
        }
        else {
            changed(ListDataEvent.CONTENTS_CHANGED, idx1, idx0);
        }
    }

    public void addListDataListener(ListDataListener listener) {
        if (!list_data_listeners.contains(listener)) {
            list_data_listeners.add(listener);
        }
    }

    public void removeListDataListener(ListDataListener listener) {
        list_data_listeners.remove(listener);
    }

    public void changed(AnimPattern pat) {
        int idx = patterns.indexOf(pat);
        if (idx != -1) {
            changed(ListDataEvent.CONTENTS_CHANGED, idx);
        }
    }

    protected void changed(int type, int index) {
        changed(type, index, index);
    }
    protected void changed(int type, int index0, int index1) {
        ListDataEvent ev = new ListDataEvent(
            this, type, index0, index1);
	
        for (Iterator ite = list_data_listeners.iterator(); ite.hasNext();) {
            ListDataListener listener = (ListDataListener)ite.next();
	    
            if (type == ListDataEvent.INTERVAL_ADDED) {
                listener.intervalAdded(ev);
            }
            else if (type == ListDataEvent.INTERVAL_REMOVED) {
                listener.intervalRemoved(ev);
            }
            else {
                listener.contentsChanged(ev);
            }
        }
    }

    public final void clear() {
        int old_size = patterns.size();
        if (old_size > 0) {
            patterns.clear();

            changed(ListDataEvent.INTERVAL_REMOVED, 0, old_size - 1);
        }
    }

    public final Iterator iterator() {
        return patterns.iterator();
    }

    public final int size() {
        return patterns.size();
    }

    public boolean dirty() {
        if (is_dirty) {
            return true;
        }

        // dirtyなパターンが一つでもあればtrue。
        for (Iterator ite = patterns.iterator(); ite.hasNext(); ) {
            AnimPattern pat = (AnimPattern)ite.next();

            if (pat.dirty()) {
                return true;
            }
        }
	
        return false;
    }
    public boolean dirty(boolean b) {
        is_dirty = b;
	
        if (b) {
            return true;
        }
        else {
            // 全パターンのdirtyフラグを解除。
            for (Iterator ite = patterns.iterator(); ite.hasNext(); ) {
                AnimPattern pat = (AnimPattern)ite.next();
		
                pat.dirty(false);
            }
            return false;
        }
    }

    public boolean has_interval_arg() {
        return
            interval.equals("random") ||
            interval.equals("talk");
    }

    public boolean is_autostarted() {
        return
            interval.equals("sometimes") ||
            interval.equals("rarely") ||
            interval.equals("random") ||
            interval.equals("always") ||
            interval.equals("runonce");
    }
    
    public boolean is_periodic() {
        return
            interval.equals("sometimes") ||
            interval.equals("rarely") ||
            interval.equals("random") ||
            interval.equals("always");
    }
}