package koropeta.core.animation;
import koropeta.*;
import java.util.*;
import java.util.regex.*;
import javax.swing.event.*;

public class Animation implements Cloneable {
    List groups; // AnimGroup
    boolean is_dirty;
    
    List list_data_listeners;
    
    public Animation() {
	groups = new Vector();
	list_data_listeners = new Vector();
    }

    public Object clone() {
	Animation clo = new Animation();
	for (Iterator ite = groups.iterator(); ite.hasNext(); ) {
	    AnimGroup group = (AnimGroup)ite.next();
	    
	    clo.groups.add(group.clone());
	}
	return clo;
    }

    public synchronized AnimGroup add(AnimGroup group) {
	// 單にaddするのでなくソートする。
	for (int i = 0; i < groups.size(); i++) {
	    AnimGroup current = (AnimGroup)groups.get(i);

	    if (group.id() <= current.id()) {
		groups.add(i, group);
		
		changed(ListDataEvent.INTERVAL_ADDED, i);
		return group;
	    }
	}

	groups.add(group);
	changed(ListDataEvent.INTERVAL_ADDED, groups.size() - 1);
	return group;
    }

    public AnimGroup group(int id) {
	for (Iterator ite = groups.iterator(); ite.hasNext(); ) {
	    AnimGroup group = (AnimGroup)ite.next();

	    if (group.id() == id) {
		return group;
	    }
	}
	return null;
    }
    
    public final int indexOf(AnimGroup group) {
	return groups.indexOf(group);
    }

    public final AnimGroup get(int idx) {
	return (AnimGroup)groups.get(idx);
    }
    public final AnimGroup set(int idx, AnimGroup group) {
	groups.set(idx, group);
	
	changed(ListDataEvent.CONTENTS_CHANGED, idx);
	return group;
    }

    public final void remove(int idx) {
	groups.remove(idx);

	changed(ListDataEvent.INTERVAL_REMOVED, idx);
    }

    public final void clear() {
	int old_size = groups.size();
	if (old_size > 0) {
	    groups.clear();

	    changed(ListDataEvent.INTERVAL_REMOVED, 0, old_size-1);
	}
    }

    public final Iterator iterator() {
	return groups.iterator();
    }

    public void swap(int idx0, int idx1) {
	AnimGroup group = (AnimGroup)groups.get(idx0);
	groups.set(idx0, groups.get(idx1));
	groups.set(idx1, group);

	if (idx0 <= idx1) {
	    changed(ListDataEvent.CONTENTS_CHANGED, idx0, idx1);
	}
	else {
	    changed(ListDataEvent.CONTENTS_CHANGED, idx1, idx0);
	}
    }

    public final int size() {
	return groups.size();
    }

    public boolean dirty() {
	if (is_dirty) {
	    return true;
	}
	
	// dirtyなグループが一つでもあればtrue。
	for (Iterator ite = groups.iterator(); ite.hasNext(); ) {
	    AnimGroup group = (AnimGroup)ite.next();

	    if (group.dirty()) {
		return true;
	    }
	}

	return false;
    }
    public boolean dirty(boolean b) {
	is_dirty = b;
	
	if (b) {
	    return true;
	}
	else {
	    // 全グループのdirtyフラグを解除
	    for (Iterator ite = groups.iterator(); ite.hasNext(); ) {
		AnimGroup group = (AnimGroup)ite.next();
		
		group.dirty(false);
	    }
	    return false;
	}
    }

    public int least_version() {
	for (Iterator ite = groups.iterator(); ite.hasNext(); ) {
	    AnimGroup group = (AnimGroup)ite.next();

	    for (Iterator ite_pat = group.iterator(); ite_pat.hasNext(); ) {
		AnimPattern pattern = (AnimPattern)ite_pat.next();

		if (pattern.has_interval() &&
		    (pattern.interval() % 10) != 0) {
		    // 1msの精度のintervalを持つ爲、SERIKO/2.0が必要。
		    return 1;
		}
	    }
	}
	return 0;
    }

    static final Pattern re_animation_key_ver0 = Pattern.compile(
	"^(\\d+)(interval|option|pattern)(\\d+)?$");
    static final Pattern re_animation_key_ver1 = Pattern.compile(
	"^animation(\\d+)\\.(.+?)(\\d+)?$");

    static final Pattern re_alternativestart_ver0 = Pattern.compile(
	"^\\d+,\\d+,alternativestart,\\[(\\d+(?:\\.\\d+)*)\\]$");
    static final Pattern re_alternativestart_ver1 = Pattern.compile(
	"^alternativestart,\\((\\d+(?:,\\d+)*)\\)$");
    
    public void load(Map desc, int version) {
	Matcher m;
	Pattern re_animation_key = version == 0 ?
	    re_animation_key_ver0 : re_animation_key_ver1;
	
	for (Iterator ite_key = desc.keySet().iterator(); ite_key.hasNext(); ) {
	    String key = (String)ite_key.next();

	    if ((m = re_animation_key.matcher(key)).find()) {
		String value = (String)desc.get(key);
		String[] fields = value.split(",");

		try {
		    int group_id = Integer.parseInt(m.group(1));
		    String type = m.group(2);

		    AnimGroup group = group(group_id);
		    if (group == null) {
			// 初見なので追加
			group = new AnimGroup(group_id);
			add(group);
		    }

		    try {
			if (type.equals("interval")) {
			    group.interval(fields[0]);
			    if (fields.length >= 2) {
				group.interval_arg(Integer.parseInt(fields[1]));
			    }
			}
			else if (type.equals("option")) {
			    group.option(value);
			}
			else if (type.equals("pattern") || type.equals("patturn")) {
			    AnimPattern pattern = new AnimPattern();
			    group.add(pattern);
			    
			    String method = fields[version == 0 ? 2 : 0];
			    pattern.method(method);
			    
			    if (method.equals("overlay") ||
				method.equals("overlayfast") ||
				method.equals("reduce")) {
				
				pattern.surface_id(
				    Integer.parseInt(fields[version == 0 ? 0 : 1]));
				pattern.interval(
				    Integer.parseInt(fields[version == 0 ? 1 : 2])
				    * (version == 0 ? 10 : 1));
				pattern.offset_x(
				    Integer.parseInt(fields[3]));
				pattern.offset_y(
				    Integer.parseInt(fields[4]));
			    }
			    else if (method.equals("base")) {
				pattern.surface_id(
				    Integer.parseInt(fields[version == 0 ? 0 : 1]));
				pattern.interval(
				    Integer.parseInt(fields[version == 0 ? 1 : 2])
				    * (version == 0 ? 10 : 1));
			    }
			    else if (method.equals("move")) {
				pattern.interval(
				    Integer.parseInt(fields[version == 0 ? 1 : 2])
				    * (version == 0 ? 10 : 1));
				pattern.offset_x(
				    Integer.parseInt(fields[3]));
				pattern.offset_y(
				    Integer.parseInt(fields[4]));
			    }
			    else if (method.equals("start")) {
				pattern.start_id(
				    Integer.parseInt(fields[version == 0 ? 3 : 1]));
			    }
			    else if (method.equals("alternativestart")) {
				Pattern re_alternativestart = version == 0 ?
				    re_alternativestart_ver0 : re_alternativestart_ver1;
				
				m = re_alternativestart.matcher(value);
				if (!m.find()) {
				    throw new RuntimeException(
					"Invalid alternativestart");
				}
				
				String[] items = m.group(1).split(version == 0 ? "\\." : ",");
				Vector ids = new Vector();
				for (int i = 0; i < items.length; i++) {
				    ids.add(new Integer(items[i]));
				}
				pattern.alternative_ids(ids);
			    }
			    else if (method.equals("bind") ||
				     method.equals("add")) {
				pattern.surface_id(
				    Integer.parseInt(fields[version == 0 ? 0 : 1]));
				pattern.offset_x(
				    Integer.parseInt(fields[3]));
				pattern.offset_y(
				    Integer.parseInt(fields[4]));
			    }
			    else if (method.equals("insert")) {
				pattern.insert_id(
				    Integer.parseInt(fields[version == 0 ? 3 : 1]));
			    }
			    else {
				throw new RuntimeException(
				    "Invalid method: "+method);
			    }
			}
			else {
			    throw new RuntimeException(
				"Invalid type: "+type);
			}
		    }
		    catch (ArrayIndexOutOfBoundsException e) {
			Main.error(
			    "Warning: some necessary parameters seem to be ommited: "+key+","+value);
		    }
		}
		catch (Exception e) {
		    throw new RuntimeException(
			"Failed to load animation: "+key+","+value, e);
		}
	    }
	}
    }

    public void save(Map desc, int version) {
	// アニメーション定義を全て削除
	for (Iterator ite_key = desc.keySet().iterator(); ite_key.hasNext(); ) {
	    String key = (String)ite_key.next();
	    
	    Matcher m;
	    if ((m = re_animation_key_ver0.matcher(key)).find() ||
		(m = re_animation_key_ver1.matcher(key)).find()) {

		ite_key.remove();
	    }
	}
	
	for (Iterator ite = groups.iterator(); ite.hasNext(); ) {
	    AnimGroup group = (AnimGroup)ite.next();
	    group.dirty(false);

	    // interval, option(あれば), patternを置く。
	    String interval_key = version == 0 ?
		group.id()+"interval" : "animation"+group.id()+".interval";
	    if (group.has_interval_arg()) {
		desc.put(interval_key, group.interval()+","+group.interval_arg());
	    }
	    else {
		desc.put(interval_key, group.interval());
	    }

	    if (group.option() != null) {
		desc.put(
		    (version == 0 ?
		     group.id()+"option" : "animation"+group.id()+".option"),
		    group.option());
	    }

	    for (int i = 0; i < group.size(); i++) {
		AnimPattern pattern = group.get(i);

		String pattern_key = version == 0 ?
		    group.id()+"pattern"+i : "animation"+group.id()+".pattern"+i;

		String method = pattern.method();
		if (method.equals("overlay") ||
		    method.equals("overlayfast") ||
		    method.equals("reduce")) {
		    if (version == 0) {
			desc.put(
			    pattern_key,
			    (pattern.surface_id()+","+
			     (int)(pattern.interval() / 10)+","+
			     pattern.method()+","+
			     pattern.offset_x()+","+
			     pattern.offset_y()));
		    }
		    else {
			desc.put(
			    pattern_key,
			    (pattern.method()+","+
			     pattern.surface_id()+","+
			     pattern.interval()+","+
			     pattern.offset_x()+","+
			     pattern.offset_y()));
		    }
		}
		else if (method.equals("base")) {
		    if (version == 0) {
			desc.put(
			    pattern_key,
			    (pattern.surface_id()+","+
			     (int)(pattern.interval() / 10)+","+
			     pattern.method()));
		    }
		    else {
			desc.put(
			    pattern_key,
			    (pattern.method()+","+
			     pattern.surface_id()+","+
			     pattern.interval()));
		    }
		}
		else if (method.equals("move")) {
		    if (version == 0) {
			desc.put(
			    pattern_key,
			    ("0,"+
			     (int)(pattern.interval() / 10)+","+
			     pattern.method()+","+
			     pattern.offset_x()+","+
			     pattern.offset_y()));
		    }
		    else {
			desc.put(
			    pattern_key,
			    (pattern.method()+","+
			     "0,"+
			     pattern.interval()+","+
			     pattern.offset_x()+","+
			     pattern.offset_y()));
		    }
		}
		else if (method.equals("start")) {
		    if (version == 0) {
			desc.put(
			    pattern_key, "0,0,start,"+pattern.start_id());
		    }
		    else {
			desc.put(
			    pattern_key,
			    (pattern.method()+","+
			     pattern.start_id()));
		    }
		}
		else if (method.equals("alternativestart")) {
		    if (version == 0) {
			desc.put(
			    pattern_key,
			    ("0,0,alternativestart,["+
			     Main.join(".", pattern.alternative_ids())+"]"));
		    }
		    else {
			desc.put(
			    pattern_key,
			    ("alternativestart,("+
			     Main.join(",", pattern.alternative_ids())+")"));
		    }
		}
		else if (method.equals("insert")) {
		    if (version == 0) {
			desc.put(
			    pattern_key,
			    "0,0,insert,"+pattern.insert_id());
		    }
		    else {
			desc.put(
			    pattern_key,
			    "insert,"+pattern.insert_id());
		    }
		}
		else {
		    throw new RuntimeException(
			"Invalid method: "+method);
		}
	    }
	}
	
	dirty(false);
    }

    public void addListDataListener(ListDataListener listener) {
	if (!list_data_listeners.contains(listener)) {
	    list_data_listeners.add(listener);
	}
    }

    public void removeListDataListener(ListDataListener listener) {
	list_data_listeners.remove(listener);
    }

    public void changed(AnimGroup group) {
	int idx = groups.indexOf(group);
	if (idx != -1) {
	    changed(ListDataEvent.CONTENTS_CHANGED, idx);
	}
    }

    protected void changed(int type, int index) {
	changed(type, index, index);
    }
    protected void changed(int type, int index0, int index1) {
	ListDataEvent ev = new ListDataEvent(
	    this, type, index0, index1);
	
	for (Iterator ite = list_data_listeners.iterator(); ite.hasNext();) {
	    ListDataListener listener = (ListDataListener)ite.next();
	    
	    if (type == ListDataEvent.INTERVAL_ADDED) {
		listener.intervalAdded(ev);
	    }
	    else if (type == ListDataEvent.INTERVAL_REMOVED) {
		listener.intervalRemoved(ev);
	    }
	    else {
		listener.contentsChanged(ev);
	    }
	}
    }
}