package koropeta.gui;
import koropeta.*;
import koropeta.gui.*;
import koropeta.core.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.color.*;

public class LayeredImageComponent extends JComponent {
    private static final int checker_width = 15;
    
    protected ListDataListener list_data_listener;
    protected LayeredImage image;

    protected int padding_top;
    protected int padding_right;
    protected int padding_bottom;
    protected int padding_left;

    protected int offset_x;
    protected int offset_y;
    protected boolean draw_axes;

    public LayeredImageComponent(LayeredImage image) {
	this.image = image;

	setBorder(new EtchedBorder(EtchedBorder.RAISED));

	list_data_listener = new ListDataListener() {
		public void intervalAdded(ListDataEvent e) { repaint(); }
		public void intervalRemoved(ListDataEvent e) { repaint(); }
		public void contentsChanged(ListDataEvent e) { repaint(); }
	    };
	image.addListDataListener(list_data_listener);
    }

    public void repaint() {
	Dimension size = getPreferredSize();
	if (size.width != getWidth() ||
	    size.height != getHeight()) {

	    setSize(size);

	    // 親コンポーネントを辿り、Windowが見付かったら
	    // そこでpack()を實行する。
	    for (Component c = this; c != null; c = c.getParent()) {
		if (c instanceof Window) {
		    ((Window)c).pack();
		}
	    }
	}
	
	super.repaint();
    }
	
    public void close() {
	image.removeListDataListener(list_data_listener);
    }

    public LayeredImage image() {
	return image;
    }

    public Dimension image_size() {
	Dimension size = image.size();
	
	if (size.width == 0 || size.height == 0) {
	    return new Dimension(300, 300);
	}
	else {
	    return size;
	}
    }

    public Dimension getMaximumSize() {
	return getPreferredSize();
    }

    public Dimension getMinimumSize() {
	return getPreferredSize();
    }

    public Dimension getPreferredSize() {
	Dimension size = image_size();
	size.width += padding_left + padding_right;
	size.height += padding_top + padding_bottom;
	return size;
    }

    public final int num_layers() {
	return image.num_layers();
    }

    public final void padding(int top, int right, int left, int bottom) {
	padding_top = top;
	padding_right = right;
	padding_bottom = bottom;
	padding_left = left;
    }

    public final void offset(int x, int y) {
	offset_x = x;
	offset_y = y;
	repaint();
    }

    public final boolean draw_axes() {
	return draw_axes;
    }
    public final boolean draw_axes(boolean b) {
	draw_axes = b;
	return b;
    }

    public void paintComponent(Graphics g) {
	Graphics2D g2 = (Graphics2D)g.create();
	Dimension size = getSize();
	
	g2.setColor(Color.lightGray);
	g2.fillRect(0, 0, size.width, size.height);

	// 背景を描畫
	for (int y = 0; y < size.height; y += checker_width * 2) {
	    for (int x = 0; x < size.width; x += checker_width * 2) {
		g2.setColor(Color.GRAY);
		g2.fillRect(x, y, checker_width, checker_width);
	    }

	    for (int x = 0; x < size.width; x += checker_width * 2) {
		g2.setColor(Color.GRAY);
		g2.fillRect(
		    x + checker_width, y + checker_width,
		    checker_width, checker_width);
	    }
	}

	// 軸を描畫
	if (draw_axes) {
	    Line2D line = new Line2D.Float();
	    g2.setColor(Color.BLACK);

	    // 横軸
	    line.setLine(
		0         , getHeight() - padding_bottom,
		getWidth(), getHeight() - padding_bottom);
	    g2.draw(line);

	    // 縦軸
	    line.setLine(
		padding_left, getHeight(),
		padding_left, 0          );
	    g2.draw(line);
	}

	draw_layers(g2, image, padding_left + offset_x, padding_top + offset_y);
    }

    private void draw_layers(Graphics2D g2, LayeredImage image, int x, int y) {
	java.util.List layers = image.layers();
	synchronized (layers) {
	    try {
		for (Iterator ite = layers.iterator(); ite.hasNext(); ) {
		    LayeredImage.Layer layer = (LayeredImage.Layer)ite.next();
		    
		    if (layer.visible()) {
			if (layer instanceof LayeredImage.ReferenceLayer) {
			    LayeredImage ref = ((LayeredImage.ReferenceLayer)layer).layers();
			
			    // 再歸
			    draw_layers(g2, ref, x + layer.x(), y + layer.y());
			}
			else {
			    BufferedImage img = layer.get();
			    if (img != null) {
				g2.drawImage(img, null, x + layer.x(), y + layer.y());
			    }
			}
		    }
		}
	    }
	    catch (ConcurrentModificationException e) {
		System.err.println("LayeredImageComponent#draw_layer(): "+e.toString());
	    }
	}
    }
}