package koropeta.gui;
import koropeta.*;
import koropeta.gui.*;
import koropeta.core.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

public class PreferencesFrame extends JFrame {
    private static PreferencesFrame _shared = null;

    public static void display() {
	if (_shared == null) {
	    _shared = new PreferencesFrame();
	}
	_shared.setVisible(true);
	_shared.toFront();
    }

    protected PreferencesFrame() {
	super(GT.tr("Preferences"));

	getContentPane().setLayout(new SpringLayout());

	getContentPane().add(new SerikoVersionPanel());
	getContentPane().add(new LookAndFeelPanel());
	getContentPane().add(new LanguagePanel());
	getContentPane().add(new SurfaceGridPanel());

	SpringUtils.makeCompactGrid(
	    getContentPane(),
	    4, 1,
	    3, 3,
	    3, 3);

	pack();
    }

    private class SerikoVersionPanel extends JPanel {
	public SerikoVersionPanel() {
	    /*
	      +- 保存する際に用いるSERIKOのバージョン --+
	      | ( ) 常にSERIKO/1.x                          |
	      | (*) 可能な限りSERIKO/1.x                   |
	      | ( ) 常にSERIKO/2.0                          |
	      +----------------------------------------------+
	    */
	    setBorder(
		new TitledBorder(
		    new EtchedBorder(EtchedBorder.LOWERED),
		    GT.tr("Version of SERIKO to choose for saving"),
		    TitledBorder.TRAILING,
		    TitledBorder.TOP));
	    
	    setLayout(
		new BoxLayout(this, BoxLayout.Y_AXIS));

	    ButtonGroup group = new ButtonGroup();
	    JRadioButton radio;
	    
	    String seriko_version = Main.ini().getProperty(
		"koropeta.pref.serikoVersion", "1-as-possible");
	    
	    radio = new JRadioButton(GT.tr("Always SERIKO/1.x"));
	    radio.setToolTipText(
		GT.tr("Some information will be dropped if they can not be "+
		      "represented in this version of SERIKO."));
	    radio.setSelected(seriko_version.equals("always-1"));
	    radio.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent ev) {
			Main.ini().setProperty(
			    "koropeta.pref.serikoVersion", "always-1");
			Main.save_ini();
		    }
		});
	    group.add(radio);
	    add(radio);
	    
	    radio = new JRadioButton(GT.tr("SERIKO/1.x as possible"));
	    radio.setToolTipText(
		GT.tr("SERIKO/2.0 or later will be chosen if some of the information "+
		      "can not be represented in SERIKO/1.x."));
	    radio.setSelected(seriko_version.equals("1-as-possible"));
	    radio.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent ev) {
			Main.ini().setProperty(
			    "koropeta.pref.serikoVersion", "1-as-possible");
			Main.save_ini();
		    }
		});
	    group.add(radio);
	    add(radio);
	    
	    radio = new JRadioButton(GT.tr("Always SERIKO/2.0"));
	    radio.setToolTipText(
		GT.tr("An old platform might not be able to interpret this version."));
	    radio.setSelected(seriko_version.equals("always-2"));
	    radio.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent ev) {
			Main.ini().setProperty(
			    "koropeta.pref.serikoVersion", "always-2");
			Main.save_ini();
		    }
		});
	    group.add(radio);
	    add(radio);
	}
    }

    private class LookAndFeelPanel extends JPanel {
	public LookAndFeelPanel() {
	    /*
	      +- 外見と感覚 ----------------------------------------+
	      | [ Metal |▼]                                         |
	      | 設定変更後、再起動するまでは効果が現れません。|
	      +-------------------------------------------------------+
	    */
	    setBorder(
		new TitledBorder(
		    new EtchedBorder(EtchedBorder.LOWERED),
		    GT.tr("Look & Feel"),
		    TitledBorder.TRAILING,
		    TitledBorder.TOP));

	    setLayout(
		new BoxLayout(this, BoxLayout.Y_AXIS));

	    final JComboBox cmb_laf = new JComboBox();
	    final UIManager.LookAndFeelInfo[] lafs = UIManager.getInstalledLookAndFeels();
	    for (int i = 0; i < lafs.length; i++) {
		cmb_laf.addItem(lafs[i].getName());

		if (lafs[i].getClassName().equals(
			Main.ini().getProperty(
			    "koropeta.gui.lookAndFeel",
			    "javax.swing.plaf.metal.MetalLookAndFeel"))) {
		    cmb_laf.setSelectedIndex(i);
		}
	    }
	    cmb_laf.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent ev) {
			String selected = lafs[cmb_laf.getSelectedIndex()].getClassName();
			try {
			    UIManager.setLookAndFeel(selected);
			}
			catch (Exception ex) {
			    throw new RuntimeException(ex);
			}
			Main.ini().setProperty(
			    "koropeta.gui.lookAndFeel", selected);
			Main.save_ini();
		    }
		});
	    add(cmb_laf);

	    add(new JLabel(
		    GT.tr("Changing will take effect only after restarting.")));
	}
    }

    private class LanguagePanel extends JPanel {
	
	final Object[][] languages;

	public int find_locale(Locale locale) {
	    // 指定されたロケールに最も良く合致するものを
	    // languagesから撰び、その位置を返す。
	    for (int i = 0; i < languages.length; i++) {
		Locale loc = (Locale)languages[i][0];
		if (loc.getLanguage().equals(locale.getLanguage()) &&
		    loc.getCountry().equals(locale.getCountry()) &&
		    loc.getVariant().equals(locale.getVariant())) {
		    return i;
		}
	    }
	    for (int i = 0; i < languages.length; i++) {
		Locale loc = (Locale)languages[i][0];
		if (loc.getLanguage().equals(locale.getLanguage()) &&
		    loc.getCountry().equals(locale.getCountry())) {
		    return i;
		}
	    }
	    for (int i = 0; i < languages.length; i++) {
		Locale loc = (Locale)languages[i][0];
		if (loc.getLanguage().equals(locale.getLanguage()) &&
		    loc.getCountry().equals(locale.getCountry())) {
		    return i;
		}
	    }
	    return -1;
	}
	
	public LanguagePanel() {
	    /*
	      +- 言語 ------------------------------------------------+
	      | [ Gaeilge |▼]                                        |
	      | 設定変更後、再起動するまでは効果が現れません。 |
	      +--------------------------------------------------------+
	    */
	    setBorder(
		new TitledBorder(
		    new EtchedBorder(EtchedBorder.LOWERED),
		    GT.tr("Language"),
		    TitledBorder.TRAILING,
		    TitledBorder.TOP));

	    setLayout(
		new BoxLayout(this, BoxLayout.Y_AXIS));

	    languages = new Object[][] {
		// ロケールのアルファベット順に並べる
		new Object[] {
		    new Locale("en"), "English"
		},
		new Object[] {
		    new Locale("ja"), "日本語"
		},
		new Object[] {
		    new Locale("ja", "JP", "TRADITIONAL"), "日本語 (正字正假名)"
		},
	    };

	    final JComboBox cmb_lang = new JComboBox();
	    cmb_lang.addItem(GT.tr("OS default"));
	    for (int i = 0; i < languages.length; i++) {
		cmb_lang.addItem((String)languages[i][1]);
	    }
	    if (Main.ini().getProperty("koropeta.gui.locale") == null) {
		cmb_lang.setSelectedIndex(0);
	    }
	    else {
		int idx = find_locale(Main.get_locale());
		cmb_lang.setSelectedIndex(idx + 1);
	    }
	    cmb_lang.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent ev) {
			int idx = cmb_lang.getSelectedIndex();
			if (idx == 0) {
			    // OS defaultなら屬性を消す
			    Main.ini().remove("koropeta.gui.locale");
			}
			else {
			    Locale loc = (Locale)languages[idx - 1][0];
			    Main.ini().setProperty("koropeta.gui.locale", loc.toString());
			}
			Main.save_ini();
		    }
		});
	    add(cmb_lang);

	    add(new JLabel(
		    GT.tr("Changing will take effect only after restarting.")));
	}
    }

    private class SurfaceGridPanel extends JPanel {
	public SurfaceGridPanel() {
	    /*
	      +- サーフィス一覧のサムネイルの大きさ -+
	      | ========#========= 横幅 [ 50]px           |
	      | ==========#======= 縦幅 [ 70]px           |
	      +--------------------------------------------+
	    */
	    setBorder(
		new TitledBorder(
		    new EtchedBorder(EtchedBorder.LOWERED),
		    GT.tr("Size of thumbnail in surface grid window"),
		    TitledBorder.TRAILING,
		    TitledBorder.TOP));

	    setLayout(new SpringLayout());

	    int width = 120, height = 120;
	    try {
		width = Integer.parseInt(
		    Main.ini().getProperty("koropeta.gui.SurfaceGridFrame.thumbnail.width"));
		height = Integer.parseInt(
		    Main.ini().getProperty("koropeta.gui.SurfaceGridFrame.thumbnail.height"));
	    }
	    catch (Exception e) {}

	    {
		final JSlider slider = new JSlider(50, 300, width);
		slider.setMajorTickSpacing(10);
		slider.setPaintTicks(true);
		slider.setSnapToTicks(true);
		add(slider);

		add(new JLabel(GT.tr("Width"), SwingConstants.TRAILING));

		final JTextField field = new JTextField(Integer.toString(width));
		field.setColumns(5);
		field.setHorizontalAlignment(JTextField.TRAILING);
		field.setEditable(false);
		add(field);

		add(new JLabel("px"));

		slider.addChangeListener(
		    new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
			    if (!slider.getValueIsAdjusting()) {
				Main.ini().setProperty(
				    "koropeta.gui.SurfaceGridFrame.thumbnail.width",
				    Integer.toString(slider.getValue()));
				Main.save_ini();
				
				for (Iterator ite = Main.shell_frames().iterator(); ite.hasNext(); ) {
				    SurfaceGridFrame sgf = (SurfaceGridFrame)ite.next();
				    sgf.resize_grid();
				}
			    }
			    field.setText(
				Integer.toString(slider.getValue()));
			}
		    });
	    }

	    {
		final JSlider slider = new JSlider(50, 300, height);
		slider.setMajorTickSpacing(10);
		slider.setPaintTicks(true);
		slider.setSnapToTicks(true);
		add(slider);

		add(new JLabel(GT.tr("Height"), SwingConstants.TRAILING));

		final JTextField field = new JTextField(Integer.toString(height));
		field.setColumns(4);
		field.setHorizontalAlignment(JTextField.TRAILING);
		field.setEditable(false);
		add(field);

		add(new JLabel("px"));

		slider.addChangeListener(
		    new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
			    if (!slider.getValueIsAdjusting()) {
				Main.ini().setProperty(
				    "koropeta.gui.SurfaceGridFrame.thumbnail.height",
				    Integer.toString(slider.getValue()));
				Main.save_ini();
				
				for (Iterator ite = Main.shell_frames().iterator(); ite.hasNext(); ) {
				    SurfaceGridFrame sgf = (SurfaceGridFrame)ite.next();
				    sgf.resize_grid();
				}
			    }
			    field.setText(
				Integer.toString(slider.getValue()));
			}
		    });
	    }

	    SpringUtils.makeCompactGrid(
		this,
		2, 4,
		0, 0,
		3, 1);
	}
    }
}