package koropeta.gui;
import koropeta.core.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.color.*;

public class SurfaceCatalogComponent extends JComponent {
    private static final int checker_width = 15;
    
    Surface surface;
    Vector action_listeners;

    boolean pressed;

    public SurfaceCatalogComponent(final Surface surface) {
	this.surface = surface;
	this.action_listeners = new Vector();
	
	setBorder(new EtchedBorder(EtchedBorder.RAISED));

	addMouseListener(
	    new MouseInputAdapter() {
		public void mouseClicked(MouseEvent ev) {
		    if (ev.getClickCount() == 2) {
			ActionEvent act = new ActionEvent(
			    SurfaceCatalogComponent.this,
			    ActionEvent.ACTION_PERFORMED,
			    "double-click");
			
			for (Iterator ite = action_listeners.iterator(); ite.hasNext(); ) {
			    ActionListener listener = (ActionListener)ite.next();
			    listener.actionPerformed(act);
			}
		    }
		}

		public void mousePressed(MouseEvent ev) {
		    if (!ev.isPopupTrigger()) {
			pressed = true;
			repaint();
		    }
		}

		public void mouseReleased(MouseEvent ev) {
		    if (!ev.isPopupTrigger()) {
			pressed = false;
			repaint();
		    }
		}
	    });
    }

    public void addActionListener(ActionListener listener) {
	if (!action_listeners.contains(listener)) {
	    action_listeners.add(listener);
	}
    }
    public void removeActionListener(ActionListener listener) {
	action_listeners.remove(listener);
    }

    public void paintComponent(Graphics g) {
	Graphics2D g2 = (Graphics2D)g.create();
	Dimension size = getSize();
	
	g2.setColor(Color.lightGray);
	g2.fillRect(0, 0, size.width, size.height);

	// 背景を描畫
	for (int y = 0; y < size.height; y += checker_width * 2) {
	    for (int x = 0; x < size.width; x += checker_width * 2) {
		g2.setColor(Color.GRAY);
		g2.fillRect(x, y, checker_width, checker_width);
	    }

	    for (int x = 0; x < size.width; x += checker_width * 2) {
		g2.setColor(Color.GRAY);
		g2.fillRect(
		    x + checker_width, y + checker_width,
		    checker_width, checker_width);
	    }
	}

	// サムネイルを描畫
	BufferedImage img = surface.image().thumbnail(getSize());
	if (img != null) {
	    g2.drawImage(img, null, 0, 0);
	}

	// IDを描畫
	Dimension id_pos = new Dimension(2, size.height - 4);
	
	g2.setPaint(Color.LIGHT_GRAY);
	g2.drawString(
	    surface.get_id(), id_pos.width - 1, id_pos.height - 1);
	g2.drawString(
	    surface.get_id(), id_pos.width - 1, id_pos.height + 1);
	g2.drawString(
	    surface.get_id(), id_pos.width + 1, id_pos.height - 1);
	g2.drawString(
	    surface.get_id(), id_pos.width + 1, id_pos.height + 1);

	g2.setPaint(Color.BLACK);
	g2.drawString(
	    surface.get_id(), id_pos.width, id_pos.height);

	// マウスが押されてゐるなら少し暗くする。
	if (pressed) {
	    g2.setPaint(new Color(0.0f, 0.0f, 0.0f, 0.2f));
	    g2.fillRect(0, 0, size.width, size.height);
	}
    }
}