package koropeta.gui;
import koropeta.*;
import koropeta.gui.*;
import koropeta.gui.surfaceeditor.*;
import koropeta.gui.metadataeditor.*;
import koropeta.core.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class SurfaceGridFrame extends JFrame {
    protected static int initial_window_x = 50;
    protected static int initial_window_y = 50;

    Shell shell;
    Map surface_editors; // <Integer, SurfaceEditorFrame>

    JMenuItem mi_save_shell;
    JPanel grid_panel;

    ShellInformationFrame shell_info_frame;

    public SurfaceGridFrame() {
	this(null);
    }
    
    public SurfaceGridFrame(File shell_path) {
	super(GT.tr(
		  /* Koropeta #VERSION */
		  "Koropeta #{0,number}", Main.VERSION));

	shell = new Shell(shell_path);
	update_title();

	surface_editors = new Hashtable();

	try {
	    setBounds(
		initial_window_x,
		initial_window_y,
		Integer.parseInt(
		    Main.ini().getProperty(
			"koropeta.gui.SurfaceGridFrame.frameWidth", "640")),
		Integer.parseInt(
		    Main.ini().getProperty(
			"koropeta.gui.SurfaceGridFrame.frameHeight", "480"))
		);
	}
	catch (Exception e) {
	    e.printStackTrace();

	    setBounds(initial_window_x, initial_window_y, 640, 480);
	}
	initial_window_x += 10;
	initial_window_y += 10;

	addComponentListener(make_component_listener());
	addWindowListener(make_window_listener());
	setJMenuBar(make_menu_bar());

	SpringLayout sl = new SpringLayout();
	getContentPane().setLayout(sl);

	JButton descr = new JButton(GT.tr("About this screen..."));
	descr.addActionListener(
	    new ActionListener() {
		public void actionPerformed(ActionEvent ev) {
		    JOptionPane.showMessageDialog(
			SurfaceGridFrame.this,
			GT.tr(
			    "In this screen, you can manipulate meta-data and contents\n"+
			    "of a shell.")+
			"\n\n"+
			GT.tr(
			    "Double click on a thumbnail to open a surface editor.")+
			"\n\n"+
			GT.tr(
			    "You can remove/duplicate a surface by a context menu."),
			GT.tr("About this screen..."),
			JOptionPane.INFORMATION_MESSAGE);
		}
	    });
	getContentPane().add(descr);
	sl.putConstraint(
	    SpringLayout.NORTH, descr,
	    1,
	    SpringLayout.NORTH, getContentPane());
	sl.putConstraint(
	    SpringLayout.WEST, descr,
	    1,
	    SpringLayout.WEST, getContentPane());
	sl.putConstraint(
	    SpringLayout.EAST, getContentPane(),
	    Spring.constant(1, 1, Integer.MAX_VALUE),
	    SpringLayout.EAST, descr);
	
	JScrollPane scr = new JScrollPane(
	    JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
	    JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	{
	    grid_panel = new TrackingScrollablePanel(new GridLayout(0, 10));
	    scr.getViewport().setView(grid_panel);
	}
	getContentPane().add(scr);
	sl.putConstraint(
	    SpringLayout.NORTH, scr,
	    1,
	    SpringLayout.SOUTH, descr);
	sl.putConstraint(
	    SpringLayout.SOUTH, getContentPane(),
	    0,
	    SpringLayout.SOUTH, scr);
	
	sl.putConstraint(
	    SpringLayout.EAST, scr,
	    0,
	    SpringLayout.EAST, getContentPane());
	sl.putConstraint(
	    SpringLayout.WEST, scr,
	    0,
	    SpringLayout.WEST, getContentPane());

	setVisible(true);
	reload_grid();
	//resize_grid();
    }

    public boolean dirty_p() {
	return shell.dirty();
    }

    public void request_close() {
	if (dirty_p()) {
	    while (true) {
		// dirtyなら確認
		int retval = JOptionPane.showConfirmDialog(
		    this,
		    GT.tr(
			"The shell \"{0}\" has been modified.\n"+
			"Do you want to save it before closing?", get_shell_name()),
		    GT.tr("Confirmation"),
		    JOptionPane.YES_NO_CANCEL_OPTION);
		
		if (retval == JOptionPane.YES_OPTION) {
		    if (save_shell()) {
			dispose();
			return;
		    }
		    else {
			continue;
		    }
		}
		else if (retval == JOptionPane.NO_OPTION) {
		    dispose();
		    return;
		}
		else {
		    return;
		}
	    }
	}
	else {
	    dispose();
	}
    }

    public void open_surface_editor(int surface_id) {
	Integer id = new Integer(surface_id);

	SurfaceEditorFrame editor = (SurfaceEditorFrame)surface_editors.get(id);
	if (editor != null) {
	    editor.toFront();
	}
	else {
	    surface_editors.put(
		id,
		new SurfaceEditorFrame(
		    this,
		    (Surface)shell.get_surfaces().get(id)));
	}
    }

    public void close_surface_editor(int surface_id) {
	Integer id = new Integer(surface_id);

	SurfaceEditorFrame editor = (SurfaceEditorFrame)surface_editors.get(id);
	if (editor != null) {
	    editor.dispose();
	    surface_editors.remove(id);
	}
    }

    public void close_all_surface_editors() {
	for (Iterator ite = surface_editors.values().iterator(); ite.hasNext(); ) {
	    SurfaceEditorFrame sef = (SurfaceEditorFrame)ite.next();
	    sef.dispose();
	}
	surface_editors.clear();
    }

    public String get_shell_name() {
	String name = (String)shell.get_description().get("name");
	return name == null ? GT.tr("Untitled") : name;
    }

    public final Shell get_shell() {
	return shell(); // alias
    }
    public final Shell shell() {
	return shell;
    }

    public void update_title() {
	update_title(null);
    }
    protected void update_title(String status) {
	String title = GT.tr(
	    /* Koropeta #VERSION: NAME-OF-SHELL */
	    "Koropeta #{0,number}: {1}",
	    new Object[] {new Integer(Main.VERSION), get_shell_name()});
	
	if (status == null) {
	    setTitle(title);
	}
	else {
	    setTitle(title+" ["+status+"]");
	}
    }

    Thread reloader_thread;
    private void reload_grid() {
	if (reloader_thread != null && reloader_thread.isAlive()) {
	    return;
	}

	reloader_thread = new Thread(
	    new Runnable() {
		public void run() {
		    do_reload_grid();
		}
	    });
	reloader_thread.setPriority(Thread.MIN_PRIORITY);
	reloader_thread.start();
    }
    private void do_reload_grid() {
	grid_panel.removeAll();

	Map surfaces = shell.get_surfaces();
	synchronized (surfaces) {
	    int i = 0;
	    for (Iterator ite = surfaces.values().iterator(); ite.hasNext(); i++) {
		final Surface surface = (Surface)ite.next();

		final SurfaceCatalogComponent item = new SurfaceCatalogComponent(surface);
		item.setPreferredSize(new Dimension(100, 100));
		item.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
			    open_surface_editor(surface.get_int_id());
			}
		    });
		{
		    final JPopupMenu pmenu = new JPopupMenu();

		    final JMenuItem dup = new JMenuItem(GT.tr("Duplicate"));
		    dup.addActionListener(
			new ActionListener() {
			    public void actionPerformed(ActionEvent e) {
				int id = new SurfaceIDChooserDialog(
				    SurfaceGridFrame.this).choose();
				if (id != -1) {
				    shell.add_clone(surface, id);
				    reload_grid();
				}
			    }
			});
		    pmenu.add(dup);

		    final JMenuItem rm = new JMenuItem(GT.tr("Remove..."));
		    rm.addActionListener(
			new ActionListener() {
			    public void actionPerformed(ActionEvent e) {
				int retval = JOptionPane.showConfirmDialog(
				    SurfaceGridFrame.this,
				    GT.tr(
					"Do you really want to remove this surface?\n"+
					"If you continue this operation, the corresponding file on your\n"+
					"filesystem will be removed immediately."),
				    GT.tr("Confirmation"),
				    JOptionPane.YES_NO_OPTION);
				if (retval == JOptionPane.YES_OPTION) {
				    shell.remove_surface(surface);
				    reload_grid();
				}
			    }
			});
		    pmenu.add(rm);

		    item.addMouseListener(
			new MouseAdapter() {
			    public void mousePressed(MouseEvent e) { popup(e); }
			    public void mouseReleased(MouseEvent e) { popup(e); }
			    private void popup(MouseEvent e) {
				if (e.isPopupTrigger()) {
				    e.consume();

				    pmenu.show(
					item, e.getX(), e.getY());
				}
			    }
			});
		}
		grid_panel.add(item);
		
		resize_grid();
		update_title(
		    GT.tr(
			"Loading surfaces... {0}/{1} loaded.",
			new Object[] {
			    new Integer(i + 1),
			    new Integer(surfaces.size())}));

		Thread.yield();
	    }

	    update_title();
	}
    }

    public void resize_grid() {
	// 高さ及び幅が120pxを下回らない最大の分割數を求める。
	GridLayout grid = (GridLayout)grid_panel.getLayout();

	int minimum_width = 120;
	int minimum_height = 120;
	try {
	    minimum_width = Integer.parseInt(
		Main.ini().getProperty("koropeta.gui.SurfaceGridFrame.thumbnail.width"));
	    minimum_height = Integer.parseInt(
		Main.ini().getProperty("koropeta.gui.SurfaceGridFrame.thumbnail.height"));
	}
	catch (Exception e) {}

	int cols = grid_panel.getWidth() / minimum_width;
	if (cols == 0) {
	    cols++;
	}
	grid.setColumns(cols);
	grid_panel.revalidate();

	if (grid_panel.getComponentCount() > 0) {
	    int rows = (int)Math.ceil(grid_panel.getComponentCount() / (double)cols);
	    
	    grid_panel.setPreferredSize(
		new Dimension(
		    grid_panel.getWidth(),
		    rows * minimum_height));
	    
	    grid_panel.revalidate();
	}

	//grid_panel.paintImmediately(new Rectangle(grid_panel.getSize()));
    }

    private ComponentAdapter make_component_listener() {
	return new ComponentAdapter() {
		public void componentResized(ComponentEvent e) {
		    resize_grid();
		}
	    };
    }
    
    private WindowAdapter make_window_listener() {
	return new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    request_close();
		}

		public void windowClosed(WindowEvent e) {
		    close_all_surface_editors();
		    Main.remove_shell_frame(SurfaceGridFrame.this);
		}
	    };
    }

    private JMenuBar make_menu_bar() {
	JMenuBar mbar = new JMenuBar();

	{
	    JMenu file = new JMenu(GT.tr("File"));
	    file.setMnemonic(KeyEvent.VK_F);
	    file.addMenuListener(
		new MenuListener() {
		    public void menuSelected(MenuEvent e) {
			mi_save_shell.setEnabled(dirty_p());
			
			// 實體が定まってゐなければ "Save..." に。
			if (shell.shell_directory() == null) {
			    mi_save_shell.setText(GT.tr("Save")+"...");
			}
			else {
			    mi_save_shell.setText(GT.tr("Save"));
			}
		    }

		    public void menuDeselected(MenuEvent e) {}
		    public void menuCanceled(MenuEvent e) {}
		});
	    mbar.add(file);

	    JMenuItem new_shell = new JMenuItem(GT.tr("New Shell"));
	    new_shell.setAccelerator(KeyStroke.getKeyStroke("control alt N"));
	    new_shell.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			// 空のシェルフレームを作る
			Main.add_shell_frame(new SurfaceGridFrame());
		    }
		});
	    file.add(new_shell);

	    JMenuItem open_shell = new JMenuItem(GT.tr("Open Shell..."));
	    open_shell.setAccelerator(KeyStroke.getKeyStroke("control O"));
	    open_shell.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			JFileChooser chooser = new JFileChooser();
			chooser.setFileSelectionMode(
			    JFileChooser.DIRECTORIES_ONLY);
			chooser.setDialogTitle(
			    GT.tr("Select a directory of shell to open..."));

			// 前囘撰擇されたディレクトリをカレントに指定
			String dirpath = Main.ini().getProperty(
			    "koropeta.gui.lastChosenDir", null);
			if (dirpath != null) {
			    File dir = new File(dirpath);
			    if (dir.isDirectory()) {
				chooser.setCurrentDirectory(dir);
			    }
			}

			int retval = chooser.showOpenDialog(SurfaceGridFrame.this);
			if (retval != JFileChooser.APPROVE_OPTION) {
			    return;
			}

			Main.ini().setProperty(
			    "koropeta.gui.lastChosenDir",
			    chooser.getSelectedFile().getParent());
			Main.save_ini();

			// このシェルが既に開かれてゐるなら、そのシェルのフレームを
			// toFront()して終了。
			for (Iterator ite = Main.shell_frames().iterator(); ite.hasNext(); ) {
			    SurfaceGridFrame sgf = (SurfaceGridFrame)ite.next();
			    File dir = sgf.shell().shell_directory();

			    if (dir != null && dir.equals(chooser.getSelectedFile())) {
				sgf.toFront();
				return;
			    }
			}
			
			if (!dirty_p() && shell.shell_directory() == null) {
			    // このフレームを乘っ取る。
			    close_all_surface_editors();
			    shell = new Shell(chooser.getSelectedFile());
			    update_title();
			    reload_grid();
			}
			else {
			    // 新たにフレームを作る。
			    Main.add_shell_frame(
				new SurfaceGridFrame(chooser.getSelectedFile()));
			}
		    }
		});
	    file.add(open_shell);

	    // FIXME: Open Recentも追加

	    file.addSeparator();

	    JMenuItem close_shell = new JMenuItem(GT.tr("Close"));
	    close_shell.setAccelerator(KeyStroke.getKeyStroke("control W"));
	    close_shell.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			request_close();
		    }
		});
	    file.add(close_shell);

	    JMenuItem save_shell = new JMenuItem(GT.tr("Save"));
	    save_shell.setAccelerator(KeyStroke.getKeyStroke("control S"));
	    save_shell.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			save_shell();
		    }
		});
	    file.add(save_shell);
	    mi_save_shell = save_shell;

	    // FIXME: Save As... も要る？

	    file.addSeparator();

	    JMenuItem quit = new JMenuItem(GT.tr("Quit"));
	    quit.setAccelerator(KeyStroke.getKeyStroke("control Q"));
	    quit.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			for (Iterator ite = Main.shell_frames().iterator(); ite.hasNext(); ) {
			    SurfaceGridFrame sgf = (SurfaceGridFrame)ite.next();
			    sgf.request_close();
			}
		    }
		});
	    file.add(quit);
	}

	{
	    JMenu mshell = new JMenu(GT.tr("Shell"));
	    mshell.setMnemonic(KeyEvent.VK_S);
	    mbar.add(mshell);

	    JMenuItem shell_info = new JMenuItem(GT.tr("Shell Information..."));
	    shell_info.setAccelerator(KeyStroke.getKeyStroke("control I"));
	    shell_info.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			if (shell_info_frame != null) {
			    shell_info_frame.setVisible(true);
			    shell_info_frame.toFront();
			}
			else {
			    shell_info_frame = new ShellInformationFrame(
				SurfaceGridFrame.this);
			}
		    }
		});
	    mshell.add(shell_info);

	    JMenuItem new_surface = new JMenuItem(GT.tr("New Surface..."));
	    new_surface.setAccelerator(KeyStroke.getKeyStroke("control N"));
	    new_surface.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			int id = new SurfaceIDChooserDialog(
			    SurfaceGridFrame.this).choose();
			if (id != -1) {
			    shell.add_surface(id);
			    reload_grid();
			}
		    }
		});
	    mshell.add(new_surface);
	}

	{
	    JMenu others = new JMenu(GT.tr("Miscellaneous"));
	    others.setMnemonic(KeyEvent.VK_O);
	    mbar.add(others);

	    JMenuItem pref = new JMenuItem(GT.tr("Preferences..."));
	    pref.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			PreferencesFrame.display();
		    }
		});
	    others.add(pref);

	    others.addSeparator();

	    JMenuItem about = new JMenuItem(GT.tr("About Koropeta..."));
	    about.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			AboutFrame.display();
		    }
		});
	    others.add(about);
	}

	return mbar;
    }

    private boolean save_shell() {
	// 保存されたらtrueを返す。
	
	if (shell.shell_directory() == null) {
	    JFileChooser chooser = new JFileChooser();
	    
	    String dirpath = Main.ini().getProperty(
		"koropeta.gui.lastChosenDir", null);
	    if (dirpath != null) {
		File dir = new File(dirpath);
		if (dir.isDirectory()) {
		    chooser.setCurrentDirectory(dir);
		}
	    }
	    
	    int retval = chooser.showSaveDialog(
		SurfaceGridFrame.this);
	    if (retval != JFileChooser.APPROVE_OPTION) {
		return false;
	    }
	    
	    if (chooser.getSelectedFile().exists()) {
		retval = JOptionPane.showConfirmDialog(
		    SurfaceGridFrame.this,
		    GT.tr(
			"The chosen file or directory \"{0}\" already exists.\n"+
			"Do you want to overwrite it?",
			new Object[] {chooser.getSelectedFile().getName()}),
		    GT.tr("Already exists..."),
		    JOptionPane.YES_NO_OPTION);
		if (retval != JOptionPane.YES_OPTION) {
		    return false;
		}
	    }
	    
	    Main.ini().setProperty(
		"koropeta.gui.lastChosenDir",
		chooser.getSelectedFile().getParent());
	    
	    shell.shell_directory(chooser.getSelectedFile());
	}
	
	shell.save();
	return true;
    }
}