package koropeta.gui;
import koropeta.*;
import koropeta.core.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class SurfaceIDChooserDialog extends JDialog {
    SurfaceGridFrame grid_frame;

    int chosen_id;
    
    public SurfaceIDChooserDialog(SurfaceGridFrame grid_frame) {
        super(
            grid_frame,
            GT.tr("Choose surface ID..."),
            true);
	
        this.grid_frame = grid_frame;

        // デフォルトのサーフィス番號を求める。
        final Map surfaces = grid_frame.get_shell().get_surfaces();
        int unused_id = 0;
        for (;; unused_id++) {
            if (!surfaces.containsKey(new Integer(unused_id))) {
                break;
            }
        }
        chosen_id = unused_id;

        getContentPane().setLayout(new BorderLayout(10, 10));
        getContentPane().add(
            new JLabel(GT.tr("Enter an ID for the new surface:")+"          "),
            BorderLayout.NORTH);

        /* buttons */
        JPanel buttons = new JPanel();
        buttons.setLayout(new FlowLayout(FlowLayout.RIGHT));
        getContentPane().add(buttons, BorderLayout.SOUTH);
	
        JButton btn_cancel = new JButton(GT.tr("Cancel"));
        btn_cancel.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    chosen_id = -1;
                    dispose();
                }
            });
        buttons.add(btn_cancel);
	
        final JButton btn_ok = new JButton(GT.tr("OK"));
        btn_ok.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    dispose();
                }
            });
        buttons.add(btn_ok);
        getRootPane().setDefaultButton(btn_ok);

        /* field and error */
        JPanel field_and_error = new JPanel();
        field_and_error.setLayout(new GridLayout(2, 1, 3, 3));
        getContentPane().add(field_and_error, BorderLayout.CENTER);
	
        final JTextField fld_id = new JTextField(Integer.toString(unused_id));
        field_and_error.add(fld_id);

        final JLabel lbl_error = new JLabel("");
        lbl_error.setForeground(Color.RED);
        field_and_error.add(lbl_error);
	
        fld_id.getDocument().addDocumentListener(
            new DocumentListener() {
                public void insertUpdate(DocumentEvent e) {check();}
                public void removeUpdate(DocumentEvent e) {check();}
                public void changedUpdate(DocumentEvent e) {check();}

                private void check() {
                    String error = invalid_id_p();
                    if (error == null) {
                        btn_ok.setEnabled(true);
                        lbl_error.setText("");
                    }
                    else {
                        btn_ok.setEnabled(false);
                        lbl_error.setText(error);
                    }
                }

                private String invalid_id_p() {
                    String id_str = fld_id.getText();

                    if (id_str.length() == 0) {
                        return "";
                    }

                    int id;
                    try {
                        id = Integer.parseInt(id_str);
                        if (id < 0) {
                            throw new Exception();
                        }
                    }
                    catch (Exception e) {
                        return GT.tr("Enter a positive number or zero.");
                    }

                    if (surfaces.containsKey(new Integer(id))) {
                        return GT.tr("That ID is already used.");
                    }

                    chosen_id = id;
                    return null;
                }
            });
	

        pack();
    }

    public int choose() {
        show();
        return chosen_id;
    }
}