package koropeta.gui.dnd;
import koropeta.gui.dnd.*;
import java.awt.datatransfer.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

public class TableSelectionTransferHandler extends TransferHandler {
    protected Transferable createTransferable(JComponent c) {
	if (!(c instanceof JTable)) {
	    throw new RuntimeException("Not a JTable: "+c.toString());
	}

	return new TableSelectionTransferable((JTable)c);
    }

    public int getSourceActions(JComponent c) {
        return MOVE;
    }
    
    public boolean importData(JComponent c, Transferable t) {
        if (canImport(c, t.getTransferDataFlavors())) {
            try {
                TableSelectionData selection =
		    (TableSelectionData)t.getTransferData(
			TableSelectionTransferable.flavor());

		if (c.hashCode() != selection.table_hashcode()) {
		    return false;
		}

                return true;
            }
	    catch (UnsupportedFlavorException ufe) {
		// do nothing
            }
	    catch (IOException ioe) {
		throw new RuntimeException(ioe);
            }
        }

        return false;
    }

    protected void exportDone(JComponent c, Transferable t, int action) {
	if (action != MOVE) {
	    return;
	}
	
	try {
	    TableSelectionData selection =
		(TableSelectionData)t.getTransferData(
		    TableSelectionTransferable.flavor());
	    
	    if (c.hashCode() != selection.table_hashcode()) {
		// おそらく不要だが、安全の爲
		return;
	    }

	    JTable table = (JTable)c;

	    // ドラッグされた項目群のうち最も上部にあるものが
	    // ドロップされた位置に來るやうにする。
	    final int dest = table.getSelectedRow();
	    if (dest == -1) {
		return;
	    }

	    int first_src = -1;
	    for (Iterator ite = selection.iterator(); ite.hasNext(); ) {
		int src = ((Integer)ite.next()).intValue();

		if (first_src == -1) {
		    first_src = src;
		}

		System.out.println("Sorry, drag & drop is not supported yet.");
		System.out.println(src+" ==> "+(src - first_src + dest));
	    }
	}
	catch (UnsupportedFlavorException ufe) {
	    // do nothing
	}
	catch (IOException ioe) {
	    throw new RuntimeException(ioe);
	}
    }
    
    public boolean canImport(JComponent c, DataFlavor[] flavors) {
        for (int i = 0; i < flavors.length; i++) {
            if (TableSelectionTransferable.flavor().equals(flavors[i])) {
                return true;
            }
        }
        return false;
    }
}