package koropeta.gui.surfaceeditor;
import koropeta.*;
import koropeta.gui.*;
import koropeta.gui.surfaceeditor.*;
import koropeta.gui.dnd.*;
import koropeta.core.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.*;
import java.util.regex.*;
import java.io.*;

public class BaseSurfacePanel extends JPanel {
    Surface surface;
    LayerHandleComponent lhc;
    JTable layers;
    LayeredImageTableModel layers_model;

    public BaseSurfacePanel(Surface s) {
	surface = s;

	SpringLayout sl = new SpringLayout();
	setLayout(sl);

	JButton descr = new JButton(GT.tr("About this screen..."));
	descr.addActionListener(
	    new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    JOptionPane.showMessageDialog(
			BaseSurfacePanel.this,
			GT.tr(
			    "In this screen, you can compose a surface from multiple image\n"+
			    "files. Differs from element-definition, a single png file\n"+
			    "(and a pna file) will be generated statically. If you already\n"+
			    "have a complete surface image, you have nothing to do here.")+
			"\n\n"+
			GT.tr(
			    "The \"Add layer...\" button does what it says, and you can\n"+
			    "remove/duplicate/reorder layers by right-clicking on the layer\n"+
			    "table (not the image). If you select a layer from the table,\n"+
			    "you can move its position by dragging on the image, or by\n"+
			    "pressing allow keys after setting focus on the image. Shift\n"+
			    "key makes the movement faster."),
			GT.tr("About this screen..."),
			JOptionPane.INFORMATION_MESSAGE);
		}
	    });
	add(descr);
	sl.putConstraint(
	    SpringLayout.NORTH, descr,
	    3,
	    SpringLayout.NORTH, this);
	sl.putConstraint(
	    SpringLayout.WEST, descr,
	    3,
	    SpringLayout.WEST, this);
	
	lhc = new LayerHandleComponent(surface.get_base_surface());
	lhc.setFocusable(true);

	// descrとlhcとの距離: 最低 3 px, 最大 ∞
	add(lhc);
	sl.putConstraint(
	    SpringLayout.NORTH, lhc,
	    Spring.constant(3, 3, Integer.MAX_VALUE),
	    SpringLayout.SOUTH, descr);
	sl.putConstraint(
	    SpringLayout.WEST, lhc,
	    3,
	    SpringLayout.WEST, this);
	sl.putConstraint(
	    SpringLayout.SOUTH, this,
	    3,
	    SpringLayout.SOUTH, lhc);

	JPanel buttons = new JPanel();
	{
	    buttons.setLayout(new FlowLayout(FlowLayout.LEADING, 2, 2));
	    
	    JButton add = new JButton(GT.tr("Add layer..."));
	    add.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			add_layer();
		    }
		});
	    buttons.add(add);
	}
	add(buttons);
	sl.putConstraint(
	    SpringLayout.NORTH, buttons,
	    3,
	    SpringLayout.SOUTH, descr);
	sl.putConstraint(
	    SpringLayout.WEST, buttons,
	    3,
	    SpringLayout.EAST, lhc);
	    
	JScrollPane scr = new JScrollPane(
	    JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
	    JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	{
	    scr.setPreferredSize(
		new Dimension(300, 300));

	    layers_model =
		new LayeredImageTableModel(surface.get_base_surface());
	    layers = new JTable(layers_model);
	    layers.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
	    layers.setRowHeight(64);
	    layers.setDragEnabled(true);
	    layers.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    layers.getSelectionModel().addListSelectionListener(
		new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
			int sel = layers.getSelectedRow();
			lhc.active(lhc.num_layers() - sel - 1);
		    }
		});
	    layers.setTransferHandler(new TableSelectionTransferHandler());
	    {
		TableColumn vis = layers.getColumn(GT.tr("Visible"));

		vis.setCellEditor(new DefaultCellEditor(new JCheckBox()));
		vis.setCellRenderer(new JCheckBoxTableCellRenderer());
		
		vis.setMinWidth(50);
		vis.setMaxWidth(50);
	    }
	    {
		TableColumn thumb = layers.getColumn(GT.tr("Thumbnail"));
		thumb.setMinWidth(64);
		thumb.setMaxWidth(64);
	    }

	    final JPopupMenu table_pmenu = new JPopupMenu();
	    {
		final Point context_menu_pt = new Point();
		
		final JMenuItem up = new JMenuItem(GT.tr("Move Up"));
		up.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    int sel = lhc.num_layers() -
				layers.rowAtPoint(context_menu_pt) - 1;
			    lhc.image().swap(sel, sel+1);
			    lhc.image().dirty(true);
			}
		    });
		table_pmenu.add(up);
		
		final JMenuItem down = new JMenuItem(GT.tr("Move Down"));
		down.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    int sel = lhc.num_layers() -
				layers.rowAtPoint(context_menu_pt) - 1;
			    lhc.image().swap(sel, sel-1);
			    lhc.image().dirty(true);
			}
		    });
		table_pmenu.add(down);

		table_pmenu.addSeparator();

		final JMenuItem dup = new JMenuItem(GT.tr("Duplicate"));
		dup.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    int sel = lhc.num_layers() -
				layers.rowAtPoint(context_menu_pt) - 1;
			    
			    LayeredImage.Layer clone =
				(LayeredImage.Layer)lhc.image().layer(sel).clone();
			    lhc.image().insert(sel + 1, clone);
			    lhc.image().dirty(true);
			}
		    });
		table_pmenu.add(dup);

		final JMenuItem rm = new JMenuItem(GT.tr("Remove"));
		rm.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    int sel = lhc.num_layers() -
				layers.rowAtPoint(context_menu_pt) - 1;
			    lhc.image().remove(sel);
			    lhc.image().dirty(true);
			}
		    });
		table_pmenu.add(rm);

		layers.addMouseListener(
		    new MouseAdapter() {
			public void mousePressed(MouseEvent e) { popup(e); }
			public void mouseReleased(MouseEvent e) { popup(e); }
			private void popup(MouseEvent e) {
			    if (e.isPopupTrigger()) {
				e.consume();
				
				context_menu_pt.setLocation(e.getPoint());
				
				int sel = layers.rowAtPoint(e.getPoint());

				up.setEnabled(sel > 0);
				down.setEnabled(sel < layers.getRowCount() - 1);

				table_pmenu.show(
				    layers, e.getX(), e.getY());
			    }
			}
		    });
	    }

	    scr.setViewportView(layers);
	}
	add(scr);
	sl.putConstraint(
	    SpringLayout.WEST, scr,
	    3,
	    SpringLayout.EAST, lhc);
	sl.getConstraints(this).setConstraint(
	    SpringLayout.EAST,
	    Spring.sum(
		Spring.constant(3),
		Spring.max(
		    sl.getConstraint(SpringLayout.EAST, buttons),
		    sl.getConstraint(SpringLayout.EAST, scr))));
	sl.putConstraint(
	    SpringLayout.SOUTH, scr,
	    -3,
	    SpringLayout.SOUTH, this);
	sl.putConstraint(
	    SpringLayout.NORTH, scr,
	    3,
	    SpringLayout.SOUTH, buttons);
    }

    public void close() {
	lhc.close();
	layers_model.close();
    }
    
    public void add_layer() {
	JFileChooser chooser = new JFileChooser();
	PngFileFilter filter = new PngFileFilter();
	chooser.addChoosableFileFilter(filter);
	chooser.setFileFilter(filter);
	chooser.setAcceptAllFileFilterUsed(false);

	// 前囘撰擇されたディレクトリをカレントに指定
	String dirpath = Main.ini().getProperty(
	    "koropeta.gui.lastChosenDir", null);
	if (dirpath != null) {
	    File dir = new File(dirpath);
	    if (dir.isDirectory()) {
		chooser.setCurrentDirectory(dir);
	    }
	}

	int retval = chooser.showOpenDialog(this);
	if (retval == JFileChooser.APPROVE_OPTION) {
	    Matcher m = Pattern.compile("^(.+?)\\.([^\\.]+)$").matcher(
		chooser.getSelectedFile().getName());
	    if (!m.find()) {
		throw new RuntimeException();
	    }

	    Main.ini().setProperty(
		"koropeta.gui.lastChosenDir",
		chooser.getSelectedFile().getParent());
	    Main.save_ini();

	    // レイヤ名の重複を避ける
	    LayeredImage li = lhc.image();
	    synchronized (li) {
		String layer_name = m.group(0);
		
		if (li.exists(layer_name)) {
		    for (int i = 1;; i++) {
			String new_name = m.group(1)+"-"+i+"."+m.group(2);
			if (!li.exists(new_name)) {
			    layer_name = new_name;
			    break;
			}
		    }
		}

		try {
		    BufferedImage bi = AlphaSplittedImage.read(
			chooser.getSelectedFile().getParentFile(),
			m.group(1));

		    Dimension size = li.size();
		    if ((size.width != 0 && size.height != 0) &&
			(size.width < bi.getWidth() ||
			 size.height < bi.getHeight())) {
			// これが2枚目以上のレイヤであり、且つ
			// 現在の畫像サイズよりも大きい

			int ret = JOptionPane.showConfirmDialog(
			    this,
			    GT.tr(
				"The chosen image \"{0}\" is larger than current surface.\n"+
				"Do you want to extend the size of surface to fit the image?",
				new Object[] {chooser.getSelectedFile().getName()}),
			    GT.tr("The image is too large..."),
			    JOptionPane.YES_NO_OPTION);
			if (ret == JOptionPane.YES_OPTION) {
			    Dimension newsize = new Dimension(
				size.width > bi.getWidth() ? size.width : bi.getWidth(),
				size.height > bi.getHeight() ? size.height : bi.getHeight());
			    li.size(newsize);
			}
		    }
		    
		    li.add(layer_name, bi);
		    li.dirty(true);
		}
		catch (Exception e) {
		    e.printStackTrace();
		}
	    }
	}
    }
}