package koropeta.gui.surfaceeditor;
import koropeta.*;
import koropeta.gui.*;
import koropeta.core.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.awt.color.*;

public class CollisionHandleComponent extends LayeredImageComponent {
    public static final boolean FLAG_ANIMATE = false;
    private static final int BOX_SIZE = 10;
    
    public static final int CORNER_TOPLEFT = 0;
    public static final int CORNER_TOPRIGHT = 1;
    public static final int CORNER_BOTTOMLEFT = 2;
    public static final int CORNER_BOTTOMRIGHT = 3;

    Set active_colls; // <Collision>; 現在アクティブな衝突判定の集合。
    int active_corner; // 現在アクティブな隅。-1ならアクティブ無し。
    
    Collisions collisions;
    Thread flasher;

    JPopupMenu  context_menu;
    Collision[] context_menu_targets;
    ShowContextMenu show_context_menu;

    Set active_listeners; // <ActiveCollisionsListener>

    public CollisionHandleComponent(LayeredImage image, final Collisions collisions) {
        super(image);
        this.collisions = collisions;

        setFocusable(true);

        MouseInputListener mil = new MouseInputAdapter() {
                public void mousePressed(MouseEvent e) {
                    CollisionHandleComponent.this.mousePressed(e);
                }

                public void mouseDragged(MouseEvent e) {
                    CollisionHandleComponent.this.mouseDragged(e);
                }
            };
        addMouseListener(mil);
        addMouseMotionListener(mil);

        addKeyListener(
            new KeyAdapter() {
                public void keyPressed(KeyEvent ev) {
                    CollisionHandleComponent.this.keyPressed(ev);
                }
            });

        active_colls = Collections.synchronizedSet(new HashSet());
        active_corner = -1;

        if (FLAG_ANIMATE) {
            flasher = new FlasherThread();
            flasher.start();
        }

        active_listeners = Collections.synchronizedSet(new HashSet());

        /* Cut / Copy / Duplicate / Remove は、もし今囘の直接の對象（つ
         * まり右クリック時にカーソルの指してゐたもの）が撰擇中の項目で
         * あったなら、撰擇中の全ての項目を對象とする。
         */
        context_menu = new JPopupMenu();
        {
            final JMenuItem cut = new JMenuItem(GT.tr("Cut"));
            cut.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        CollisionSelection sel = new CollisionSelection(context_menu_targets);

                        Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
                        clip.setContents(sel, sel);
			    
                        collisions.remove(context_menu_targets);
                        collisions.dirty(true);
                    }
                });
            context_menu.add(cut);

            final JMenuItem copy = new JMenuItem(GT.tr("Copy"));
            copy.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        CollisionSelection sel = new CollisionSelection(context_menu_targets);

                        Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
                        clip.setContents(sel, sel);
                    }
                });
            context_menu.add(copy);

            final JMenuItem paste = new JMenuItem(GT.tr("Paste"));
            paste.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
                        Transferable trans = clip.getContents(this);
                        if (trans != null &&
                            trans.isDataFlavorSupported(Collision.data_flavor())) {
                            try {
                                Collision[] targets
                                    = (Collision[])trans.getTransferData(Collision.data_flavor());
                                    
                                collisions.add(targets);
                                collisions.dirty(true);
                            }
                            catch (Exception ex) {
                                throw new RuntimeException(ex);
                            }
                        }
                    }
                });
            context_menu.add(paste);

            final JMenuItem dup = new JMenuItem(GT.tr("Duplicate"));
            dup.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        Collision[] clone = new Collision[context_menu_targets.length];
                        for (int i = 0; i < context_menu_targets.length; i++) {
                            clone[i] = (Collision)context_menu_targets[i].clone();
                        }

                        collisions.add(clone);
                        collisions.dirty(true);
                    }
                });
            context_menu.add(dup);

            final JMenuItem rm = new JMenuItem(GT.tr("Remove"));
            rm.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        collisions.remove(context_menu_targets);
                        collisions.dirty(true);
                    }
                });
            context_menu.add(rm);

            final JMenuItem sel_all = new JMenuItem(GT.tr("Select All"));
            sel_all.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        activateAll();
                    }
                });
            context_menu.add(sel_all);

            show_context_menu
                = new ShowContextMenu() {
                        public void show(MouseEvent e) {
                            final Collision direct_target = collision_at_point(e.getPoint());

                            if (direct_target == null) {
                                context_menu_targets = new Collision[] {};
                            }
                            else {
                                if (active_colls.contains(direct_target)) {
                                    Object[] src = active_colls.toArray();
                                    context_menu_targets = new Collision[ src.length ];

                                    System.arraycopy(
                                        src, 0,
                                        context_menu_targets, 0,
                                        src.length
                                        );
                                }
                                else {
                                    context_menu_targets = new Collision[] { direct_target };
                                }
                            }

                            final boolean any_targets = context_menu_targets.length > 0;

                            cut    .setEnabled(any_targets);
                            copy   .setEnabled(any_targets);
                            dup    .setEnabled(any_targets);
                            rm     .setEnabled(any_targets);
                            sel_all.setEnabled(collisions.size() > 0);

                            Clipboard    clip  = Toolkit.getDefaultToolkit().getSystemClipboard();
                            Transferable trans = clip.getContents(this);
                            paste.setEnabled(
                                trans != null &&
                                trans.isDataFlavorSupported(Collision.data_flavor()));
        
                            context_menu.show(
                                CollisionHandleComponent.this, e.getX(), e.getY());
                        }
                    };
        }
    }

    public final void addActiveCollisionsListener(ActiveCollisionsListener lsnr) {
        active_listeners.add(lsnr);
    }

    public final void removeActiveCollisionsListener(ActiveCollisionsListener lsnr) {
        active_listeners.remove(lsnr);
    }

    // 與へられた一つの當たり判定のみアクティブにする。
    public final void activate(int coll_idx) {
        activate(collisions.get(coll_idx));
    }
    public final void activate(Collision coll) {
        Set indices = new HashSet();

        indices.add(coll);

        activate(indices);
    }

    public final void activateAlso(int coll_idx) {
        activateAlso(collisions.get(coll_idx));
    }
    public final void activateAlso(Collision coll) {
        synchronized (active_colls) {
            Set indices = new HashSet(active_colls);

            indices.add(coll);

            activate(indices);
        }
    }

    public final void activateAll() {
        Set indices = new HashSet();

        for (Iterator ite = collisions.iterator(); ite.hasNext(); ) {
            indices.add(ite.next());
        }

        activate(indices);
    }

    public final void activate(int[] coll_indices) {
        Set indices = new HashSet();

        synchronized (coll_indices) {
            synchronized (collisions) {
                for (int i = 0; i < coll_indices.length; i++) {
                    final int coll_idx = coll_indices[i];

                    indices.add(collisions.get(coll_idx));
                }
            }
        }

        activate(indices);
    }

    public final void deactivateAll() {
        activate(new HashSet());
    }
                                   
    public final void activate(Set coll_indices) {
        assert coll_indices != null;
        
        synchronized (coll_indices) {
            synchronized (active_colls) {
                if (!coll_indices.equals(active_colls)) {

                    active_colls.clear();
                    active_colls.addAll(coll_indices);
                    active_corner = -1;
                    repaint();

                    notify_change_of_active_collisions();
                }
            }
        }
    }

    public final void notify_change_of_active_collisions() {
        final Set set = Collections.unmodifiableSet(active_colls);
        
        for (Iterator ite = active_listeners.iterator(); ite.hasNext(); ) {
            ActiveCollisionsListener lsnr = (ActiveCollisionsListener)ite.next();

            lsnr.activeCollisionsChanged(set);
        }
    }

    public final Collisions collisions() {
        return collisions;
    }

    /* [Shift 時]
     * - 何も無いところで press →何も起きない
     * - 特定の當たり判定上で press →それをアクティブ集合へ追加するだけ
     *
     * [非 Shift 時]
     * - 何も無いところで press →全て非アクティブに
     * - 非アクティブな當たり判定上で press →それのみをアクティブに
     * - アクティブな當たり判定上で press →corner撰擇のみ行ふ
     */

    private Point pt_begin;
    protected void mousePressed(MouseEvent e) {
        e.consume();
        
        if (e.isPopupTrigger()) {
            show_context_menu.show(e);
            return;
        }
        
        requestFocusInWindow();

        final Point     pt = pt_begin = e.getPoint();
        final Collision c  = collision_at_point(pt);

        if (e.isShiftDown()) {
            if (c != null) {
                activateAlso(c);
            }
            return;
        }

        if (c == null) {
            deactivateAll();
        }
        else if (!active_colls.contains(c)) {
            activate(c);
        }

        // active_cornerを決定
        if (c == null) {
            active_corner = -1;
        }
        else {
            do {
                assert c != null;

                Rectangle r = c.rect();
		
                // top left
                r.width = BOX_SIZE;
                r.height = BOX_SIZE;
                if (r.contains(pt)) {
                    active_corner = CORNER_TOPLEFT;
                    break;
                }

                // top right
                r.x += c.width() - BOX_SIZE;
                if (r.contains(pt)) {
                    active_corner = CORNER_TOPRIGHT;
                    break;
                }

                // bottom right
                r.y += c.height() - BOX_SIZE;
                if (r.contains(pt)) {
                    active_corner = CORNER_BOTTOMRIGHT;
                    break;
                }

                // bottom left
                r.x = c.x();
                if (r.contains(pt)) {
                    active_corner = CORNER_BOTTOMLEFT;
                    break;
                }

                active_corner = -1;
            } while (false);
        }

        repaint();
    }

    private void move_collision(int dx, int dy) {
        if (dx == 0 && dy == 0) {
            return;
        }
        
        for (Iterator ite = active_colls.iterator(); ite.hasNext(); ) {
            final Collision c = (Collision)ite.next();
	
            switch (active_corner) {
            case CORNER_TOPLEFT:
                c.x(c.x() + dx);
                c.y(c.y() + dy);
                c.width(c.width() - dx);
                c.height(c.height() - dy);
                break;

            case CORNER_TOPRIGHT:
                // c.x is unchanged
                c.y(c.y() + dy);
                c.width(c.width() + dx);
                c.height(c.height() - dy);
                break;

            case CORNER_BOTTOMRIGHT:
                // c.x is unchanged
                // c.y is unchanged
                c.width(c.width() + dx);
                c.height(c.height() + dy);
                break;

            case CORNER_BOTTOMLEFT:
                c.x(c.x() + dx);
                // c.y is unchanged
                c.width(c.width() - dx);
                c.height(c.height() + dy);
                break;

            default:
                c.x(c.x() + dx);
                c.y(c.y() + dy);
            }

            collisions.changed(c);
        }

        collisions.dirty(true);
        repaint();
    }

    protected void mouseDragged(MouseEvent e) {
        if (e.isShiftDown()
            || e.isPopupTrigger()
            || active_colls.isEmpty()) {
            
            return;
        }

        Point pt = e.getPoint();

        int dx = pt.x - pt_begin.x;
        int dy = pt.y - pt_begin.y;
        move_collision(dx, dy);

        pt_begin = pt;
    }

    protected void keyPressed(KeyEvent ev) {
        if (active_colls.isEmpty()) {
            return;
        }

        int dx = 0, dy = 0;
        switch (ev.getKeyCode()) {
	    case KeyEvent.VK_LEFT:
            dx = -1;
            break;
	    
        case KeyEvent.VK_RIGHT:
            dx = 1;
            break;
	    
        case KeyEvent.VK_UP:
            dy = -1;
            break;
	    
        case KeyEvent.VK_DOWN:
            dy = 1;
            break;
        }

        if (ev.isShiftDown()) {
            dx *= 10;
            dy *= 10;
        }

        move_collision(dx, dy);
    }

    public void close() {
        if (flasher != null) {
            flasher.interrupt();
        }
        super.close();
    }

    public Collision collision_at_point(Point pt) {
        for (int i = 0; i < collisions.size(); i++) {
            final Collision coll = collisions.get(i);

            if (coll.rect().contains(pt)) {
                return coll;
            }
        }
        return null;
    }

    static final Stroke solid = new BasicStroke(
        1.0f, // width
        BasicStroke.CAP_BUTT,
        BasicStroke.JOIN_BEVEL,
        1.0f, // miterlimit
        null,
        0.0f);
    
    static final Stroke dotted_p1 = new BasicStroke(
        1.0f, // width
        BasicStroke.CAP_BUTT,
        BasicStroke.JOIN_BEVEL,
        1.0f, // miterlimit
        new float[] {3.0f, 3.0f},
        0.0f); // dash_phase

    static final Stroke dotted_p2 = new BasicStroke(
        1.0f, // width
        BasicStroke.CAP_BUTT,
        BasicStroke.JOIN_BEVEL,
        1.0f, // miterlimit
        new float[] {3.0f, 3.0f},
        3.0f); // dash_phase

    private boolean flip;
    private Rectangle rect;
    private Line2D line;
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        final Graphics2D g2 = (Graphics2D)g.create();

        if (rect == null) {
            rect = new Rectangle();
            line = new Line2D.Float();
        }

        for (int i = 0; i < collisions.size(); i++) {
            final Collision c         = collisions.get(i);
            final boolean   is_active = active_colls.contains(c);

            // --- id --- //
            g2.setPaint(Color.WHITE);
            g2.drawString(
                c.id(),
                c.x() + BOX_SIZE + 4,
                c.y() + BOX_SIZE + 1);

            g2.setPaint(is_active ? Color.RED : Color.BLACK);
            g2.drawString(
                c.id(),
                c.x() + BOX_SIZE + 3,
                c.y() + BOX_SIZE);

            // --- pattern 1 --- //
            g2.setStroke(solid);
            g2.setPaint(Color.WHITE);

            // frame
            rect.x = c.x();
            rect.y = c.y();
            rect.width = c.width();
            rect.height = c.height();
            g2.draw(rect);

            // top left
            line.setLine(
                c.x()           , c.y() + BOX_SIZE,
                c.x() + BOX_SIZE, c.y() + BOX_SIZE);
            g2.draw(line);
            line.setLine(
                c.x() + BOX_SIZE, c.y() + BOX_SIZE,
                c.x() + BOX_SIZE, c.y()           );
            g2.draw(line);

            // top right
            line.setLine(
                c.x() + c.width() - BOX_SIZE, c.y()           ,
                c.x() + c.width() - BOX_SIZE, c.y() + BOX_SIZE);
            g2.draw(line);
            line.setLine(
                c.x() + c.width() - BOX_SIZE, c.y() + BOX_SIZE,
                c.x() + c.width()           , c.y() + BOX_SIZE);
            g2.draw(line);

            // bottom right
            line.setLine(
                c.x() + c.width()           , c.y() + c.height() - BOX_SIZE,
                c.x() + c.width() - BOX_SIZE, c.y() + c.height() - BOX_SIZE);
            g2.draw(line);
            line.setLine(
                c.x() + c.width() - BOX_SIZE, c.y() + c.height() - BOX_SIZE,
                c.x() + c.width() - BOX_SIZE, c.y() + c.height());
            g2.draw(line);

            // bottom left
            line.setLine(
                c.x()           , c.y() + c.height() - BOX_SIZE,
                c.x() + BOX_SIZE, c.y() + c.height() - BOX_SIZE);
            g2.draw(line);
            line.setLine(
                c.x() + BOX_SIZE, c.y() + c.height() - BOX_SIZE,
                c.x() + BOX_SIZE, c.y() + c.height()           );
            g2.draw(line);

            // --- pattern 2 --- //
            g2.setStroke(flip ? dotted_p1 : dotted_p2);
            g2.setPaint(is_active ? Color.RED : Color.BLACK);

            // frame
            rect.x = c.x();
            rect.y = c.y();
            rect.width = c.width();
            rect.height = c.height();
            g2.draw(rect);

            // top left
            if (is_active) {
                g2.setStroke(
                    active_corner == CORNER_TOPLEFT ?
                    solid : (flip ? dotted_p1 : dotted_p2));
                g2.setPaint(
                    active_corner == CORNER_TOPLEFT ?
                    Color.BLUE : Color.RED);
            }
            else {
                g2.setStroke(flip ? dotted_p1 : dotted_p2);
            }
            line.setLine(
                c.x()           , c.y() + BOX_SIZE,
                c.x() + BOX_SIZE, c.y() + BOX_SIZE);
            g2.draw(line);
            line.setLine(
                c.x() + BOX_SIZE, c.y() + BOX_SIZE,
                c.x() + BOX_SIZE, c.y()           );
            g2.draw(line);

            // top right
            if (is_active) {
                g2.setStroke(
                    active_corner == CORNER_TOPRIGHT ?
                    solid : (flip ? dotted_p1 : dotted_p2));
                g2.setPaint(
                    active_corner == CORNER_TOPRIGHT ?
                    Color.BLUE : Color.RED);
            }
            line.setLine(
                c.x() + c.width() - BOX_SIZE, c.y()           ,
                c.x() + c.width() - BOX_SIZE, c.y() + BOX_SIZE);
            g2.draw(line);
            line.setLine(
                c.x() + c.width() - BOX_SIZE, c.y() + BOX_SIZE,
                c.x() + c.width()           , c.y() + BOX_SIZE);
            g2.draw(line);

            // bottom right
            if (is_active) {
                g2.setStroke(
                    active_corner == CORNER_BOTTOMRIGHT ?
                    solid : (flip ? dotted_p1 : dotted_p2));
                g2.setPaint(
                    active_corner == CORNER_BOTTOMRIGHT ?
                    Color.BLUE : Color.RED);
            }
            line.setLine(
                c.x() + c.width()           , c.y() + c.height() - BOX_SIZE,
                c.x() + c.width() - BOX_SIZE, c.y() + c.height() - BOX_SIZE);
            g2.draw(line);
            line.setLine(
                c.x() + c.width() - BOX_SIZE, c.y() + c.height() - BOX_SIZE,
                c.x() + c.width() - BOX_SIZE, c.y() + c.height());
            g2.draw(line);

            // bottom left
            if (is_active) {
                g2.setStroke(
                    active_corner == CORNER_BOTTOMLEFT ?
                    solid : (flip ? dotted_p1 : dotted_p2));
                g2.setPaint(
                    active_corner == CORNER_BOTTOMLEFT ?
                    Color.BLUE : Color.RED);
            }
            line.setLine(
                c.x()           , c.y() + c.height() - BOX_SIZE,
                c.x() + BOX_SIZE, c.y() + c.height() - BOX_SIZE);
            g2.draw(line);
            line.setLine(
                c.x() + BOX_SIZE, c.y() + c.height() - BOX_SIZE,
                c.x() + BOX_SIZE, c.y() + c.height()           );
            g2.draw(line);
        }

        if (FLAG_ANIMATE) {
            flip = !flip;
        }
    }

    private class FlasherThread extends Thread {
        volatile boolean flag_term;

        public FlasherThread() {
            setPriority(Thread.MIN_PRIORITY);
	    
            flag_term = false;
        }
	
        public void run() {
            while (!flag_term) {
                CollisionHandleComponent.this.repaint();
		
                try {
                    sleep(500);
                }
                catch (InterruptedException e) {}
            }
        }

        public void terminate() {
            flag_term = true;
            interrupt();
            while (isAlive()) {
                try {
                    join();
                }
                catch (InterruptedException e) {}
            }
        }
    }

    public interface ActiveCollisionsListener {
        public void activeCollisionsChanged(Set collisions); // <Collision>
    }

    private interface ShowContextMenu {
        public void show(MouseEvent e);
    }
}