package koropeta.gui.surfaceeditor;
import koropeta.*;
import koropeta.gui.*;
import koropeta.gui.surfaceeditor.*;
import koropeta.core.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.regex.*;
import java.io.*;

public class CollisionPanel extends JPanel {
    Surface surface;
    CollisionHandleComponent chc;
    JTable collisions;
    CollisionTableModel collisions_model;

    boolean ignore_list_selection_event;

    ListDataListener collisions_lsnr;
    CollisionHandleComponent.ActiveCollisionsListener active_colls_lsnr;

    public CollisionPanel(Surface s) {
        surface = s;

        SpringLayout sl = new SpringLayout();
        setLayout(sl);

        JButton descr = new JButton(GT.tr("About this screen..."));
        descr.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JOptionPane.showMessageDialog(
                        CollisionPanel.this,
                        GT.tr(
                            "In this screen, you can define regions of collision.")+
                        "\n\n"+
                        GT.tr(
                            "You can copy/paste/remove/duplicate/reorder collisions by right-\n"+
                            "clicking on either the image or collision table.\n"+
                            "\n"+
                            "You can also select multiple collisions by Shift + Left click.\n"+
                            "\n"+
                            "A selected collision can be resized by dragging its handles at\n"+
                            "corner, and be moved by dragging the other place."),
                        GT.tr("About this screen..."),
                        JOptionPane.INFORMATION_MESSAGE);
                }
            });
        add(descr);
        sl.putConstraint(
            SpringLayout.NORTH, descr,
            3,
            SpringLayout.NORTH, this);
        sl.putConstraint(
            SpringLayout.WEST, descr,
            3,
            SpringLayout.WEST, this);

        chc = new CollisionHandleComponent(surface.image(), surface.collisions());
        chc.setFocusable(true);

        // descrとchcとの距離: 最低 3 px, 最大 ∞
        add(chc);
        sl.putConstraint(
            SpringLayout.NORTH, chc,
            Spring.constant(3, 3, Integer.MAX_VALUE),
            SpringLayout.SOUTH, descr);
        sl.putConstraint(
            SpringLayout.WEST, chc,
            3,
            SpringLayout.WEST, this);
        sl.putConstraint(
            SpringLayout.SOUTH, this,
            3,
            SpringLayout.SOUTH, chc);
	
        JPanel buttons = new JPanel();
        {
            buttons.setLayout(new FlowLayout(FlowLayout.LEADING, 2, 2));

            JButton add = new JButton(GT.tr("Add collision..."));
            add.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        add_collision();
                    }
                });
            buttons.add(add);
        }
        add(buttons);
        sl.putConstraint(
            SpringLayout.NORTH, buttons,
            3,
            SpringLayout.SOUTH, descr);
        sl.putConstraint(
            SpringLayout.WEST, buttons,
            3,
            SpringLayout.EAST, chc);

        final JScrollPane scr = new JScrollPane(
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        {
            scr.setPreferredSize(
                new Dimension(300, 300));

            collisions_model =
                new CollisionTableModel(surface.collisions());
            collisions = new JTable(collisions_model);
            collisions.setDragEnabled(false);
            collisions.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
            collisions.getSelectionModel().addListSelectionListener(
                new ListSelectionListener() {
                    public void valueChanged(ListSelectionEvent e) {
                        if (!ignore_list_selection_event) {
                            chc.activate(collisions.getSelectedRows());
                        }
                    }
                });

            final JPopupMenu table_pmenu = new JPopupMenu();
            {
                final Point context_menu_pt = new Point();

                // Move Up / Move Down は、複數撰擇時には選べない。

                final JMenuItem up = new JMenuItem(GT.tr("Move Up"));
                up.addActionListener(
                    new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int sel = collisions.rowAtPoint(context_menu_pt);
                            chc.collisions().swap(sel, sel-1);
                            chc.collisions().dirty(true);
                            
                            collisions.setRowSelectionInterval(sel-1, sel-1);
                        }
                    });
                table_pmenu.add(up);

                final JMenuItem down = new JMenuItem(GT.tr("Move Down"));
                down.addActionListener(
                    new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int sel = collisions.rowAtPoint(context_menu_pt);
                            chc.collisions().swap(sel, sel+1);
                            chc.collisions().dirty(true);
                            
                            collisions.setRowSelectionInterval(sel+1, sel+1);
                        }
                    });
                table_pmenu.add(down);

                /* Cut / Copy / Duplicate / Remove は、もし今囘
                 * の直接の對象（つまり右クリック時にカーソルの指してゐ
                 * たもの）が撰擇中の項目であったなら、撰擇中の全ての項
                 * 目を對象とする。
                 */
                table_pmenu.addSeparator();

                final JMenuItem cut = new JMenuItem(GT.tr("Cut"));
                cut.addActionListener(
                    new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            Collision[]    targets = get_targets(context_menu_pt);
                            CollisionSelection sel = new CollisionSelection(targets);
			    
                            Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
                            clip.setContents(sel, sel);
			    
                            chc.collisions().remove(targets);
                            chc.collisions().dirty(true);
                        }
                    });
                table_pmenu.add(cut);

                final JMenuItem copy = new JMenuItem(GT.tr("Copy"));
                copy.addActionListener(
                    new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            Collision[]    targets = get_targets(context_menu_pt);
                            CollisionSelection sel = new CollisionSelection(targets);
			    
                            Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
                            clip.setContents(sel, sel);
                        }
                    });
                table_pmenu.add(copy);

                final JMenuItem paste = new JMenuItem(GT.tr("Paste"));
                paste.addActionListener(
                    new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int sel = collisions.rowAtPoint(context_menu_pt);

                            Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
                            Transferable trans = clip.getContents(this);
                            if (trans != null &&
                                trans.isDataFlavorSupported(Collision.data_flavor())) {
                                try {
                                    Collision[] targets
                                        = (Collision[])trans.getTransferData(Collision.data_flavor());
                                    
                                    chc.collisions().insert(sel + 1, targets);
                                    chc.collisions().dirty(true);
                                }
                                catch (Exception ex) {
                                    throw new RuntimeException(ex);
                                }
                            }
                        }
                    });
                table_pmenu.add(paste);

                final JMenuItem dup = new JMenuItem(
                    /* Duplicate as verb; command in a context menu */
                    GT.tr("Duplicate")
                    );
                dup.addActionListener(
                    new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            Collision[] targets = get_targets(context_menu_pt);

                            Collision[] clone = new Collision[targets.length];
                            for (int i = 0; i < targets.length; i++) {
                                clone[i] = (Collision)targets[i].clone();
                            }

                            int dest = get_duplicate_destination(context_menu_pt);
			    
                            chc.collisions().insert(dest, clone);
                            chc.collisions().dirty(true);
                        }
                    });
                table_pmenu.add(dup);

                final JMenuItem rm = new JMenuItem(GT.tr("Remove"));
                rm.addActionListener(
                    new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            Collision[]  targets = get_targets(context_menu_pt);
			    
                            chc.collisions().remove(targets);
                            chc.collisions().dirty(true);
                        }
                    });
                table_pmenu.add(rm);

                final JMenuItem sel_all = new JMenuItem(GT.tr("Select All"));
                sel_all.addActionListener(
                    new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            collisions.selectAll();
                        }
                    });
                table_pmenu.add(sel_all);

                collisions.addMouseListener(
                    new MouseAdapter() {
                        public void mousePressed(MouseEvent e) {
                            popup(e);
                        }

                        public void mouseReleased(MouseEvent e) {
                            popup(e);
                        }

                        public void popup(MouseEvent e) {
                            if (e.isPopupTrigger()) {
                                e.consume();

                                context_menu_pt.setLocation(e.getPoint());

                                int     sel         = collisions.rowAtPoint(e.getPoint());
                                int     n_rows      = collisions.getRowCount();
                                int     n_selected  = collisions.getSelectedRowCount();
                                boolean is_selected = collisions.isRowSelected(sel);

                                up  .setEnabled(!(n_selected >= 2 && is_selected)
                                                && sel > 0);
                                down.setEnabled(!(n_selected >= 2 && is_selected)
                                                && sel < n_rows - 1);

                                Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
                                Transferable trans = clip.getContents(this);
                                paste.setEnabled(
                                    trans != null &&
                                    trans.isDataFlavorSupported(Collision.data_flavor()));

                                table_pmenu.show(
                                    collisions, e.getX(), e.getY());
                            }
                        }
                    });
            }

            final JPopupMenu scr_pmenu = new JPopupMenu();
            {
                final JMenuItem paste = new JMenuItem(GT.tr("Paste"));
                paste.addActionListener(
                    new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
                            Transferable trans = clip.getContents(this);
                            if (trans != null &&
                                trans.isDataFlavorSupported(Collision.data_flavor())) {
                                try {
                                    Collision[] colls
                                        = (Collision[])trans.getTransferData(Collision.data_flavor());
                                    chc.collisions().add(colls);
                                    chc.collisions().dirty(true);
                                }
                                catch (Exception ex) {
                                    throw new RuntimeException(ex);
                                }
                            }
                        }
                    });
                scr_pmenu.add(paste);
		
                scr.addMouseListener(
                    new MouseAdapter() {
                        public void mousePressed(MouseEvent e) {
                            popup(e);
                        }

                        public void mouseReleased(MouseEvent e) {
                            popup(e);
                        }

                        public void popup(MouseEvent e) {
                            if (e.isPopupTrigger()) {
                                e.consume();

                                Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
                                Transferable trans = clip.getContents(this);
                                paste.setEnabled(
                                    trans != null &&
                                    trans.isDataFlavorSupported(Collision.data_flavor()));

                                scr_pmenu.show(
                                    scr, e.getX(), e.getY());
                            }
                        }
                    });
            }

            scr.setViewportView(collisions);
        }
        add(scr);
        sl.putConstraint(
            SpringLayout.WEST, scr,
            3,
            SpringLayout.EAST, chc);
        sl.getConstraints(this).setConstraint(
            SpringLayout.EAST,
            Spring.sum(
                Spring.constant(3),
                Spring.max(
                    sl.getConstraint(SpringLayout.EAST, buttons),
                    sl.getConstraint(SpringLayout.EAST, scr))));
        sl.putConstraint(
            SpringLayout.SOUTH, scr,
            -3,
            SpringLayout.SOUTH, this);
        sl.putConstraint(
            SpringLayout.NORTH, scr,
            3,
            SpringLayout.SOUTH, buttons);


        collisions_lsnr
            = new ListDataListener() {
                    public void contentsChanged(ListDataEvent e) { chc.repaint(); }
                    public void intervalAdded  (ListDataEvent e) { chc.repaint(); }
                    public void intervalRemoved(ListDataEvent e) { chc.repaint(); }
                };
        chc.collisions().addListDataListener(collisions_lsnr);

        ignore_list_selection_event = false;
        active_colls_lsnr
            = new CollisionHandleComponent.ActiveCollisionsListener() {
                    public void activeCollisionsChanged(Set colls) {
                        try {
                            ignore_list_selection_event = true;
                            collisions.clearSelection();

                            for (Iterator ite = colls.iterator(); ite.hasNext(); ) {
                                Collision coll = (Collision)ite.next();

                                int row = chc.collisions().indexOf(coll);
                                if (row != -1) {
                                    collisions.addRowSelectionInterval(row, row);
                                }
                            }
                        }
                        finally {
                            ignore_list_selection_event = false;
                        }
                    }
                };
        chc.addActiveCollisionsListener(active_colls_lsnr);
    }

    private Collision[] get_targets(Point context_menu_pt) {
        final int   sel = collisions.rowAtPoint(context_menu_pt);
        Collision[] targets;
        
        if (collisions.isRowSelected(sel)) {
            // int[] -> targets[] のマッピングに何
            // 故こんなに文字数を割かなければならな
            // いのか。
            int[] selected = collisions.getSelectedRows();
                                
            targets = new Collision[selected.length];
                                
            for (int i = 0; i < selected.length; i++) {
                targets[i] = (Collision)chc.collisions().get(selected[i]);
            }
        }
        else {
            targets = new Collision[] {
                (Collision)chc.collisions().get(sel),
            };
        }

        return targets;
    }

    private int get_duplicate_destination(Point context_menu_pt) {
        final int sel = collisions.rowAtPoint(context_menu_pt);

        if (collisions.isRowSelected(sel)) {
            // 一連の撰擇範圍の最後に插入する。
            final int[] selected = collisions.getSelectedRows();
            int last_idx = selected[0];

            for (int i = 0; i < selected.length; i++) {
                if (selected[i] > last_idx) {
                    last_idx = selected[i];
                }
            }

            return last_idx + 1;
        }
        else {
            return sel + 1;
        }
    }

    public void close() {
        chc.collisions().removeListDataListener(collisions_lsnr);
        chc.removeActiveCollisionsListener(active_colls_lsnr);
        chc.close();
        collisions_model.close();
    }

    public void add_collision() {
        String id = JOptionPane.showInputDialog(
            GT.tr("Enter an identifier (name) of the new collision:"));

        if (id == null) {
            return;
        }
	
        Collision c = new Collision(
            new Rectangle(0, 0, 100, 100), id);

        surface.collisions().add(c);
        surface.collisions().dirty(true);
    }
}