package koropeta.gui.surfaceeditor;
import koropeta.core.*;
import java.util.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

public class CollisionTableModel implements TableModel {
    Collisions collisions;
    Vector listeners;

    ListDataListener collision_change_listener;

    /*
      +------------+------+-----+-------+--------+
      | Identifier | Left | Top | Width | Height |
      +============+======+=====+=======+========+
      | Head       | 100  | 100 | 50    | 50     |
      +------------+------+-----+-------+--------+
      | Body       | 50   | 50  | 100   | 100    |
      +------------+------+-----+-------+--------+
    */

    public CollisionTableModel(Collisions coll) {
        collisions = coll;
        listeners = new Vector();

        collision_change_listener = new ListDataListener() {
                public void intervalAdded(ListDataEvent e) {
                    changed(TableModelEvent.INSERT, e);
                }

                public void intervalRemoved(ListDataEvent e) {
                    changed(TableModelEvent.DELETE, e);
                }

                public void contentsChanged(ListDataEvent e) {
                    changed(TableModelEvent.UPDATE, e);
                }

                protected void changed(int type, ListDataEvent e) {
                    int idx0 = e.getIndex0();
                    int idx1 = e.getIndex1();
		    
                    TableModelEvent ev = new TableModelEvent(
                        CollisionTableModel.this,
                        idx0,
                        idx1,
                        TableModelEvent.ALL_COLUMNS,
                        type);

                    for (Iterator ite = listeners.iterator(); ite.hasNext();) {
                        TableModelListener l = (TableModelListener)ite.next();
                        l.tableChanged(ev);
                    }
                }
            };
        collisions.addListDataListener(collision_change_listener);
    }

    public void close() {
        collisions.removeListDataListener(collision_change_listener);
    }

    public void addTableModelListener(TableModelListener l) {
        listeners.add(l);
    }

    public Class getColumnClass(int columnIndex) {
        switch (columnIndex) {
        case 0:
            return String.class;

        case 1:
        case 2:
        case 3:
        case 4:
            return Integer.class;

        default:
            throw new RuntimeException("Invalid column index: "+columnIndex);
        }
    }

    public int getColumnCount() {
        return 5;
    }

    public String getColumnName(int columnIndex) {
        switch (columnIndex) {
        case 0:
            return GT.tr("Collision ID");

        case 1:
            return GT.tr(
                /* Left as coordinate */
                "Left"
                );
            
        case 2:
            return GT.tr(
                /* Top as coordinate */
                "Top"
                );

        case 3:
            return GT.tr("Width");

        case 4:
            return GT.tr("Height");

        default:
            throw new RuntimeException("Invalid column index: "+columnIndex);
        }
    }

    public final int getRowCount() {
        return collisions.size();
    }

    public Object getValueAt(final int rowIndex, final int columnIndex) {
        Collision coll = collisions.get(rowIndex);

        switch (columnIndex) {
        case 0:
            return coll.id();

        case 1:
            return new Integer(coll.x());

        case 2:
            return new Integer(coll.y());

        case 3:
            return new Integer(coll.width());

        case 4:
            return new Integer(coll.height());

        default:
            throw new RuntimeException("Invalid column index: "+columnIndex);
        }
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return columnIndex >= 0 && columnIndex < 5;
    }

    public void removeTableModelListener(TableModelListener l) {
        listeners.remove(l);
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        Collision coll = collisions.get(rowIndex);

        switch (columnIndex) {
        case 0:
            coll.id((String)aValue);
            break;

        case 1:
            do {
                int val = ((Integer)aValue).intValue();
                if (val >= 0) {
                    coll.x(val);
                }
            } while (false);
            break;

        case 2:
            do {
                int val = ((Integer)aValue).intValue();
                if (val >= 0) {
                    coll.y(val);
                }
            } while (false);
            break;

        case 3:
            do {
                int val = ((Integer)aValue).intValue();
                if (val >= 0) {
                    coll.width(val);
                }
            } while (false);
            break;

        case 4:
            do {
                int val = ((Integer)aValue).intValue();
                if (val >= 0) {
                    coll.width(val);
                }
            } while (false);
            break;

        default:
            throw new RuntimeException("Not editable column: "+columnIndex);
        }

        collisions.dirty(true);
        collisions.changed(coll);
    }
}