package koropeta.gui.surfaceeditor;
import koropeta.*;
import koropeta.gui.*;
import koropeta.gui.surfaceeditor.*;
import koropeta.gui.dnd.*;
import koropeta.core.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.*;
import java.util.regex.*;
import java.io.*;

public class ElementPanel extends JPanel {
    SurfaceEditorFrame editor;
    Surface surface;
    LayerHandleComponent lhc;
    JTable layers;
    LayeredImageTableModel layers_model;

    public ElementPanel(SurfaceEditorFrame sef) {
	editor = sef;
	surface = sef.surface();

	SpringLayout sl = new SpringLayout();
	setLayout(sl);

	JButton descr = new JButton(GT.tr("About this screen..."));
	descr.addActionListener(
	    new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    JOptionPane.showMessageDialog(
			ElementPanel.this,
			GT.tr(
			    "In this screen, you can define elements with image files.")+
			"\n\n"+
			GT.tr(
			    "The \"Add element...\" button does what it says, and you can\n"+
			    "remove/duplicate/reorder elements by right-clicking on the layer\n"+
			    "table (not the image). If you select an element from the table,\n"+
			    "you can move its position by dragging on the image, or by\n"+
			    "pressing allow keys after setting focus on the image. Shift\n"+
			    "key makes the movement faster."),
			GT.tr("About this screen..."),
			JOptionPane.INFORMATION_MESSAGE);
		}
	    });
	add(descr);
	sl.putConstraint(
	    SpringLayout.NORTH, descr,
	    3,
	    SpringLayout.NORTH, this);
	sl.putConstraint(
	    SpringLayout.WEST, descr,
	    3,
	    SpringLayout.WEST, this);

	lhc = new LayerHandleComponent(surface.get_elements());
	lhc.setFocusable(true);

	// descrとlhcとの距離: 最低 3 px, 最大 ∞
	add(lhc);
	sl.putConstraint(
	    SpringLayout.NORTH, lhc,
	    Spring.constant(3, 3, Integer.MAX_VALUE),
	    SpringLayout.SOUTH, descr);
	sl.putConstraint(
	    SpringLayout.WEST, lhc,
	    3,
	    SpringLayout.WEST, this);
	sl.putConstraint(
	    SpringLayout.SOUTH, this,
	    3,
	    SpringLayout.SOUTH, lhc);

	JPanel buttons = new JPanel();
	{
	    buttons.setLayout(new FlowLayout(FlowLayout.LEADING, 2, 2));

	    JButton add = new JButton(GT.tr("Add element..."));
	    add.addActionListener(
		new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			add_element();
		    }
		});
	    buttons.add(add);
	}
	add(buttons);
	sl.putConstraint(
	    SpringLayout.NORTH, buttons,
	    3,
	    SpringLayout.SOUTH, descr);
	sl.putConstraint(
	    SpringLayout.WEST, buttons,
	    3,
	    SpringLayout.EAST, lhc);
	
	JScrollPane scr = new JScrollPane(
	    JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
	    JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	{
	    scr.setPreferredSize(
		new Dimension(300, 300));

	    layers_model =
		new LayeredImageTableModel(surface.get_elements());
	    layers = new JTable(layers_model);
	    layers.setRowHeight(64);
	    layers.setDragEnabled(true);
	    layers.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    layers.getSelectionModel().addListSelectionListener(
		new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
			int sel = layers.getSelectedRow();
			lhc.active(lhc.num_layers() - sel - 1);
		    }
		});
	    layers.setTransferHandler(new TableSelectionTransferHandler());
	    {
		TableColumn vis = layers.getColumn(GT.tr("Visible"));

		vis.setCellEditor(new DefaultCellEditor(new JCheckBox()));
		vis.setCellRenderer(new JCheckBoxTableCellRenderer());
		
		vis.setMinWidth(50);
		vis.setMaxWidth(50);
	    }
	    {
		TableColumn thumb = layers.getColumn(GT.tr("Thumbnail"));
		thumb.setMinWidth(64);
		thumb.setMaxWidth(64);
	    }

	    final JPopupMenu table_pmenu = new JPopupMenu();
	    {
		final Point context_menu_pt = new Point();
		
		final JMenuItem up = new JMenuItem(GT.tr("Move Up"));
		up.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    int sel = lhc.num_layers() -
				layers.rowAtPoint(context_menu_pt) - 1;
			    lhc.image().swap(sel, sel+1);
			    lhc.image().dirty(true);
			}
		    });
		table_pmenu.add(up);

		final JMenuItem down = new JMenuItem(GT.tr("Move Down"));
		down.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    int sel = lhc.num_layers() -
				layers.rowAtPoint(context_menu_pt) - 1;
			    lhc.image().swap(sel, sel-1);
			    lhc.image().dirty(true);
			}
		    });
		table_pmenu.add(down);

		table_pmenu.addSeparator();

		final JMenuItem dup = new JMenuItem(GT.tr("Duplicate"));
		dup.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    int sel = lhc.num_layers() -
				layers.rowAtPoint(context_menu_pt) - 1;
			    
			    LayeredImage.Layer clone =
				(LayeredImage.Layer)lhc.image().layer(sel).clone();
			    lhc.image().insert(sel + 1, clone);
			    lhc.image().dirty(true);
			}
		    });
		table_pmenu.add(dup);

		final JMenuItem rm = new JMenuItem(GT.tr("Remove"));
		rm.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    int sel = lhc.num_layers() -
				layers.rowAtPoint(context_menu_pt) - 1;
			    lhc.image().remove(sel);
			    lhc.image().dirty(true);
			}
		    });
		table_pmenu.add(rm);

		layers.addMouseListener(
		    new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
			    popup(e);
			}
			
			public void mouseReleased(MouseEvent e) {
			    popup(e);
			}

			private void popup(MouseEvent e) {
			    if (e.isPopupTrigger()) {
				e.consume();
				
				context_menu_pt.setLocation(e.getPoint());
				
				int sel = layers.rowAtPoint(e.getPoint());

				up.setEnabled(sel > 0);
				down.setEnabled(sel < layers.getRowCount() - 1);

				table_pmenu.show(
				    layers, e.getX(), e.getY());
			    }
			}
		    });
	    }

	    scr.setViewportView(layers);
	}
	add(scr);
	sl.putConstraint(
	    SpringLayout.WEST, scr,
	    3,
	    SpringLayout.EAST, lhc);
	sl.getConstraints(this).setConstraint(
	    SpringLayout.EAST,
	    Spring.sum(
		Spring.constant(3),
		Spring.max(
		    sl.getConstraint(SpringLayout.EAST, buttons),
		    sl.getConstraint(SpringLayout.EAST, scr))));
	sl.putConstraint(
	    SpringLayout.SOUTH, scr,
	    -3,
	    SpringLayout.SOUTH, this);
	sl.putConstraint(
	    SpringLayout.NORTH, scr,
	    3,
	    SpringLayout.SOUTH, buttons);
    }

    public void close() {
	lhc.close();
	layers_model.close();
    }

    public void add_element() {
	if (surface.dir() == null || !surface.dir().exists()) {
	    JOptionPane.showMessageDialog(
		this,
		GT.tr("Sorry, you can add elements only after saving the shell once."),
		GT.tr("This shell has never been saved before..."),
		JOptionPane.ERROR_MESSAGE);
	    return;
	}
	
	JFileChooser chooser = new JFileChooser();
	chooser.setFileFilter(new PngFileFilter());
	chooser.setAcceptAllFileFilterUsed(false);

	// 前囘撰擇されたディレクトリをカレントに指定
	String dirpath = Main.ini().getProperty(
	    "koropeta.gui.lastChosenDir", null);
	if (dirpath != null) {
	    File dir = new File(dirpath);
	    if (dir.isDirectory()) {
		chooser.setCurrentDirectory(dir);
	    }
	}

	int retval = chooser.showOpenDialog(this);
	if (retval == JFileChooser.APPROVE_OPTION) {
	    File chosen = chooser.getSelectedFile();
	    
	    Matcher m = Pattern.compile("^(.+?)\\.([^\\.]+)$").matcher(
		chosen.getName());
	    if (!m.find()) {
		throw new RuntimeException();
	    }

	    Main.ini().setProperty(
		"koropeta.gui.lastChosenDir",
		chosen.getParent());
	    Main.save_ini();

	    /*
	      1. 撰擇された畫像が當シェル内の畫像であれば、そのまま登録。
	      2. さうでなければ、
	        a. 同名のファイルが當該シェル内にあるなら、それを上書きして良いかどうか確認。
	        b. surface0001.png のやうに當該シェルによって上書きされる可能性のある場合、
		   A. 同じIDを持つ畫像ファイルが當シェル内に存在するなら、上書きして良いかどうか確認。
		   B. さうでないなら、上書きの可能性を警告した上で、當シェル内に畫像をコピーして登録。
		c. サーフィスとは無關係のファイルであれば、そのまま當シェル内にコピーして登録。
	    */
	    if (chosen.getParentFile().equals(
		    surface.dir().getAbsoluteFile())) {
		// 特別な處理は居らない。
	    }
	    else {
		File dest = new File(surface.dir(), chosen.getName());
		
		if (dest.exists()) {
		    // 警告
		    int ret = JOptionPane.showConfirmDialog(
			this,
			GT.tr(
			    "The chosen image \"{0}\" is not owned by this shell, so it has to\n"+
			    "be copied into the shell.",
			    new Object[] {chosen.getName()})+
			" "+
			GT.tr(
			    "But it has a file having the same name \"{0}\".",
			    new Object[] {chosen.getName()})+
			"\n"+
			GT.tr("Is it OK to overwrite it?"),
			GT.tr("Already exists..."),
			JOptionPane.YES_NO_OPTION);

		    if (ret != JOptionPane.YES_OPTION) {
			return;
		    }
		}
		else {
		    Pattern re_surface = Pattern.compile(
			"^surface(\\d+)\\.(?:png|krps)$", Pattern.CASE_INSENSITIVE);
		
		    Matcher ma = re_surface.matcher(chosen.getName());
		    if (ma.find()) {
			int id = -1;
			try {
			    id = Integer.parseInt(ma.group(1));
			}
			catch (NumberFormatException e) {}
			
			// 同じIDのファイルが存在しているか？
			String same_id = editor.grid_frame().shell().find_surface_having_this_id(id);
			if (same_id != null) {
			    // 上書きを警告
			    int ret = JOptionPane.showConfirmDialog(
				this,
				GT.tr(
				    "The chosen image \"{0}\" is not owned by this shell, so it has to\n"+
				    "be copied into the shell.",
				    new Object[] {chosen.getName()})+
				" "+
				GT.tr(
				    "But surface {0} is already in it.",
				    new Object[] {same_id})+
				"\n"+
				GT.tr("Is it OK to overwrite it?"),
				GT.tr("Already exists..."),
				JOptionPane.YES_NO_OPTION);
			    
			    if (ret != JOptionPane.YES_OPTION) {
				return;
			    }
			    
			    // コピー先のファイルをこのファイルにする。
			    dest = new File(surface.dir(), "surface"+same_id+".png");
			}
			else {
			    // 上書きの可能性を警告
			    int ret = JOptionPane.showConfirmDialog(
				this,
				GT.tr(
				    "The chosen image \"{0}\" is not owned by this shell, so it has to\n"+
				    "be copied into the shell.",
				    new Object[] {chosen.getName()})+
				" "+
				GT.tr(
				    "But \"{0}\" can be overwritten when you\n"+
				    "make or modify surface {1} in this shell. Is it OK?",
				    new Object[] {
					chosen.getName(),
					ma.group(1)}),
				GT.tr("The image is not owned, but..."),
				JOptionPane.YES_NO_OPTION);
			
			    if (ret != JOptionPane.YES_OPTION) {
				return;
			    }
			}
		    }
		}

		// ファイルを複製
		try {
		    Main.cp(chosen, dest);
		}
		catch (Exception e) {
		    throw new RuntimeException(
			"Failed to copy file: "+chosen+" to "+dest, e);
		}
	    }

	    // 登録
	    try {
		LayeredImage.Layer l = surface.get_elements().add(
		    chosen.getName(),
		    AlphaSplittedImage.read(surface.dir(), m.group(1)));
	    }
	    catch (Exception e) {
		throw new RuntimeException(
		    "Failed to load image: "+chosen, e);
	    }
	}
    }
}
