package koropeta.gui.surfaceeditor;
import koropeta.*;
import koropeta.gui.*;
import koropeta.core.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.color.*;

public class LayerHandleComponent extends LayeredImageComponent {
    public static final int NOTHING_IS_ACTIVE = -1; // アクティブ無し。
    public static final int OFFSET_IS_ACTIVE = -2; // オフセットが移動對象。
    int active; // 現在アクティブなレイヤ番號。

    public LayerHandleComponent(LayeredImage image) {
	super(image);
	this.active = -1;

	setFocusable(true);

	MouseInputListener mil = new MouseInputAdapter() {
		public void mousePressed(MouseEvent e) {
		    LayerHandleComponent.this.mousePressed(e);
		}

		public void mouseDragged(MouseEvent e) {
		    LayerHandleComponent.this.mouseDragged(e);
		}
	    };
	addMouseListener(mil);
	addMouseMotionListener(mil);

	addKeyListener(
	    new KeyAdapter() {
		public void keyPressed(KeyEvent ev) {
		    LayerHandleComponent.this.keyPressed(ev);
		}
	    });
    }

    public final int active(String layer) {
	return active(image.indexOf(layer));
    }
    public final int active(int layer) {
	active = layer;
	return layer;
    }
    public final int active() {
	return active;
    }

    protected void move_layer(LayeredImage.Layer l, int x, int y) {
	l.x(x);
	l.y(y);
	image.dirty(true);
    }

    protected void move_offset(int x, int y) {
	offset(x, y);
    }

    protected void keyPressed(KeyEvent ev) {
	if (active == NOTHING_IS_ACTIVE) {
	    return;
	}
	
	LayeredImage.Layer l = null;
	if (active != OFFSET_IS_ACTIVE) {
	    l = image.layer(active);
	    if (l == null) {
		return;
	    }
	}
	
	int dx = 0, dy = 0;
	
	switch (ev.getKeyCode()) {
	case KeyEvent.VK_LEFT:
	    dx = -1;
	    break;
	    
	case KeyEvent.VK_RIGHT:
	    dx = 1;
	    break;
	    
	case KeyEvent.VK_UP:
	    dy = -1;
	    break;
	    
	case KeyEvent.VK_DOWN:
	    dy = 1;
	    break;
	}

	if (ev.isShiftDown()) {
	    dx *= 10;
	    dy *= 10;
	}

	if (dx != 0 || dy != 0) {
	    if (active == OFFSET_IS_ACTIVE) {
		move_offset(offset_x + dx, offset_y + dy);
	    }
	    else {
		move_layer(l, l.x() + dx, l.y() + dy);
	    }
	}
    }

    // コンポーネント系に於けるドラッグ開始位置
    Point initial_drag_pt_component;
    // 畫像系に於けるドラッグ開始位置
    Point initial_drag_pt_image;
    protected void mousePressed(MouseEvent e) {
	requestFocusInWindow();

	if (active == NOTHING_IS_ACTIVE) {
	    return;
	}

	initial_drag_pt_component = e.getPoint();

	if (active == OFFSET_IS_ACTIVE) {
	    initial_drag_pt_image = new Point(offset_x, offset_y);
	}
	else {
	    LayeredImage.Layer l = image.layer(active);
	    if (l == null) {
		return;
	    }
	    initial_drag_pt_image = new Point(l.x(), l.y());
	}
    }

    protected void mouseDragged(MouseEvent e) {
	if (active == NOTHING_IS_ACTIVE) {
	    return;
	}

	Point cur = e.getPoint(); // current

	if (active == OFFSET_IS_ACTIVE) {
	    move_offset(
		initial_drag_pt_image.x + (
		    cur.x - initial_drag_pt_component.x),
		initial_drag_pt_image.y + (
		    cur.y - initial_drag_pt_component.y));
	}
	else {
	    LayeredImage.Layer l = image.layer(active);
	    if (l == null) {
		return;
	    }
	    
	    move_layer(
		l,
		initial_drag_pt_image.x + (
		    cur.x - initial_drag_pt_component.x),
		initial_drag_pt_image.y + (
		    cur.y - initial_drag_pt_component.y));
	}
    }
}