package koropeta.gui.surfaceeditor;
import koropeta.core.*;
import java.util.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

public class LayeredImageTableModel implements TableModel {
    LayeredImage image;
    Vector listeners;

    ListDataListener image_change_listener;

    /*
      +------------------------------------+
      | 可視 | ｻﾑﾈｲﾙ | ファイル         |
      +-------+--------+-------------------+
      |       | +----+ |                   |
      | [x]   | |####| | foo.png           |
      |       | +----+ |                   |
      +-------+--------+-------------------+
      |       | +----+ |                   |
      | [ ]   | |%%%%| | bar.png           |
      |       | +----+ |                   |
      +-------+--------+-------------------+
    */

    public LayeredImageTableModel(LayeredImage img) {
	image = img;
	listeners = new Vector();

	image_change_listener = new ListDataListener() {
		public void intervalAdded(ListDataEvent e) {
		    changed(TableModelEvent.INSERT, e);
		}

		public void intervalRemoved(ListDataEvent e) {
		    changed(TableModelEvent.DELETE, e);
		}

		public void contentsChanged(ListDataEvent e) {
		    changed(TableModelEvent.UPDATE, e);
		}

		protected void changed(int type, ListDataEvent e) {
		    // このモデル内では順序が反轉してゐる事に注意
		    int idx0 = image.num_layers() - e.getIndex0() - 1;
		    int idx1 = image.num_layers() - e.getIndex1() - 1;
		    
		    TableModelEvent ev = new TableModelEvent(
			LayeredImageTableModel.this,
			idx0,
			idx1,
			TableModelEvent.ALL_COLUMNS,
			type);

		    for (Iterator ite = listeners.iterator(); ite.hasNext();) {
			TableModelListener l = (TableModelListener)ite.next();
			l.tableChanged(ev);
		    }
		}
	    };
	image.addListDataListener(image_change_listener);
    }

    public void close() {
	image.removeListDataListener(image_change_listener);
    }

    public void addTableModelListener(TableModelListener l) {
	listeners.add(l);
    }

    public Class getColumnClass(int columnIndex) {
	switch (columnIndex) {
	case 0:
	    return Boolean.class;

	case 1:
	    return ImageIcon.class;

	case 2:
	    return String.class;

	default:
	    throw new RuntimeException("Invalid column index: "+columnIndex);
	}
    }

    public int getColumnCount() {
	return 3;
    }

    public String getColumnName(int columnIndex) {
	switch (columnIndex) {
	case 0:
	    return GT.tr("Visible");

	case 1:
	    return GT.tr("Thumbnail");

	case 2:
	    return GT.tr("File");

	default:
	    throw new RuntimeException("Invalid column index: "+columnIndex);
	}
    }

    public final int getRowCount() {
	return image.num_layers();
    }

    public Object getValueAt(final int rowIndex, final int columnIndex) {
	LayeredImage.Layer layer =
	    image.layer(image.num_layers() - rowIndex - 1);
	
	switch (columnIndex) {
	case 0:
	    return new Boolean(layer.visible());

	case 1:
	    return layer.thumbnail_icon(64, 64);

	case 2: 
	    return layer.name();

	default:
	    throw new RuntimeException("Invalid column index: "+columnIndex);
	}
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
	return columnIndex == 0;
    }

    public void removeTableModelListener(TableModelListener l) {
	listeners.remove(l);
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
	LayeredImage.Layer layer =
	    image.layer(image.num_layers() - rowIndex - 1);
	
	if (columnIndex == 0) {
	    Boolean val = (Boolean)aValue;
	    layer.visible(val.booleanValue());
	    image.dirty(true);
	}
	else {
	    throw new RuntimeException("Not editable column: "+columnIndex);
	}
    }
}

