package koropeta.gui.surfaceeditor;
import koropeta.*;
import koropeta.gui.*;
import koropeta.gui.surfaceeditor.*;
import koropeta.gui.surfaceeditor.animation.*;
import koropeta.core.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class SurfaceEditorFrame extends JFrame {
    protected static int initial_window_x = 100;
    protected static int initial_window_y = 50;
    
    SurfaceGridFrame grid_frame;
    Surface surface;

    JTabbedPane tab;
    BaseSurfacePanel base_surface_panel;
    ElementPanel element_panel;
    CollisionPanel collision_panel;
    AnimationPanel animation_panel;

    ListDataListener surface_listener;
    
    public SurfaceEditorFrame(
	SurfaceGridFrame grid_frame, Surface surface) {

	super(grid_frame.get_shell_name()+" : surface"+surface.get_id());
	
	this.grid_frame = grid_frame;
	this.surface = surface;

	try {
	    setBounds(
		initial_window_x,
		initial_window_y,
		0, 0);
	}
	catch (Exception e) {
	    e.printStackTrace();

	    setBounds(initial_window_x, initial_window_y, 0, 0);
	}
	initial_window_x += 10;
	initial_window_y += 10;

	addWindowListener(make_window_listener());

	getContentPane().setLayout(new BorderLayout());

	tab = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.WRAP_TAB_LAYOUT);
	base_surface_panel = new BaseSurfacePanel(surface);
	element_panel = new ElementPanel(this);
	collision_panel = new CollisionPanel(surface);
	animation_panel = new AnimationPanel(this);
	tab.addTab(GT.tr("Base Picture"), base_surface_panel);
	tab.addTab(GT.tr("Element"), element_panel);
	tab.addTab(GT.tr("Collision"), collision_panel);
	tab.addTab(GT.tr("Animation"), animation_panel);
	getContentPane().add(tab, BorderLayout.CENTER);
	try {
	    tab.setSelectedIndex(
		Integer.parseInt(
		    Main.ini().getProperty(
			"koropeta.gui.SurfaceEditorFrame.selectedTab", "0")));
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
	tab.addChangeListener(
	    new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
		    Main.ini().setProperty(
			"koropeta.gui.SurfaceEditorFrame.selectedTab",
			Integer.toString(tab.getSelectedIndex()));
		    Main.save_ini();
		}
	    });

	surface_listener = new ListDataListener() {
		public void intervalAdded(ListDataEvent e) {
		    update_tabs();
		}
		public void intervalRemoved(ListDataEvent e) {
		    update_tabs();
		}
		public void contentsChanged(ListDataEvent e) {
		    update_tabs();
		}
	    };
	surface.get_base_surface().addListDataListener(surface_listener);
	surface.get_elements().addListDataListener(surface_listener);

	update_tabs();

	//setResizable(false);
	setVisible(true);
	pack();
    }

    public final SurfaceGridFrame grid_frame() {
	return grid_frame;
    }
    
    public final Surface surface() {
	return surface;
    }

    private WindowAdapter make_window_listener() {
	return new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    surface.get_base_surface().removeListDataListener(surface_listener);
		    surface.get_elements().removeListDataListener(surface_listener);
		    
		    base_surface_panel.close();
		    element_panel.close();
		    collision_panel.close();
		    animation_panel.close();
		    grid_frame.close_surface_editor(surface.get_int_id());
		}
	    };
    }

    protected void update_tabs() {
	// 基本畫像タブの無效/有效
	if (surface.has_elements()) {
	    if (tab.getSelectedIndex() == 0) {
		tab.setSelectedIndex(1);
	    }

	    tab.setEnabledAt(0, false);
	    tab.setToolTipTextAt(
		0,
		GT.tr(
		    "This surface can't have a base picture because it is defined by elements."));
	}
	else {
	    tab.setEnabledAt(0, true);
	    tab.setToolTipTextAt(0, null);
	}
	
	// Elementタブの無效/有效
	if (surface.has_base_surface()) {
	    if (tab.getSelectedIndex() == 1) {
		tab.setSelectedIndex(0);
	    }
	    
	    tab.setEnabledAt(1, false);
	    tab.setToolTipTextAt(
		1,
		GT.tr(
		    "This surface can't have elements because it is defined by the base picture."));
	}
	else {
	    tab.setEnabledAt(1, true);
	    tab.setToolTipTextAt(1, null);
	}
    }
}