package koropeta.gui.surfaceeditor.animation;
import koropeta.core.*;
import koropeta.core.animation.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

public class AnimationGroupsTableModel implements TableModel {
    Animation animation;
    Vector listeners;

    ListDataListener anim_change_listener;

    /*
      +----+-------------------+
      | ID | グループ         |
      +----+-------------------+
      |  0 | sometimes         |
      |  1 | never (exclusive) |
      |  2 | random 1/3        |
      |  3 | talk 5            |
      +----+-------------------+
    */

    public AnimationGroupsTableModel(Animation anim) {
        animation = anim;
        listeners = new Vector();

        anim_change_listener = new ListDataListener() {
                public void intervalAdded(ListDataEvent e) {
                    changed(TableModelEvent.INSERT, e);
                }

                public void intervalRemoved(ListDataEvent e) {
                    changed(TableModelEvent.DELETE, e);
                }

                public void contentsChanged(ListDataEvent e) {
                    changed(TableModelEvent.UPDATE, e);
                }

                protected void changed(int type, ListDataEvent e) {
                    TableModelEvent ev = new TableModelEvent(
                        AnimationGroupsTableModel.this,
                        e.getIndex0(),
                        e.getIndex1(),
                        TableModelEvent.ALL_COLUMNS,
                        type);

                    for (Iterator ite = listeners.iterator(); ite.hasNext();) {
                        TableModelListener l = (TableModelListener)ite.next();
                        l.tableChanged(ev);
                    }
                }
            };
        animation.addListDataListener(anim_change_listener);
    }

    public void close() {
        animation.removeListDataListener(anim_change_listener);
    }

    public int id_of_row(int row) {
        return row < 0 ? -1 : animation.get(row).id();
    }
    
    public void addTableModelListener(TableModelListener l) {
        listeners.add(l);
    }

    public Class getColumnClass(int idx) {
        switch (idx) {
        case 0:
            return Integer.class;

        case 1:
            return String.class;

        default:
            throw new RuntimeException("Invalid column index: "+idx);
        }
    }

    public int getColumnCount() {
        return 2;
    }

    public String getColumnName(int idx) {
        switch (idx) {
        case 0:
            return GT.tr("ID");

        case 1:
            return GT.tr("Group");

        default:
            throw new RuntimeException("Invalid column index: "+idx);
        }
    }

    public final int getRowCount() {
        return animation.size();
    }

    public Object getValueAt(final int row, final int col) {
        switch (col) {
        case 0:
            return new Integer(id_of_row(row));

        case 1:
            AnimGroup group = animation.get(row);
            StringBuffer buf = new StringBuffer();
            buf.append(group.interval());
            if (group.interval().equals("random")) {
                buf.append(" 1/");
                buf.append(group.interval_arg());
            }
            else if (group.interval().equals("talk")) {
                buf.append(' ');
                buf.append(group.interval_arg());
            }
            if (group.option() != null) {
                buf.append(" (");
                buf.append(group.option());
                buf.append(')');
            }
            return buf.toString();

        default:
            throw new RuntimeException("Invalid column index: "+col);
        }
    }

    public boolean isCellEditable(int row, int col) {
        return false;
    }

    public void removeTableModelListener(TableModelListener l) {
        listeners.remove(l);
    }

    public void setValueAt(Object val, int row, int col) {
        throw new RuntimeException("Not editable column: "+col);
    }
}
