package koropeta.gui.surfaceeditor.animation;
import koropeta.*;
import koropeta.gui.*;
import koropeta.gui.surfaceeditor.*;
import koropeta.core.*;
import koropeta.core.animation.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.color.*;

public class AnimationHandleComponent extends LayerHandleComponent {
    Animation animation;
    List pattern_offset_listeners;

    /*
      レイヤ命名規則:
      -- Base Surface: 基本畫像。baseメソッドによって變更される事がある。
      -- Group %d    : overlayその他によって追加されるサーフィス。
    */
    
    public AnimationHandleComponent(Animation anim, LayeredImage base) {
        super(new LayeredImage());

        if (base == null) {
            throw new RuntimeException("Base surface cannot be null");
        }
	
        animation = anim;
        pattern_offset_listeners = new Vector();
	
        image.add("Base Surface", base);

        /*
          padding(50, 50, 50, 50);
          offset(0, 0);
          draw_axes(true);
        */
    }

    public void close() {
        image.clear();
        super.close();
    }

    public Animation animation() {
        return animation;
    }

    public void addPatternOffsetListener(PatternOffsetListener listener) {
        if (!pattern_offset_listeners.contains(listener)) {
            pattern_offset_listeners.add(listener);
        }
    }

    public void removePatternOffsetListener(PatternOffsetListener listener) {
        pattern_offset_listeners.remove(listener);
    }

    public final LayeredImage.Layer base_surface() {
        return image.layer("Base Surface");
    }
    public LayeredImage.Layer base_surface(LayeredImage img) {
        if (img == null) {
            throw new RuntimeException("Base surface cannot be null");
        }
        LayeredImage.Layer l = image.layer("Base Surface");
        ((LayeredImage.ReferenceLayer)l).layers(img);
        return l;
    }

    public final LayeredImage.Layer group_layer(int gid) {
        return image.layer("Group "+gid);
    }
    public final LayeredImage.Layer group_layer(int gid, LayeredImage img, int x, int y) {
        String name = "Group "+gid;
        if (img == null) {
            image.remove(name);
            return null;
        }
	
        LayeredImage.Layer l = image.layer(name);
        if (l == null) {
            // 無いので追加
            l = image.add(name, img, x, y, true);
        }
        else {
            ((LayeredImage.ReferenceLayer)l).layers(img);
            l.x(x);
            l.y(y);
        }
        return l;
    }

    public void move_group_layer(int gid, int x, int y) {
        LayeredImage.Layer l = group_layer(gid);
        l.x(x);
        l.y(y);
    }

    public void remove_group_layers() {
        for (int i = 0; i < image.num_layers(); i++) {
            LayeredImage.Layer l = image.layer(i);
            if (l.name().startsWith("Group ")) {
                image.remove(i);
                i--;
            }
        }
    }

    public void remove_group_layers_except(int gid) {
        for (int i = 0; i < image.num_layers(); i++) {
            LayeredImage.Layer l = image.layer(i);
            if (l.name().startsWith("Group ")) {
                try {
                    int id = Integer.parseInt(
                        l.name().substring(6));
                    if (id != gid) {
                        image.remove(i);
                        i--;
                    }
                }
                catch (NumberFormatException e) {}
            }
        }
    }

    public final void activate_base_surface() {
        active(LayerHandleComponent.OFFSET_IS_ACTIVE);
    }

    public final void activate_group_layer(int gid) {
        active("Group "+gid);
    }

    protected void move_layer(LayeredImage.Layer l, int x, int y) {
        super.move_layer(l, x, y);
	
        PatternOffsetEvent ev = new PatternOffsetEvent(this, x, y);
        for (Iterator ite = pattern_offset_listeners.iterator(); ite.hasNext(); ) {
            PatternOffsetListener listener = (PatternOffsetListener)ite.next();
            listener.offsetChanged(ev);
        }
    }

    protected void move_offset(int x, int y) {
        super.move_offset(x, y);

        PatternOffsetEvent ev = new PatternOffsetEvent(this, x, y);
        for (Iterator ite = pattern_offset_listeners.iterator(); ite.hasNext(); ) {
            PatternOffsetListener listener = (PatternOffsetListener)ite.next();
            listener.offsetChanged(ev);
        }
    }
}
