package koropeta.gui.surfaceeditor.animation;
import koropeta.*;
import koropeta.gui.*;
import koropeta.gui.surfaceeditor.*;
import koropeta.core.*;
import koropeta.core.animation.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.*;
import java.util.regex.*;
import java.io.*;

public class AnimationPanel extends JPanel {
    SurfaceEditorFrame editor;
    Surface surface;
    AnimationHandleComponent ahc;
    Player player;
    
    GroupInfoPanel group_info;
    JToggleButton btn_play_all;
    JTable groups;
    AnimationGroupsTableModel groups_model;

    PatternInfoPanel pattern_info;
    JButton btn_add_pattern;
    JToggleButton btn_play;
    JTable patterns;
    AnimationPatternsTableModel patterns_model;

    public AnimationPanel(SurfaceEditorFrame sef) {
	editor = sef;
	surface = sef.surface();

	SpringLayout sl = new SpringLayout();
	setLayout(sl);

	JButton descr = new JButton(GT.tr("About this screen..."));
	descr.addActionListener(
	    new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    JOptionPane.showMessageDialog(
			AnimationPanel.this,
			GT.tr(
			    "In this screen, you can create a SERIKO animation for a surface.")+
			"\n\n"+
			GT.tr(
			    "The screen has three panes: Surface Pane (at left), Group Pane (at\n"+
			    "top-right), and Pattern Pane (at bottom-right). If you don't know\n"+
			    "about the SERIKO specification, you probably have to read it first.")+
			"\n\n"+
			GT.tr(
			    ":: Group Pane ::\n"+
			    "Buttons:\n"+
			    "\"Add group...\": adds a new group.\n"+
			    "\"Play all\": starts playing of all groups forever. Push it again\n"+
			    "to stop them.\n"+
			    "Context Menu (at an item in the table):\n"+
			    "\"Duplicate\": duplicates a group.\n"+
			    "\"Remove...\": removes a group.")+
			"\n\n"+
			GT.tr(
			    ":: Pattern Pane ::\n"+
			    "Buttons:\n"+
			    "\"Add pattern\": adds a new pattern.\n"+
			    "\"Play\": starts playing of the current group. Push it again to\n"+
			    "stop it. Any patterns whose method is \"start\" or \"alternativestart\"\n"+
			    "will be ignored because those patterns normally invoke an another group.\n"+
			    "Context Menu (at an item in the table):\n"+
			    "\"Move Up\": moves the pattern up.\n"+
			    "\"Move Down\": moves the pattern down.\n"+
			    "\"Duplicate\": duplicates the pattern.\n"+
			    "\"Remove\": removes the pattern.")+
			"\n\n"+
			GT.tr(
			    ":: Surface Pane ::\n"+
			    "When you select a pattern having an offset value (e.g. overlay, move),\n"+
			    "you can move its position by dragging on this pane, or by arrow keys."),
			GT.tr("About this screen..."),
			JOptionPane.INFORMATION_MESSAGE);
		}
	    });
	add(descr);
	sl.putConstraint(
	    SpringLayout.NORTH, descr,
	    3,
	    SpringLayout.NORTH, this);
	sl.putConstraint(
	    SpringLayout.WEST, descr,
	    3,
	    SpringLayout.WEST, this);

	ahc = new AnimationHandleComponent(surface.animation(), surface.image());
	ahc.setFocusable(true);

	// ここでplayerを作る材料が揃ふ。
	player = new Player(
	    editor.grid_frame().shell(),
	    surface.get_int_id(),
	    ahc);
	player.addPlayerListener(
	    new PlayerListener() {
		public void playerBeganAll(PlayerEvent ev) {
		    group_info.playerBeganAll(ev);
		    pattern_info.playerBeganAll(ev);
		}
		public void playerEndedAll(PlayerEvent ev) {
		    group_info.playerEndedAll(ev);
		    pattern_info.playerEndedAll(ev);
		}
		public void playerBeganGroup(PlayerEvent ev) {
		    
		}
		public void playerEndedGroup(PlayerEvent ev) {
		    
		}
		public void playerChangedPattern(PlayerEvent ev) {
		    pattern_info.playerChangedPattern(ev);
		}
	    });

	// descrとlhcとの距離: 最低 3 px, 最大 ∞
	add(ahc);
	sl.putConstraint(
	    SpringLayout.NORTH, ahc,
	    Spring.constant(3, 3, Integer.MAX_VALUE),
	    SpringLayout.SOUTH, descr);
	sl.putConstraint(
	    SpringLayout.WEST, ahc,
	    3,
	    SpringLayout.WEST, this);
	sl.putConstraint(
	    SpringLayout.SOUTH, this,
	    3,
	    SpringLayout.SOUTH, ahc);

	JPanel group_panel = new JPanel();
	{
	    SpringLayout slt = new SpringLayout();
	    group_panel.setLayout(slt);

	    group_info = new GroupInfoPanel();
	    group_panel.add(group_info);
	    slt.putConstraint(
		SpringLayout.NORTH, group_info,
		3,
		SpringLayout.NORTH, group_panel);
	    slt.putConstraint(
		SpringLayout.WEST, group_info,
		3,
		SpringLayout.WEST, group_panel);

	    JPanel buttons_group = new JPanel();
	    {
		SpringLayout splt = new SpringLayout();
		buttons_group.setLayout(splt);

		JButton add = new JButton(GT.tr("Add group..."));
		add.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    add_group();
			}
		    });
		buttons_group.add(add);
		splt.putConstraint(
		    SpringLayout.NORTH, add,
		    0,
		    SpringLayout.NORTH, buttons_group);
		splt.putConstraint(
		    SpringLayout.WEST, add,
		    0,
		    SpringLayout.WEST, buttons_group);
		
		JToggleButton play_all = btn_play_all = new JToggleButton(GT.tr("Play all"));
		play_all.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    if (player.is_playing()) {
				player.stop_all();
			    }
			    else {
				player.play_all();
			    }
			}
		    });
		buttons_group.add(play_all);
		splt.putConstraint(
		    SpringLayout.NORTH, play_all,
		    0,
		    SpringLayout.NORTH, buttons_group);
		splt.putConstraint(
		    SpringLayout.WEST, play_all,
		    2,
		    SpringLayout.EAST, add);

		splt.putConstraint(
		    SpringLayout.SOUTH, buttons_group,
		    0,
		    SpringLayout.SOUTH, add); // どうせaddもplay_allも高さは變はらない
		splt.putConstraint(
		    SpringLayout.EAST, buttons_group,
		    0,
		    SpringLayout.EAST, play_all);
	    }
	    group_panel.add(buttons_group);
	    slt.putConstraint(
		SpringLayout.WEST, buttons_group,
		3,
		SpringLayout.WEST, group_panel);
	    slt.putConstraint(
		SpringLayout.NORTH, buttons_group,
		3,
		SpringLayout.SOUTH, group_info);

	    JScrollPane scr_groups = new JScrollPane(
		JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
		JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	    {
		scr_groups.setPreferredSize(
		    new Dimension(300, 100));
		
		groups_model =
		    new AnimationGroupsTableModel(surface.animation());
		groups = new JTable(groups_model);
		groups.setDragEnabled(false);
		groups.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		groups.getSelectionModel().addListSelectionListener(
		    new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
			    if (group_info.active() != -1) {
				player.stop_group(group_info.active());
			    }
			    
			    int row = groups.getSelectedRow();
			    int id = groups_model.id_of_row(row);
			    group_info.active(id);
			    patterns_model.active(id);

			    update_button_availability();
			}
		    });
		{
		    TableColumn col_id = groups.getColumn(GT.tr("ID"));
		    col_id.setMaxWidth(40);
		}

		final JPopupMenu table_pmenu = new JPopupMenu();
		{
		    final Point context_menu_pt = new Point();

		    final JMenuItem dup = new JMenuItem(GT.tr("Duplicate"));
		    dup.addActionListener(
			new ActionListener() {
			    public void actionPerformed(ActionEvent e) {
				// 希望のグループIDを尋ねる。
				GroupIDChooserDialog chooser = new GroupIDChooserDialog(editor, surface.animation());
				int gid = chooser.choose();
				
				if (gid == -1) {
				    return;
				}

				int row = groups.rowAtPoint(context_menu_pt);
				int src_gid = groups_model.id_of_row(row);
				AnimGroup source = surface.animation().group(src_gid);
				
				AnimGroup group = (AnimGroup)source.clone();
				group.id(gid);
				
				surface.animation().add(group);
				surface.animation().dirty(true);
			    }
			});
		    table_pmenu.add(dup);

		    final JMenuItem rm = new JMenuItem(GT.tr("Remove..."));
		    rm.addActionListener(
			new ActionListener() {
			    public void actionPerformed(ActionEvent e) {
				int retval = JOptionPane.showConfirmDialog(
				    editor,
				    GT.tr("Do you really want to remove this animation group?"),
				    GT.tr("Confirmation"),
				    JOptionPane.YES_NO_OPTION);
				if (retval != JOptionPane.YES_OPTION) {
				    return;
				}

				int row = groups.rowAtPoint(context_menu_pt);
				int gid = groups_model.id_of_row(row);
				AnimGroup group = surface.animation().group(gid);

				if (gid == group_info.active()) {
				    group_info.active(-1);
				}
				
				surface.animation().remove(
				    surface.animation().indexOf(group));
				surface.animation().dirty(true);
			    }
			});
		    table_pmenu.add(rm);

		    groups.addMouseListener(
			new MouseAdapter() {
			    public void mousePressed(MouseEvent e) { popup(e); }
			    public void mouseReleased(MouseEvent e) { popup(e); }
			    private void popup(MouseEvent e) {
				if (e.isPopupTrigger()) {
				    e.consume();

				    context_menu_pt.setLocation(e.getPoint());

				    table_pmenu.show(
					groups, e.getX(), e.getY());
				}
			    }
			});
		}

		scr_groups.setViewportView(groups);
	    }
	    group_panel.add(scr_groups);
	    slt.putConstraint(
		SpringLayout.WEST, scr_groups,
		3,
		SpringLayout.WEST, group_panel);

	    slt.getConstraints(group_panel).setConstraint(
		SpringLayout.EAST,
		Spring.sum(
		    Spring.constant(3),
		    Spring.max(
			slt.getConstraint(SpringLayout.EAST, scr_groups),
			Spring.max(
			    slt.getConstraint(SpringLayout.EAST, group_info),
			    slt.getConstraint(SpringLayout.EAST, buttons_group)))));

	    slt.putConstraint(
		SpringLayout.SOUTH, scr_groups,
		-3,
		SpringLayout.SOUTH, group_panel);
	    slt.putConstraint(
		SpringLayout.NORTH, scr_groups,
		3,
		SpringLayout.SOUTH, buttons_group);

	    group_panel.setMinimumSize(
		new Dimension(
		    (int)Math.max(
			group_info.getMinimumSize().width,
			buttons_group.getMinimumSize().width),
			0));
	}

	JPanel pattern_panel = new JPanel();
	{
	    SpringLayout slt = new SpringLayout();
	    pattern_panel.setLayout(slt);

	    pattern_info = new PatternInfoPanel();
	    pattern_panel.add(pattern_info);
	    slt.putConstraint(
		SpringLayout.NORTH, pattern_info,
		3,
		SpringLayout.NORTH, pattern_panel);
	    slt.putConstraint(
		SpringLayout.WEST, pattern_info,
		3,
		SpringLayout.WEST, pattern_panel);

	    JPanel buttons_pattern = new JPanel();
	    {
		SpringLayout splt = new SpringLayout();
		buttons_pattern.setLayout(splt);
		
		JButton add = btn_add_pattern = new JButton(GT.tr("Add pattern"));
		add.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    add_pattern();
			}
		    });
		buttons_pattern.add(add);
		splt.putConstraint(
		    SpringLayout.NORTH, add,
		    0,
		    SpringLayout.NORTH, buttons_pattern);
		splt.putConstraint(
		    SpringLayout.WEST, add,
		    0,
		    SpringLayout.WEST, buttons_pattern);
	    
		JToggleButton play = btn_play = new JToggleButton(
		    GT.tr(/* Play the animation */ "Play"));
		play.addActionListener(
		    new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    int active_group_id = group_info.active();
			    
			    if (player.is_playing()) {
				player.stop_all();
			    }
			    else {
				player.play_group(active_group_id);
			    }
			}
		    });
		buttons_pattern.add(play);
		splt.putConstraint(
		    SpringLayout.NORTH, play,
		    0,
		    SpringLayout.NORTH, buttons_pattern);
		splt.putConstraint(
		    SpringLayout.WEST, play,
		    2,
		    SpringLayout.EAST, add);

		splt.putConstraint(
		    SpringLayout.SOUTH, buttons_pattern,
		    0,
		    SpringLayout.SOUTH, add); // どうせaddもplayも高さは變はらない
		splt.putConstraint(
		    SpringLayout.EAST, buttons_pattern,
		    0,
		    SpringLayout.EAST, play);
	    }
	    pattern_panel.add(buttons_pattern);
	    slt.putConstraint(
		SpringLayout.WEST, buttons_pattern,
		3,
		SpringLayout.WEST, pattern_panel);
	    slt.putConstraint(
		SpringLayout.NORTH, buttons_pattern,
		3,
		SpringLayout.SOUTH, pattern_info);

	    JScrollPane scr_patterns = new JScrollPane(
		JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
		JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	    {
		scr_patterns.setPreferredSize(
		    new Dimension(300, 100));
		
		patterns_model =
		    new AnimationPatternsTableModel(surface.animation(), -1);
		patterns = new JTable(patterns_model);
		patterns.setDragEnabled(false);
		patterns.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		patterns.getSelectionModel().addListSelectionListener(
		    new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
			    int row = patterns.getSelectedRow();
			    pattern_info.active(row);

			    if (!player.is_playing()) {
				player.shadow_play(group_info.active(), pattern_info.active());
			    }
			}
		    });
		{
		    TableColumn col_id = patterns.getColumn(GT.tr("Method"));
		    col_id.setMaxWidth(100);
		}

		final JPopupMenu table_pmenu = new JPopupMenu();
		{
		    final Point context_menu_pt = new Point();

		    final JMenuItem up = new JMenuItem(GT.tr("Move Up"));
		    up.addActionListener(
			new ActionListener() {
			    public void actionPerformed(ActionEvent e) {
				AnimGroup group = surface.animation().group(group_info.active());
				int pid = patterns.rowAtPoint(context_menu_pt);

				group.swap(pid, pid - 1);
				group.dirty(true);
			    }
			});
		    table_pmenu.add(up);

		    final JMenuItem down = new JMenuItem(GT.tr("Move Down"));
		    down.addActionListener(
			new ActionListener() {
			    public void actionPerformed(ActionEvent e) {
				AnimGroup group = surface.animation().group(group_info.active());
				int pid = patterns.rowAtPoint(context_menu_pt);

				group.swap(pid, pid + 1);
				group.dirty(true);
			    }
			});
		    table_pmenu.add(down);

		    table_pmenu.addSeparator();

		    final JMenuItem dup = new JMenuItem(GT.tr("Duplicate"));
		    dup.addActionListener(
			new ActionListener() {
			    public void actionPerformed(ActionEvent e) {
				AnimGroup group = surface.animation().group(group_info.active());
				int pid = patterns.rowAtPoint(context_menu_pt);

				AnimPattern source = group.get(pid);
				AnimPattern clone = (AnimPattern)source.clone();
				group.insert(pid + 1, clone);
				group.dirty(true);
			    }
			});
		    table_pmenu.add(dup);

		    final JMenuItem rm = new JMenuItem(GT.tr("Remove"));
		    rm.addActionListener(
			new ActionListener() {
			    public void actionPerformed(ActionEvent e) {
				AnimGroup group = surface.animation().group(group_info.active());
				int pid = patterns.rowAtPoint(context_menu_pt);

				group.remove(pid);
				group.dirty(true);
			    }
			});
		    table_pmenu.add(rm);

		    patterns.addMouseListener(
			new MouseAdapter() {
			    public void mousePressed(MouseEvent e) { popup(e); }
			    public void mouseReleased(MouseEvent e) { popup(e); }
			    private void popup(MouseEvent e) {
				if (e.isPopupTrigger()) {
				    e.consume();
				
				    context_menu_pt.setLocation(e.getPoint());
				
				    int sel = patterns.rowAtPoint(e.getPoint());

				    up.setEnabled(sel > 0);
				    down.setEnabled(sel < patterns.getRowCount() - 1);

				    table_pmenu.show(
					patterns, e.getX(), e.getY());
				}
			    }
			});
		}
		
		scr_patterns.setViewportView(patterns);
	    }
	    pattern_panel.add(scr_patterns);
	    slt.putConstraint(
		SpringLayout.WEST, scr_patterns,
		3,
		SpringLayout.WEST, pattern_panel);
	    
	    slt.getConstraints(pattern_panel).setConstraint(
		SpringLayout.EAST,
		Spring.sum(
		    Spring.constant(3),
		    Spring.max(
			slt.getConstraint(SpringLayout.EAST, scr_patterns),
			Spring.max(
			    slt.getConstraint(SpringLayout.EAST, pattern_info),
			    slt.getConstraint(SpringLayout.EAST, buttons_pattern)))));

	    slt.putConstraint(
		SpringLayout.SOUTH, scr_patterns,
		-3,
		SpringLayout.SOUTH, pattern_panel);
	    slt.putConstraint(
		SpringLayout.NORTH, scr_patterns,
		3,
		SpringLayout.SOUTH, buttons_pattern);
	}

	group_panel.setPreferredSize(new Dimension(300, 100));
	JSplitPane group_and_pattern = new JSplitPane(
	    JSplitPane.VERTICAL_SPLIT,
	    group_panel,
	    pattern_panel);
	group_and_pattern.setOneTouchExpandable(true);
	group_and_pattern.setResizeWeight(0.3);
	
	add(group_and_pattern);
	sl.putConstraint(
	    SpringLayout.WEST, group_and_pattern,
	    3,
	    SpringLayout.EAST, ahc);
	sl.putConstraint(
	    SpringLayout.EAST, this,
	    3,
	    SpringLayout.EAST, group_and_pattern);

	sl.putConstraint(
	    SpringLayout.SOUTH, group_and_pattern,
	    -3,
	    SpringLayout.SOUTH, this);
	sl.putConstraint(
	    SpringLayout.NORTH, group_and_pattern,
	    3,
	    SpringLayout.SOUTH, descr);

	update_button_availability();
    }

    public void close() {
	groups_model.close();
	ahc.close();
    }

    public Surface surface() {
	return surface;
    }

    protected void update_button_availability() {
	int row = groups.getSelectedRow();
	
	btn_add_pattern.setEnabled(row >= 0);
	btn_play.setEnabled(row >= 0);
    }

    public void add_group() {
	// 希望のグループIDを尋ねる。
	GroupIDChooserDialog chooser = new GroupIDChooserDialog(editor, surface.animation());
	int gid = chooser.choose();
	
	if (gid == -1) {
	    return;
	}

	// interval: sometimesでグループ追加
	AnimGroup group = new AnimGroup(gid);
	group.interval("sometimes");
	
	surface.animation().add(group);
	surface.animation().dirty(true);
    }

    public void add_pattern() {
	// 最後のパターン、もしくは現在撰擇中パターンの
	// 複製を追加。無ければoverlayを追加。
	AnimGroup group = surface.animation().group(group_info.active());
	AnimPattern pattern;
	
	int pid = patterns.getSelectedRow();
	if (pid == -1) {
	    pid = group.size() - 1;
	}
	
	if (group.size() == 0) {
	    pattern = new AnimPattern();
	    pattern.method("overlay");
	    pattern.surface_id(-1);
	}
	else {
	    pattern = (AnimPattern)group.get(pid).clone();
	}

	group.insert(pid + 1, pattern);
    }

    private class GroupInfoPanel
	extends JPanel implements ActionListener {
	
	int group_id = -1;

	CardLayout crd_empty_or_not;
	JComboBox cmb_interval;
	JCheckBox chk_exclusive;

	JPanel pnl_types;
	CardLayout crd_types;
	Map arg_fields; // type => JTextField

	public GroupInfoPanel() {
	    arg_fields = new Hashtable();
	    
	    crd_empty_or_not = new CardLayout();
	    setLayout(crd_empty_or_not);
	    
	    add(new JPanel(), "Empty"); // 空のカード

	    JPanel panel = new JPanel();
	    {
		SpringLayout sl = new SpringLayout();
		panel.setLayout(sl);

		// interval
		cmb_interval = new JComboBox();
		cmb_interval.addActionListener(this);
		panel.add(cmb_interval);
		sl.putConstraint(
		    SpringLayout.NORTH, cmb_interval,
		    0,
		    SpringLayout.NORTH, panel);
		sl.putConstraint(
		    SpringLayout.WEST, cmb_interval,
		    0,
		    SpringLayout.WEST, panel);

		// option exclusive
		chk_exclusive = new JCheckBox("exclusive");
		chk_exclusive.addActionListener(this);
		panel.add(chk_exclusive);
		sl.putConstraint(
		    SpringLayout.NORTH, chk_exclusive,
		    0,
		    SpringLayout.NORTH, panel);
		sl.putConstraint(
		    SpringLayout.WEST, chk_exclusive,
		    2,
		    SpringLayout.EAST, cmb_interval);

		// types
		pnl_types = new JPanel();
		{
		    crd_types = new CardLayout();
		    pnl_types.setLayout(crd_types);
		}
		panel.add(pnl_types);
		sl.getConstraints(pnl_types).setConstraint(
		    SpringLayout.NORTH,
		    Spring.sum(
			Spring.constant(2),
			Spring.max(
			    sl.getConstraint(SpringLayout.SOUTH, cmb_interval),
			    sl.getConstraint(SpringLayout.SOUTH, chk_exclusive))));
		sl.putConstraint(
		    SpringLayout.WEST, pnl_types,
		    0,
		    SpringLayout.WEST, panel);

		// 枠
		sl.getConstraints(panel).setConstraint(
		    SpringLayout.EAST,
		    Spring.max(
			sl.getConstraint(SpringLayout.EAST, chk_exclusive),
			sl.getConstraint(SpringLayout.EAST, pnl_types)));
		sl.putConstraint(
		    SpringLayout.SOUTH, panel,
		    0,
		    SpringLayout.SOUTH, pnl_types);
	    }
	    add(panel, "Not Empty");

	    def_interval(
		"sometimes", GT.tr(
		    /* "prob." = probability, "sec" = second (1/60 minute) */
		    "Prob. 1/2 per sec"));
	    def_interval(
		"rarely", GT.tr("Prob. 1/4 per sec"));
	    def_interval(
		"random", GT.tr("Prob. 1/{0} per sec"), "{0}");
	    def_interval(
		"always", GT.tr("Loops infinitely"));
	    def_interval(
		"runonce", GT.tr("Once after the surface showed up"));
	    def_interval(
		"yen-e", GT.tr("When a script calls \\e"));
	    def_interval(
		"talk", GT.tr("After a script shows {0} letters"), "{0}");
	    def_interval(
		"never", GT.tr("No automatic invocation"));
	    def_interval(
		"bind", GT.tr("Dress changing"));

	    update_outer_cards();
	}

	public int active(int id) {
	    group_id = id;
	    update_outer_cards();
	    return id;
	}
	public int active() {
	    return group_id;
	}

	public void playerBeganAll(PlayerEvent ev) {
	    btn_play_all.setSelected(true);
	}

	public void playerEndedAll(PlayerEvent ev) {
	    btn_play_all.setSelected(false);
	}

	public void actionPerformed(ActionEvent ev) {
	    if (group_id == -1) {
		return;
	    }
	    
	    AnimGroup group = surface.animation().group(group_id);
	    
	    if (ev.getSource() == cmb_interval) {
		String selected = (String)cmb_interval.getSelectedItem();
		
		if (group.interval() != selected) {
		    group.interval(selected);
		    group.dirty(true);
		    surface.animation().changed(group);
		    
		    update_inner_cards();
		}
	    }
	    else if (ev.getSource() == chk_exclusive) {
		boolean selected = chk_exclusive.isSelected();

		if ((selected &&
		     (group.option() == null || !group.option().equals("exclusive"))) ||
		    (!selected &&
		     (group.option() != null && group.option().equals("exclusive")))) {

		    group.option(selected ? "exclusive" : null);
		    group.dirty(true);
		    surface.animation().changed(group);
		}
	    }
	}

	private void update_outer_cards() {
	    if (group_id == -1) {
		// パネル全體を非表示
		crd_empty_or_not.show(this, "Empty");
	    }
	    else {
		// cmb_intervalとchk_exclusiveを更新
		AnimGroup group = surface.animation().group(group_id);
		cmb_interval.setSelectedItem(group.interval());
		chk_exclusive.setSelected(
		    group.option() != null && group.option().equals("exclusive"));
		
		update_inner_cards();
		crd_empty_or_not.show(this, "Not Empty");
	    }
	}

	private void update_inner_cards() {
	    AnimGroup group = surface.animation().group(group_id);
	    String type = (String)cmb_interval.getSelectedItem();
	    JTextField field = (JTextField)arg_fields.get(type);

	    if (field != null) {
		field.setText(Integer.toString(group.interval_arg()));
	    }

	    crd_types.show(pnl_types, type);
	}

	private void def_interval(String type, String label_str) {
	    cmb_interval.addItem(type);

	    JPanel panel = new JPanel();
	    {
		SpringLayout sl = new SpringLayout();
		panel.setLayout(sl);

		JLabel label = new JLabel(label_str);
		panel.add(label);
		sl.putConstraint(
		    SpringLayout.WEST, label,
		    0,
		    SpringLayout.WEST, panel);
		sl.putConstraint(
		    SpringLayout.NORTH, label,
		    3,
		    SpringLayout.NORTH, panel);

		sl.putConstraint(
		    SpringLayout.EAST, panel,
		    0,
		    SpringLayout.EAST, label);
		sl.putConstraint(
		    SpringLayout.SOUTH, panel,
		    3,
		    SpringLayout.SOUTH, label);
	    }
	    pnl_types.add(panel, type);
	}

	private void def_interval(String type, String label_str, String replace) {
	    cmb_interval.addItem(type);

	    String[] labels = label_str.split("\\Q"+replace+"\\E");

	    JPanel panel = new JPanel();
	    {
		SpringLayout sl = new SpringLayout();
		panel.setLayout(sl);

		JLabel label = new JLabel(labels[0]);
		panel.add(label);
		sl.putConstraint(
		    SpringLayout.WEST, label,
		    0,
		    SpringLayout.WEST, panel);
		sl.putConstraint(
		    SpringLayout.NORTH, label,
		    3,
		    SpringLayout.NORTH, panel);

		final JTextField field = new JTextField(2);
		field.setMaximumSize(field.getPreferredSize());
		field.setHorizontalAlignment(JTextField.TRAILING);
		field.getDocument().addDocumentListener(
		    new DocumentListener() {
			public void changedUpdate(DocumentEvent ev) { changed(); }
			public void insertUpdate(DocumentEvent ev) { changed(); }
			public void removeUpdate(DocumentEvent ev) { changed(); }
			public void changed() {
			    AnimGroup group = surface.animation().group(group_id);
			    try {
				int arg = Integer.parseInt(field.getText());
				if (arg <= 0) {
				    throw new Exception();
				}
				group.interval_arg(arg);
				group.dirty(true);
				surface.animation().changed(group);
			    }
			    catch (Exception e) {}
			}
		    });
		arg_fields.put(type, field);
		panel.add(field);
		sl.putConstraint(
		    SpringLayout.WEST, field,
		    2,
		    SpringLayout.EAST, label);
		sl.putConstraint(
		    SpringLayout.NORTH, field,
		    0,
		    SpringLayout.NORTH, panel);

		JLabel label2 = new JLabel(labels[1]);
		panel.add(label2);
		sl.putConstraint(
		    SpringLayout.WEST, label2,
		    2,
		    SpringLayout.EAST, field);
		sl.putConstraint(
		    SpringLayout.NORTH, label2,
		    3,
		    SpringLayout.NORTH, panel);

		sl.putConstraint(
		    SpringLayout.EAST, panel,
		    0,
		    SpringLayout.EAST, label2);
		sl.getConstraints(panel).setConstraint(
		    SpringLayout.SOUTH,
		    Spring.max(
			sl.getConstraint(SpringLayout.SOUTH, label),
			Spring.max(
			    sl.getConstraint(SpringLayout.SOUTH, label2),
			    sl.getConstraint(SpringLayout.SOUTH, field))));
	    }
	    pnl_types.add(panel, type);
	}
    }

    private class PatternInfoPanel
	extends JPanel
	implements ActionListener, PatternOffsetListener {

	public static final int HAVE_SURFACE_ID = 1 << 0;
	public static final int HAVE_INTERVAL   = 1 << 1;
	public static final int HAVE_OFFSETS    = 1 << 2;
	public static final int HAVE_START_ID   = 1 << 3;
	public static final int HAVE_ALTERNATIVE_IDS = 1 << 4;
	public static final int HAVE_INSERT_ID  = 1 << 5;

	int pat_id = -1;

	CardLayout crd_empty_or_not;
	JComboBox cmb_method;

	JPanel pnl_types;
	CardLayout crd_types;
	Map arg_components; // type => {method => JTextField}

	public PatternInfoPanel() {
	    arg_components = new Hashtable();
	    
	    crd_empty_or_not = new CardLayout();
	    setLayout(crd_empty_or_not);

	    add(new JPanel(), "Empty"); // 空のカード

	    JPanel panel = new JPanel();
	    {
		SpringLayout sl = new SpringLayout();
		panel.setLayout(sl);

		// method
		cmb_method = new JComboBox();
		cmb_method.addActionListener(this);
		panel.add(cmb_method);
		sl.putConstraint(
		    SpringLayout.NORTH, cmb_method,
		    0,
		    SpringLayout.NORTH, panel);
		sl.putConstraint(
		    SpringLayout.WEST, cmb_method,
		    0,
		    SpringLayout.WEST, panel);

		// types
		pnl_types = new JPanel();
		{
		    crd_types = new CardLayout();
		    pnl_types.setLayout(crd_types);
		}
		panel.add(pnl_types);
		sl.putConstraint(
		    SpringLayout.NORTH, pnl_types,
		    2,
		    SpringLayout.SOUTH, cmb_method);
		sl.putConstraint(
		    SpringLayout.WEST, pnl_types,
		    0,
		    SpringLayout.WEST, panel);

		// 枠
		sl.getConstraints(panel).setConstraint(
		    SpringLayout.EAST,
		    Spring.max(
			sl.getConstraint(SpringLayout.EAST, cmb_method),
			sl.getConstraint(SpringLayout.EAST, pnl_types)));
		sl.putConstraint(
		    SpringLayout.SOUTH, panel,
		    0,
		    SpringLayout.SOUTH, pnl_types);
	    }
	    add(panel, "Not Empty");

	    def_method(
		"overlay", HAVE_SURFACE_ID | HAVE_INTERVAL | HAVE_OFFSETS);
	    def_method(
		"overlayfast", HAVE_SURFACE_ID | HAVE_INTERVAL | HAVE_OFFSETS);
	    def_method(
		"base", HAVE_SURFACE_ID | HAVE_INTERVAL);
	    def_method(
		"move", HAVE_INTERVAL | HAVE_OFFSETS);
	    def_method(
		"reduce", HAVE_SURFACE_ID | HAVE_INTERVAL | HAVE_OFFSETS);
	    def_method(
		"start", HAVE_START_ID);
	    def_method(
		"alternativestart", HAVE_ALTERNATIVE_IDS);
	    def_method(
		"bind", HAVE_SURFACE_ID | HAVE_OFFSETS);
	    def_method(
		"add", HAVE_SURFACE_ID | HAVE_OFFSETS);
	    def_method(
		"insert", HAVE_INSERT_ID);

	    update_outer_cards();

	    ahc.addPatternOffsetListener(this);
	}

	public final int active(int id) {
	    pat_id = id;
	    
	    if (!player.is_playing()) {
		update_outer_cards();
	    }
	    
	    return id;
	}
	public final int active() {
	    return pat_id;
	}

	public void playerBeganAll(PlayerEvent ev) {
	    ahc.active(-1);
	    btn_play.setSelected(true);
	}

	public void playerEndedAll(PlayerEvent ev) {
	    player.shadow_play(group_info.active(), this.active());
	    btn_play.setSelected(false);
	    update_outer_cards();
	}

	public void playerChangedPattern(PlayerEvent ev) {
	    if (ev.group_id() == group_info.active()) {
		patterns.changeSelection(ev.pattern_id(), 0, false, false);
	    }
	}

	public void offsetChanged(PatternOffsetEvent ev) {
	    if (pat_id == -1) {
		return;
	    }
	    AnimGroup group = surface.animation().group(group_info.active());
	    AnimPattern pattern = group.get(pat_id);

	    pattern.offset_x(ev.new_offset_x());
	    pattern.offset_y(ev.new_offset_y());
	    pattern.dirty(true);
	    
	    group.changed(pattern);
	}

	public void actionPerformed(ActionEvent ev) {
	    if (pat_id == -1) {
		return;
	    }

	    AnimGroup group = surface.animation().group(group_info.active());
	    AnimPattern pattern = group.get(pat_id);

	    if (ev.getSource() == cmb_method) {
		String selected = (String)cmb_method.getSelectedItem();

		if (pattern.method() != selected) {
		    pattern.method(selected);
		    pattern.dirty(true);
		    group.changed(pattern);

		    update_inner_cards();

		    if (!player.is_playing()) {
			player.shadow_play(group_info.active(), pat_id);
		    }
		}
	    }
	}

	private void update_outer_cards() {
	    if (pat_id == -1) {
		// パネル全體を非表示
		crd_empty_or_not.show(this, "Empty");
	    }
	    else {
		// cmb_methodを更新
		AnimGroup group = surface.animation().group(group_info.active());
		AnimPattern pattern = group.get(pat_id);
		cmb_method.setSelectedItem(pattern.method());

		update_inner_cards();
		crd_empty_or_not.show(this, "Not Empty");
	    }
	}

	private void update_inner_cards() {
	    String type = (String)cmb_method.getSelectedItem();
	    Map components = (Map)arg_components.get(type);
	    
	    AnimGroup group = surface.animation().group(group_info.active());
	    AnimPattern pattern = group.get(pat_id);
	    
	    JTextField sid = (JTextField)components.get("surface_id");
	    if (sid != null) {
		sid.setText(Integer.toString(pattern.surface_id()));
	    }
	    
	    JTextField intv = (JTextField)components.get("interval");
	    if (intv != null) {
		intv.setText(Integer.toString(pattern.interval()));
	    }
	    
	    JTextField start = (JTextField)components.get("start_id");
	    if (start != null) {
		start.setText(Integer.toString(pattern.start_id()));
	    }
	    
	    JTextField alt = (JTextField)components.get("alternative_ids");
	    if (alt != null) {
		alt.setText(Main.join(",", pattern.alternative_ids()));
	    }
	    
	    JTextField ins = (JTextField)components.get("insert_id");
	    if (ins != null) {
		ins.setText(Integer.toString(pattern.insert_id()));
	    }
	    
	    crd_types.show(pnl_types, type);
	}

	private void def_method(final String method, final int definition) {
	    cmb_method.addItem(method);
	    cmb_method.setPreferredSize(cmb_method.getMinimumSize());
	    cmb_method.setMaximumSize(cmb_method.getMinimumSize());

	    Map components = new Hashtable(); // name => JComponent
	    arg_components.put(method, components);
	    
	    Box items = Box.createHorizontalBox();
	    {
		if ((definition & HAVE_SURFACE_ID) > 0) {
		    Box sid_box = Box.createHorizontalBox();
		    items.add(sid_box);
		    
		    JLabel label = new JLabel(GT.tr("Surface ID:"));
		    sid_box.add(label);
		    sid_box.add(Box.createHorizontalStrut(3));

		    final JTextField field = new JTextField("100000");
		    field.setMaximumSize(field.getPreferredSize());
		    field.setHorizontalAlignment(JTextField.TRAILING);
		    field.getDocument().addDocumentListener(
			new DocumentListener() {
			    public void changedUpdate(DocumentEvent ev) { changed(); }
			    public void insertUpdate(DocumentEvent ev) { changed(); }
			    public void removeUpdate(DocumentEvent ev) { changed(); }
			    public void changed() {
				// sufrace idとして許される値は、
				// -1 (terminate this), -2 (termiante all), 0以上
				// 但し存在しないサーフィスは認めない。
				AnimGroup group = surface.animation().group(group_info.active());
				AnimPattern pattern = group.get(pat_id);
				try {
				    int arg = Integer.parseInt(field.getText());
				    if (arg < -2 ||
					(arg >= 0 && !editor.grid_frame().shell().have_surface(arg))) {
					
					throw new Exception();
				    }
				    pattern.surface_id(arg);
				    pattern.dirty(true);
				    group.changed(pattern);

				    if (!player.is_playing()) {
					player.shadow_play(group_info.active(), pattern_info.active());
				    }
				}
				catch (Exception e) {}
			    }
			});
		    components.put("surface_id", field);
		    sid_box.add(field);
		}

		if ((definition & HAVE_INTERVAL) > 0) {
		    if (items.getComponentCount() > 0) {
			items.add(Box.createHorizontalStrut(5));
		    }
		    
		    Box intv_box = Box.createHorizontalBox();
		    items.add(intv_box);

		    JLabel label = new JLabel(
			GT.tr(/* ms = millisecond */ "Interval (ms):"));
		    intv_box.add(label);
		    intv_box.add(Box.createHorizontalStrut(3));

		    final JTextField field = new JTextField("10000");
		    field.setMaximumSize(field.getPreferredSize());
		    field.setHorizontalAlignment(JTextField.TRAILING);
		    field.getDocument().addDocumentListener(
			new DocumentListener() {
			    public void changedUpdate(DocumentEvent ev) { changed(); }
			    public void insertUpdate(DocumentEvent ev) { changed(); }
			    public void removeUpdate(DocumentEvent ev) { changed(); }
			    public void changed() {
				// intervalとして許される値は負でない整數
				AnimGroup group = surface.animation().group(group_info.active());
				AnimPattern pattern = group.get(pat_id);
				try {
				    int arg = Integer.parseInt(field.getText());
				    if (arg < 0) {
					throw new Exception();
				    }
				    pattern.interval(arg);
				    pattern.dirty(true);
				    group.changed(pattern);
				}
				catch (Exception e) {}
			    }
			});
		    components.put("interval", field);
		    intv_box.add(field);
		}

		if ((definition & HAVE_START_ID) > 0) {
		    if (items.getComponentCount() > 0) {
			items.add(Box.createHorizontalStrut(5));
		    }
		    
		    Box start_box = Box.createHorizontalBox();
		    items.add(start_box);

		    JLabel label = new JLabel(GT.tr("Group ID:"));
		    start_box.add(label);
		    start_box.add(Box.createHorizontalStrut(3));

		    final JTextField field = new JTextField("1000");
		    field.setMaximumSize(field.getPreferredSize());
		    field.setHorizontalAlignment(JTextField.TRAILING);
		    field.getDocument().addDocumentListener(
			new DocumentListener() {
			    public void changedUpdate(DocumentEvent ev) { changed(); }
			    public void insertUpdate(DocumentEvent ev) { changed(); }
			    public void removeUpdate(DocumentEvent ev) { changed(); }
			    public void changed() {
				// 存在しないグループIDは認めない。
				AnimGroup group = surface.animation().group(group_info.active());
				AnimPattern pattern = group.get(pat_id);
				try {
				    int gid = Integer.parseInt(field.getText());
				    if (surface.animation().group(gid) == null) {
					throw new Exception();
				    }
				    pattern.start_id(gid);
				    pattern.dirty(true);
				    group.changed(pattern);
				}
				catch (Exception e) {}
			    }
			});
		    components.put("start_id", field);
		    start_box.add(field);
		}

		if ((definition & HAVE_ALTERNATIVE_IDS) > 0) {
		    if (items.getComponentCount() > 0) {
			items.add(Box.createHorizontalStrut(5));
		    }

		    Box alt_box = Box.createHorizontalBox();
		    items.add(alt_box);

		    JLabel label = new JLabel(GT.tr("Group ID (comma-separated):"));
		    alt_box.add(label);
		    alt_box.add(Box.createHorizontalStrut(3));

		    final JTextField field = new JTextField("1,2,3,4,5,6,7");
		    field.setHorizontalAlignment(JTextField.TRAILING);
		    field.getDocument().addDocumentListener(
			new DocumentListener() {
			    public void changedUpdate(DocumentEvent ev) { changed(); }
			    public void insertUpdate(DocumentEvent ev) { changed(); }
			    public void removeUpdate(DocumentEvent ev) { changed(); }
			    public void changed() {
				// 負でない整數をカンマで區切ったものであれば良い。
				// 但し存在しないグループIDの存在は認めない。
				AnimGroup group = surface.animation().group(group_info.active());
				AnimPattern pattern = group.get(pat_id);
				try {
				    Vector ids = new Vector();
				    String[] parts = field.getText().split(",");
				    for (int i = 0; i < parts.length; i++) {
					int gid = Integer.parseInt(parts[i]);
					if (surface.animation().group(gid) == null) {
					    // 存在しない
					    throw new Exception();
					}
					ids.add(new Integer(gid));
				    }
				    pattern.alternative_ids(ids);
				    pattern.dirty(true);
				    group.changed(pattern);
				}
				catch (Exception e) {}
			    }
			});
		    components.put("alternative_ids", field);
		    alt_box.add(field);
		}

		if ((definition & HAVE_INSERT_ID) > 0) {
		    if (items.getComponentCount() > 0) {
			items.add(Box.createHorizontalStrut(5));
		    }

		    Box ins_box = Box.createHorizontalBox();
		    items.add(ins_box);

		    JLabel label = new JLabel(GT.tr("Group ID:"));
		    ins_box.add(label);
		    ins_box.add(Box.createHorizontalStrut(3));

		    final JTextField field = new JTextField("1000");
		    field.setMaximumSize(field.getPreferredSize());
		    field.setHorizontalAlignment(JTextField.TRAILING);
		    field.getDocument().addDocumentListener(
			new DocumentListener() {
			    public void changedUpdate(DocumentEvent ev) { changed(); }
			    public void insertUpdate(DocumentEvent ev) { changed(); }
			    public void removeUpdate(DocumentEvent ev) { changed(); }
			    public void changed() {
				// 存在しないグループIDは認めない。
				AnimGroup group = surface.animation().group(group_info.active());
				AnimPattern pattern = group.get(pat_id);
				try {
				    int gid = Integer.parseInt(field.getText());
				    if (surface.animation().group(gid) == null) {
					throw new Exception();
				    }
				    pattern.insert_id(gid);
				    pattern.dirty(true);
				    group.changed(pattern);
				}
				catch (Exception e) {}
			    }
			});
		    components.put("insert_id", field);
		    ins_box.add(field);
		}
	    }

	    items.setMinimumSize(items.getPreferredSize());
	    pnl_types.add(items, method);
	}
    }
}
