package koropeta.gui.surfaceeditor.animation;
import koropeta.core.*;
import koropeta.core.animation.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.text.*;

public class AnimationPatternsTableModel implements TableModel {
    Animation animation;
    int group_id;
    Vector listeners;

    ListDataListener group_change_listener;

    /*
      +-----------+---------------------------+
      | Method    | Parameter                 |
      +===========+===========================+
      | overlay   | \s[1000], 200ms, (10, 30) |
      | move      | 200ms, (10, 50)           |
      +-----------+---------------------------+
    */

    public AnimationPatternsTableModel(Animation anim, int gid) {
        animation = anim;
        group_id = gid;
        listeners = new Vector();

        group_change_listener = new ListDataListener() {
                public void intervalAdded(ListDataEvent e) {
                    changed(TableModelEvent.INSERT, e);
                }

                public void intervalRemoved(ListDataEvent e) {
                    changed(TableModelEvent.DELETE, e);
                }

                public void contentsChanged(ListDataEvent e) {
                    changed(TableModelEvent.UPDATE, e);
                }

                protected void changed(int type, ListDataEvent e) {
                    TableModelEvent ev = new TableModelEvent(
                        AnimationPatternsTableModel.this,
                        e.getIndex0(),
                        e.getIndex1(),
                        TableModelEvent.ALL_COLUMNS,
                        type);

                    for (Iterator ite = listeners.iterator(); ite.hasNext();) {
                        TableModelListener l = (TableModelListener)ite.next();
                        l.tableChanged(ev);
                    }
                }
            };
        // リスナーのaddはactive變更時に行ふ。
    }

    public void close() {
        if (group_id != -1) {
            AnimGroup group = animation.group(group_id);
            if (group != null) {
                group.removeListDataListener(group_change_listener);
            }
        }
    }

    public int active(int gid) {
        if (gid != group_id) {
            if (group_id != -1) {
                AnimGroup group = animation.group(group_id);
                if (group != null) {
                    group.removeListDataListener(group_change_listener);
                }
            }
	    
            group_id = gid;

            if (group_id != -1) {
                AnimGroup group = animation.group(group_id);
                group.addListDataListener(group_change_listener);
            }
	    
            // テーブル全體が變化した。
            TableModelEvent ev = new TableModelEvent(this);
            for (Iterator ite = listeners.iterator(); ite.hasNext(); ) {
                TableModelListener l = (TableModelListener)ite.next();
                l.tableChanged(ev);
            }
        }
        return gid;
    }
    public int active() {
        return group_id;
    }

    public void addTableModelListener(TableModelListener l) {
        listeners.add(l);
    }

    public Class getColumnClass(int idx) {
        switch (idx) {
        case 0:
            return String.class;

        case 1:
            return String.class;

        default:
            throw new RuntimeException("Invalid column index: "+idx);
        }
    }

    public int getColumnCount() {
        return 2;
    }

    public String getColumnName(int idx) {
        switch (idx) {
        case 0:
            return GT.tr("Method");

        case 1:
            return GT.tr("Parameters");

        default:
            throw new RuntimeException("Invalid column index: "+idx);
        }
    }

    public final int getRowCount() {
        if (group_id == -1) {
            return 0;
        }
        else {
            AnimGroup group = animation.group(group_id);
            return group.size();
        }
    }

    public Object getValueAt(final int row, final int col) {
        AnimGroup group = animation.group(group_id);
        AnimPattern pattern = group.get(row);
	
        switch (col) {
        case 0:
            return pattern.method();

        case 1:
            String method = pattern.method();
            if (method.equals("overlay") ||
                method.equals("overlayfast") ||
                method.equals("reduce")) {

                return GT.tr(
                    /* ms: millisecond */
                    "\\s[{0}], {1}ms, ({2}, {3})",
                    new Object[] {
                        Integer.toString(pattern.surface_id()),
                        Integer.toString(pattern.interval()),
                        Integer.toString(pattern.offset_x()),
                        Integer.toString(pattern.offset_y())});
            }
            else if (method.equals("base")) {
                return GT.tr(
                    /* ms: millisecond */
                    "\\s[{0}], {1}ms",
                    new Object[] {
                        Integer.toString(pattern.surface_id()),
                        Integer.toString(pattern.interval())});
            }
            else if (method.equals("move")) {
                return GT.tr(
                    /* ms: millisecond */
                    "{0}ms, ({1}, {2})",
                    new Object[] {
                        Integer.toString(pattern.interval()),
                        Integer.toString(pattern.offset_x()),
                        Integer.toString(pattern.offset_y())});
            }
            else if (method.equals("start")) {
                return "\\i[" + pattern.start_id() + "]";
            }
            else if (method.equals("alternativestart")) {
                List ids = pattern.alternative_ids();
                if (ids != null && ids.size() > 0) {
                    StringBuffer buf = new StringBuffer();
                    for (Iterator ite = ids.iterator(); ite.hasNext(); ) {
                        if (buf.length() > 0) {
                            buf.append(", ");
                        }
                        buf.append("\\i[");
                        buf.append(((Integer)ite.next()).intValue());
                        buf.append(']');
                    }
                    return buf.toString();
                }
                else {
                    return GT.tr(
                        /* This pattern has no destinations */
                        "(No Destinations)");
                }
            }
            else if (method.equals("bind") ||
                     method.equals("add")) {
                return GT.tr(
                    "\\s[{0}], ({1}, {2})",
                    new Object[] {
                        Integer.toString(pattern.surface_id()),
                        Integer.toString(pattern.offset_x()),
                        Integer.toString(pattern.offset_y())});
            }
            else if (method.equals("insert")) {
                return Integer.toString(pattern.insert_id());
            }
            else {
                throw new RuntimeException(
                    "Invalid method: "+method);
            }

        default:
            throw new RuntimeException("Invalid column index: "+col);
        }
    }

    public boolean isCellEditable(int row, int col) {
        return false;
    }

    public void removeTableModelListener(TableModelListener l) {
        listeners.remove(l);
    }

    public void setValueAt(Object val, int row, int col) {
        throw new RuntimeException("Not editable column: "+col);
    }
}
